/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */ 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define PLAT_GTK 1
//#define DEBUG

#include <libgen.h>
#include <stdio.h>
#include <gtk/gtk.h>


#include "cssedwindow.h"
#include "document.h"
#include "callbacks.h"
#include "selectorparser.h"
#include "cssdialogs-interface.h"
#include "interface.h"
#include "support.h"
#include "utils.h"

#include <Scintilla.h>
#include <SciLexer.h>
#include <ScintillaWidget.h>

void
on_notebook_docs_switch_page (GtkNotebook * notebook, GtkNotebookPage * page,
			      guint page_num, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	GtkWidget *text_output;
	GtkTextBuffer *buffer;
	Tcssed_dyn_menus *menus;
	GtkWidget *window_widget;

	window = CSSED_WINDOW (user_data);
	menus = window->dynamic_menus;

#ifdef DEBUG
	g_print ("callbacks.c: on_notebook_docs_switch_page() - Page switched to %d\n\tnotebook\t%x\n\tpage\t\t%x\n\n", page_num, notebook, page);
#endif

	tmpdoc = document_get_from_notebook_page_num (window, page_num);
	if (tmpdoc == NULL)
	{
		cssed_error_message
			("Programming error !!",
			 "An invalid document have been passed,\nthings are going to fail from this point.\nPlease tell that this happened to the developers.");
		return;
	}
	else
	{
		gtk_widget_grab_focus (GTK_WIDGET (tmpdoc->editor));
		gtk_widget_queue_resize (GTK_WIDGET (tmpdoc->editor));
#ifdef DEBUG
		if (tmpdoc->filename != NULL)
			g_print ("callbacks.c: on_notebook_docs_switch_page() - Doc : \n\t\tfilename %s\n\n", tmpdoc->filename);
		else
			g_print ("callbacks.c: on_notebook_docs_switch_page() - Doc : \n\t\tfilename NULL\n\n");
#endif
		GtkWidget *window_widget =
			lookup_widget (window->notebook_docs, "window");
		gchar* strout;
		strout = g_strdup_printf("cssed: %s",tmpdoc->filename ==
				      NULL ? "Untitled doc" : tmpdoc->filename );
		
		gtk_window_set_title (GTK_WINDOW (window_widget),strout);
		g_free(strout);
	}
	
}
void
on_menu_doc_info_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	GtkListStore *store;
	GtkTreeIter iter;
	gint len;
	gchar* lenmsg;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	store = window->output_store;

	gtk_list_store_clear (store);
	gtk_list_store_insert (store, &iter, 0);

	if (tmpdoc->filename == NULL)
	{
		gtk_list_store_set (store, &iter, 0, "Untitled doc", -1);
	}
	else
	{
		gtk_list_store_set (store, &iter, 0, tmpdoc->filename, -1);
	}

	gtk_list_store_insert (store, &iter, 1);

	if (tmpdoc->modified)
	{
		gtk_list_store_set (store, &iter, 0, "Document modified (not saved)", -1);
	}
	else
	{
		gtk_list_store_set (store, &iter, 0, "Document not modified (saved)",
				    -1);
	}
	
	gtk_list_store_insert (store, &iter, 2);
	len = document_get_length(tmpdoc);
	lenmsg = g_strdup_printf("Document length: %d",len); 
	gtk_list_store_set (store, &iter, 0, lenmsg,   -1);
	g_free(lenmsg);
	
	gtk_list_store_insert (store, &iter, 3);
	
	if( tmpdoc->autocompletion ){
		gtk_list_store_set (store, &iter, 0, "Autocompletion activated for this document",   -1);
	}else{
		gtk_list_store_set (store, &iter, 0, "Autocompletion disabled for this document",   -1);
	}	
}

void
on_menu_nuevo_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkNotebook *notebook;
	GtkWidget *editor;
	GtkWidget *label;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);
	notebook = GTK_NOTEBOOK (window->notebook_docs);

	if (!GTK_IS_NOTEBOOK (notebook))
	{
		cssed_error_message
			("Programming error :-(","Please see standard output and tell\nwhat happened to the developers !!!");
#ifdef DEBUG
		g_print ("callbacks.c:  on_menu_nuevo_activate()\n\n%x Is NOT a GtkNotebook\n", notebook);
#endif
		return;
	}
	create_and_attach_new_doc (window, "Untitled");
}
void
on_menu_abrir_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);
	GtkWidget *opensel = create_open_fileselection (window);
	if (opensel != NULL)
	{
		gtk_widget_show (opensel);
	}
}
void
on_menu_guardar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	GtkWidget *file_selector;
	gchar *doc_buffer;
	gint doclen;
	size_t nwriten;
	FILE *f;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc->filename == NULL)
	{
		file_selector = create_save_fileselection (window);
		gtk_widget_show (file_selector);
	}
	else
	{
#ifdef DEBUG
		g_print ("Trying to save a document called %s\n",
			 tmpdoc->filename);
#endif
		f = fopen (tmpdoc->filename, "w");
		if (f == NULL)
		{
			gchar *msg;
			msg = g_strdup_printf
				("I'm unable to open the file\n%s\nfor writting",
				 tmpdoc->filename);
			cssed_error_message ( "Can't open file",msg );
			g_free (msg);
		}
		else
		{
			doclen = document_get_length (tmpdoc);
			doc_buffer = g_malloc (++doclen);
			document_get_text (tmpdoc, doclen, doc_buffer);

			nwriten =
				fwrite (doc_buffer, sizeof (gchar), doclen,
					f);
			if (nwriten != doclen)
			{
				cssed_error_message
					("Error while writting to file",
					 "There is a mismatch between the file size\nand the bytes written.\nThe file can be damaged.");
			}
			document_set_modified (tmpdoc, FALSE);
			g_free (doc_buffer);
			fclose (f);
		}
	}
}
void
on_save_fileselection_ok_button_clicked (GtkButton * button,
					 gpointer user_data)
{
	Tcssed_doc *savedoc;
	GtkWidget *notebook;
	GtkWidget *file_selector;
	Tcssed_fileselector_data *data;
	Tcssed_window *window;	// main window object
	gchar *doc_buffer;
	gint doclen;
	size_t nwriten;
	const gchar *selected_filename;
	FILE *f;

	data = (Tcssed_fileselector_data *) user_data;
	window = (Tcssed_window *) data->main_window;
	notebook = GTK_WIDGET (data->notebook);
	file_selector = GTK_WIDGET (data->fileselector);
	savedoc = document_get_current (window);

	selected_filename =
		gtk_file_selection_get_filename (GTK_FILE_SELECTION
						 (file_selector));
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));	// destroy the selector

	f = fopen (selected_filename, "w");
	if (f == NULL)
	{
		gchar *msg;
		msg = g_strdup_printf ("The file %s\ncan not be opened for writting.",
				       selected_filename);
		cssed_error_message ("Can't open file", msg);
		g_free (msg);
	}
	else
	{
		doclen = document_get_length (savedoc);
		doc_buffer = g_malloc (++doclen);
		document_get_text (savedoc, doclen, doc_buffer);

		nwriten = fwrite (doc_buffer, sizeof (gchar), doclen, f);
		if (nwriten != doclen)
		{
				cssed_error_message
					("Error while writting to file",
					 "There is a mismatch between the file size\nand the bytes written.\nThe file can be damaged.");
		}
		document_set_modified (savedoc, FALSE);
		g_free (doc_buffer);
		fclose (f);
	}

	g_free (data);
}
void
on_save_as_fileselection_ok_button_clicked (GtkButton * button,
					    gpointer user_data)
{
	Tcssed_doc *savedoc;
	Tcssed_fileselector_data *data;
	Tcssed_window *window;	// main window object
	GtkWidget *notebook;
	GtkWidget *file_selector;
	GtkWidget *editor;
	const gchar *selected_filename;
	gchar *doc_buffer;
	gchar *filename;
	gint doclen;
	gint current_index;
	size_t nwriten;
	FILE *f;

	gchar *test;

	data = (Tcssed_fileselector_data *) user_data;
	window = (Tcssed_window *) data->main_window;
	notebook = GTK_WIDGET (data->notebook);
	file_selector = GTK_WIDGET (data->fileselector);
	savedoc = document_get_current (window);

	selected_filename =
		gtk_file_selection_get_filename (GTK_FILE_SELECTION
						 (file_selector));
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));	// destroy the selector

	f = fopen (selected_filename, "w");
	if (f == NULL)
	{
		gchar *msg;
		msg = g_strdup_printf ("The file %s\ncan not be opened for writting.",
				       selected_filename);
		cssed_error_message (msg,"Can't open file");
		g_free (msg);
	}
	else
	{
		doclen = document_get_length (savedoc);
		doc_buffer = g_malloc (++doclen);
		document_get_text (savedoc, doclen, doc_buffer);

		nwriten = fwrite (doc_buffer, sizeof (gchar), doclen, f);
		if (nwriten != doclen)
		{
			cssed_error_message
			("There is a mismatch between the file size\nand the bytes written.\nThe file can be damaged.","Error while writting to file");
		}
		document_set_modified (savedoc, FALSE);
		current_index =
			gtk_notebook_get_current_page (GTK_NOTEBOOK
						       (notebook));
		editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
						    current_index);
		filename = g_strdup (selected_filename);	// to avoid compiler warnings
		gtk_notebook_set_tab_label_text (GTK_NOTEBOOK (notebook),
						 editor, basename (filename));
		g_free (filename);
		g_free (savedoc->filename);
		savedoc->filename = g_strdup (selected_filename);
		g_free (doc_buffer);
		fclose (f);
	}
	g_free (data);
}
void
on_menu_guardar_como_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *file_selector;
	Tcssed_window *window;	// main window object

	window = CSSED_WINDOW (user_data);
	file_selector = create_save_as_fileselection (window);
	gtk_widget_show (file_selector);
}
void// FIXME check all docs for unsaved ones and prompt to user for saving
on_menu_salir_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	gtk_main_quit ();
}
void
on_menu_cortar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_cortar_clicked (NULL, user_data);
}
void
on_menu_copiar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_copiar_clicked (NULL, user_data);
}
void
on_menu_pegar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	on_button_pegar_clicked (NULL, user_data);
}


void
on_menu_borrar_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	
	document_clear_sel(tmpdoc);
}


void
on_menu_about_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWidget *about = create_about ();
	if (about != NULL)
		gtk_widget_show (about);
}

/* dynamic menu items */
// needs fix            
void
on_menu_line_numbers_toggled (GtkCheckMenuItem * checkmenuitem,
			      gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_line_numbers_toggled  - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_line_numbers_visible (tmpdoc, visible);
	}
}

// needs fix
void
on_menu_line_endings_toggled (GtkCheckMenuItem * checkmenuitem,
			      gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_line_endings_toggled - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_line_endings_visible (tmpdoc, visible);
	}
}
// needs fix
void
on_menu_line_wrapping_toggled (GtkCheckMenuItem * checkmenuitem,
			       gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean wrapped;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_line_endings_toggled - document NULL");
		return;
	}
	else
	{
		wrapped = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_lines_wrapped (tmpdoc, wrapped);
	}
}
// needs fix
void
on_menu_ver_white_spaces_toggled (GtkCheckMenuItem * checkmenuitem,
				  gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean visible;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{	// FIXME say something to user
		g_print ("callbacks.c:on_menu_ver_white_spaces_toggled - document NULL");
		return;
	}
	else
	{
		visible = gtk_check_menu_item_get_active (checkmenuitem);
		document_set_white_spaces_visible (tmpdoc, visible);
	}
}

void
on_menu_item_autocompletion_toggled	(GtkCheckMenuItem      *checkmenuitem,   
	gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean active;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);	
	
	active =  gtk_check_menu_item_get_active (checkmenuitem);
	tmpdoc->autocompletion = active;
}

// needs fix
void
on_menu_eol_mode_cr_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	
	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_ver_white_spaces_toggled - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_CR);
	}
}
// needs fix
void
on_menu_eol_mode_lf_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_ver_white_spaces_toggled - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_LF);
	}
}


// needs fix
void
on_menu_eol_mode_crlf_activate (GtkMenuItem * checkmenuitem,
				gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{			// FIXME say something to user
		g_print ("callbacks.c:on_menu_ver_white_spaces_toggled - document NULL");
		return;
	}
	else
	{
		document_convert_eols (tmpdoc, SC_EOL_CRLF);
	}
}

/* set the child menus state */
void
on_menu_view_activate (GtkMenuItem * checkmenuitem, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	Tcssed_dyn_menus *menus;
	gboolean visible;
	gint eolmode;

	window = CSSED_WINDOW (user_data);
	menus = window->dynamic_menus;
	tmpdoc = document_get_current (window);

	if (tmpdoc == NULL)
	{
		cssed_error_message
			("Can't get current document to change menus.\nMenus state will be out of sync with the document\n",
			 "Menu error");
		return;
	}
	visible = document_get_line_numbers_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_line_numbers, visible);
	visible = document_get_line_endings_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_endings, visible);
	visible = document_get_lines_wrapped (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_lines_wrapped, visible);
	visible = document_get_white_spaces_visible (tmpdoc);
	gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_white_spaces, visible);
	if( tmpdoc->autocompletion ){
		gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_autocompletion, TRUE);
	}else{
		gtk_check_menu_item_set_active ((GtkCheckMenuItem *) menus->
					menu_item_autocompletion, FALSE);
	}
}

/* 
	scan the current selector: extract the buffer and calls 
	parser_css_selector_buffer_to_list()
*/
void
on_menu_scan_selector_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	ScintillaObject *sci;
	gint position;
	gint open_brace;
	gint close_brace;
	gchar *buffer;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);

	sci = (ScintillaObject *) sci_get_from_document (tmpdoc);
	position = document_get_current_position (tmpdoc);

#ifdef DEBUG
	g_print ("Position at %d\n", position);
#endif

	// set the anchor
	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	open_brace = SSM (sci, SCI_SEARCHPREV, 0, (sptr_t) "{");

#ifdef DEBUG
	if (open_brace == -1)
		g_print ("No open brace before\n");
	else
		g_print ("Open brace at %d\n", open_brace);
#endif

	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	close_brace = SSM (sci, SCI_SEARCHNEXT, 0, (sptr_t) "}");

#ifdef DEBUG
	if (close_brace == -1)
		g_print ("No close brace after\n");
	else
		g_print ("Close brace at %d\n", close_brace);
#endif

	if (close_brace < position)
	{
		cssed_error_message
			("You must place the cursor between the opening\nand closing breaces of any selector to be scanned\n",
			 "The cursor must be into a selector.");
	}
	else
	{
		if ((open_brace != -1) && (close_brace != -1))
		{		// we're in a selector then
			SSM (sci, SCI_SETSEL, open_brace + 1, close_brace);
			buffer = g_malloc ((close_brace -
					    open_brace) * sizeof (gchar));
			SSM (sci, SCI_GETSELTEXT, 0, (sptr_t) buffer);

			g_strstrip (buffer);
#ifdef DEBUG
			g_print ("-------\n%s\n-----\n", buffer);
#endif
			parser_css_selector_buffer_to_list (buffer, window,
							    tmpdoc);
		}
	}
}
void
on_button_abrir_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_abrir_activate (NULL, user_data);
}
void
on_button_nuevo_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_nuevo_activate (NULL, user_data);
}
void
on_button_cerrar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	Tcssed_doc *doc;
	gint page_num;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	if (doc->modified)
	{
		//  FIXME [ yes | no | cancel ] dialog must pop
		g_print ("Doc modified closing\n\n");
	}
	document_dispose (doc);
}
void
on_button_buscar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *searchdlg;

	searchdlg = create_search_dialog (user_data);
	gtk_widget_show (searchdlg);
}
void
on_button_reemplazar_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *replacedlg;

	replacedlg = create_replace_dialog (user_data);	
	gtk_widget_show (replacedlg);
}
void
on_button_copiar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_copy (tmpdoc);
}
void
on_button_pegar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_paste (tmpdoc);
}

void
on_button_cortar_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;

	window = CSSED_WINDOW (user_data);
	tmpdoc = document_get_current (window);
	document_cut (tmpdoc);
}

void
on_button_color_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *colorselection_dlg;
	GdkColor color;
	gchar* str;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	str = g_strdup("Choose color"); // will be disposed by callbacks
	colorselection_dlg = create_color_selector_dialog (window, str, FALSE);
	gtk_widget_show (colorselection_dlg);
}


void
on_button_guardar_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_guardar_activate (NULL, user_data);
}


void
on_button_guardarcomo_clicked (GtkButton * button, gpointer user_data)
{
	on_menu_guardar_como_activate (NULL, user_data);
}

void
on_button_next_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gint line;
	gint position;


	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	position = document_get_current_position (doc);
	line = document_get_line_from_position (doc, position);

	if (!document_marker_next (doc, ++line, 1))
	{
		cssed_error_message ("There are not markers between the current\nposition and the end of the document.", "No next markers");
	}
}

void
on_button_prev_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gint line;
	gint position;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	position = document_get_current_position (doc);
	line = document_get_line_from_position (doc, position);

	if (!document_marker_prev (doc, --line, 1))
	{
		cssed_error_message ("There are not markers between the current\nposition and the start of the document.", "No previous markers");
	}
}

void
on_button_undo_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	Tcssed_doc *doc;
	gint page_num;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	document_undo (doc);
}

void
on_button_redo_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *editor;
	GtkWidget *notebook;
	Tcssed_doc *doc;
	gint page_num;
	Tcssed_window *window;

	window = CSSED_WINDOW (user_data);

	notebook = GTK_WIDGET (window->notebook_docs);
	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
					    page_num);
	doc = (Tcssed_doc *) document_get_from_editor (window, editor);

	document_redo (doc);
}

void
on_button_zoom_in_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_in (doc);
}

void
on_button_zoom_out_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_out (doc);
}

void
on_button_zoom_off_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	document_zoom_off (doc);
}

// fixme ask to save modified documents
void
on_window_destroy (GtkObject * object, gpointer user_data)
{
	gtk_main_quit ();
}
/*
void
on_color_selection_ok_button_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *colorselectiondlg;
	Tcssed_window *window;
	Tcssed_colorselector_data *cseldata;
	Tcssed_doc *tmpdoc;
	GtkColorSelection *colorsel;
	GdkColor color;
	gchar *rgbstring;

	cseldata = (Tcssed_colorselector_data *) user_data;
	colorselectiondlg = cseldata->colorselectordlg;
	colorsel =
		GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG
				     (colorselectiondlg)->colorsel);
	window = CSSED_WINDOW (cseldata->main_window);
	tmpdoc = document_get_current (window);
	gtk_color_selection_get_current_color (colorsel, &color);

#ifdef DEBUG
	g_print ("User_data %x, data %x, window %x,doc %x\n", user_data,
		 cseldata, window, tmpdoc);
	g_print ("callbacks.c:on_color_selection_ok_button_clicked()\n\t\tColor decimal R=%d G=%d B=%d\n\n", color.red, color.green, color.blue);
#endif
	rgbstring = gdk_color_to_rgb_triplet_string (&color);

	if ((cseldata->previous_string) != NULL)
	{
		document_add_text (tmpdoc,
				   (gchar *) cseldata->previous_string);
		document_add_text (tmpdoc, ":");
		document_add_text (tmpdoc, rgbstring);
		document_add_text (tmpdoc, ";\n\t");
		// it was reserved in on_treeview_css_properties_row_activated()
		// with strdup, so must be freed
		g_free (cseldata->previous_string);
	}
	else
	{
		document_add_text (tmpdoc, rgbstring);
	}

	gtk_widget_destroy (colorselectiondlg);
	g_free (user_data);
	g_free (rgbstring);
}
*/
void
on_search_dialog_find_button_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_text_to_find");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}

	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}

void
on_dialog_replace_search_button_clicked (GtkButton * button,
					 gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_find");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}

	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}

void
on_dialog_replace_apply_button_clicked (GtkButton * button,
					gpointer user_data)
{
	GtkWidget *entry_text_to_find;
	GtkWidget *entry_text_to_replace;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	gboolean isregexp;
	gboolean is_perl_regexp;
	gboolean is_match_case;
	gboolean is_whole_word;
	gboolean is_word_start;
	gint search_flags;
	Tcssed_window *window;
	Tcssed_doc *doc;

	gint selection_start;
	gint selection_end;

	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);

	selection_start = document_get_selection_start (doc);
	selection_end = document_get_selection_end (doc);

	entry_text_to_find =
		lookup_widget (GTK_WIDGET (button), "entry_find");
	entry_text_to_replace =
		lookup_widget (GTK_WIDGET (button), "entry_replace");
	checkbutton_regex =
		lookup_widget (GTK_WIDGET (button), "checkbutton_regex");
	radiobutton_perl_regex =
		lookup_widget (GTK_WIDGET (button), "radiobutton_perl_regex");
	checkbutton_match_case =
		lookup_widget (GTK_WIDGET (button), "checkbutton_match_case");
	checkbutton_whole_word =
		lookup_widget (GTK_WIDGET (button), "checkbutton_whole_word");
	checkbutton_word_start =
		lookup_widget (GTK_WIDGET (button), "checkbutton_word_start");

	if (selection_start >= selection_end)
	{
		cssed_error_message	("There must be some text selected\nto be replaced.\nPlease select the text to replace.",
				     "Nothing to replace");
		return;
	}
	else
	{
		document_replace_sel (doc,
				      (gchar *)
				      gtk_entry_get_text (GTK_ENTRY
							  (entry_text_to_replace)));
	}
	// search next
	isregexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_regex));
	is_perl_regexp =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (radiobutton_perl_regex));
	is_match_case =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_match_case));
	is_whole_word =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_whole_word));
	is_word_start =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_word_start));

	if (isregexp)
	{
		if (is_perl_regexp)
		{
			search_flags |= SCFIND_REGEXP;
		}
		else
		{
			search_flags |= SCFIND_POSIX;
		}
	}
	if (is_whole_word)
		search_flags |= SCFIND_WHOLEWORD;
	if (is_match_case)
		search_flags |= SCFIND_MATCHCASE;
	if (is_word_start)
		search_flags |= SCFIND_WORDSTART;
	document_search_text (doc,
			      (gchar *)
			      gtk_entry_get_text (GTK_ENTRY
						  (entry_text_to_find)),
			      search_flags);
}

void 
on_treeview_css_properties_row_activated (GtkTreeView * treeview,
					  GtkTreePath * path,
					  GtkTreeViewColumn * col,
					  gpointer user_data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *attribute;
	gchar *property;
	Tcssed_doc *tmpdoc;
	Tcssed_window *window;
	gchar *line_to_insert;
	gchar* strdialog;
	gint depth;
	GtkWidget *dialog;

	window = CSSED_WINDOW (user_data);
	depth = gtk_tree_path_get_depth (path);
	tmpdoc = document_get_current(window);
	
	if (depth != 3)// catch only the third col signals
	{			
#ifdef DEBUG
		g_print ("callbacks.c:on_treeview_css_properties_row_activated\n\ttree path depth %d ignoring\n\n", depth);
#endif
		return;
	}
	model = gtk_tree_view_get_model (treeview);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 2, &attribute, -1);	// columns 0,1,2
#ifdef DEBUG
		g_print ("callbacks.c:on_treeview_css_properties_row_activated\n* Double-clicked row contains:\n\t\tattr -> %s\n", attribute);
#endif
	}else{
		return;
	}
	gtk_tree_path_up (path);
	if (gtk_tree_model_get_iter (model, &iter, path))
	{
		gtk_tree_model_get (model, &iter, 1, &property, -1);	// columns 0,1,2
#ifdef DEBUG
		g_print ("\t\tproperty -> %s\n\n", property);
#endif
	}
	else
	{
		g_free (attribute);
		return;
	}
	// call fire dialog on document.c
	fire_dialog_from_strings_or_write_output(window,tmpdoc,attribute,property, TRUE );
	g_free (attribute);
	g_free (property);
}


void 
on_menu_autocompletion_activate	
		(GtkMenuItem	  *menuitem,   		gpointer user_data)
{
	Tcssed_window *window;
	Tcssed_doc *doc;
	gchar* list;
	
	window = CSSED_WINDOW (user_data);
	doc = document_get_current (window);
	list = window->keywords_list;

	document_show_autocompletion_list(doc,0,list);	
}

void
popmenu_autocomp_activate_cb (GtkMenuItem *menuitem, gpointer user_data)
{
	Tcssed_popmenu_data* data;
	Tcssed_window* window;
	Tcssed_doc* doc;
	gchar* attribute;
	gchar* property; // previous_string
	gboolean add_property_string;	
	
	data = (Tcssed_popmenu_data*) user_data;	
	window = CSSED_WINDOW(data->main_window);
	doc = (Tcssed_doc*) data->document;
#ifdef DEBUG	
	g_print("popmenu_autocomp_activate_cb %s - %s\n",(gchar*) data->attribute,(gchar*) data->property);
#endif	
	attribute = g_strdup( (gchar*) data->attribute);
	property = g_strdup( (gchar*) data->property); 
	add_property_string = data->add_property_string;
	// Memory is disposed in parent's "unmap" event ( see below ). This menu is only for one use.
	fire_dialog_from_strings_or_write_output(window,doc,attribute,property,add_property_string);
	g_free(attribute);
	g_free(property);
}

/* When the pop menu is created some memory is dinamically allocated and must
   be disposed. This callback will track the popmenu unmap menu and clear all
   memory allocated. The pointers to the data allocated are in a doubly
   linked list passed as user_data                                            */
void
popmenu_autocomp_event_after_cb(GtkWidget *widget,
                                GdkEvent *event,
                                gpointer user_data)
{
	GList* list;
	Tcssed_popmenu_data* data;
	
	if ( event->type == GDK_UNMAP) {
		list = g_list_first((GList*) user_data);
#ifdef DEBUG
				g_print("* After menu unmap event let's dispose allocated memory\n");
#endif			
		if( list != NULL ){
			while( list != NULL ){
				data = (Tcssed_popmenu_data*) list->data;
#ifdef DEBUG
				g_print("\tCleaning element %p - containig %s %s\n",data, data->attribute,data->property);
#endif				
				g_free(data->attribute);
				g_free(data->property);
				g_free(data);
				list = g_list_next(list);
			}
			g_list_free (list);
		}
#ifdef DEBUG	
		else
		{
			g_print("\tList is empty or NULL\n");		
		}
#endif	
		g_list_free(user_data);	
		
		if( GTK_IS_WIDGET(widget) ){// ensure the menu is destroyed
			gtk_widget_destroy(widget);
#ifdef DEBUG	
			g_print("Menu destroyed\n");
#endif	
		}
	}
}
/* common to all file selector dialogs */
void file_selector_close (GtkDialog       *dialog,
                          gpointer         user_data)
{
	g_free(user_data);
}
									
void
file_selector_cancel_button_clicked (GtkButton       *button,  
									gpointer         user_data)
{
	GtkWidget* dialog;
	
	dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
	if(GTK_WIDGET_TOPLEVEL (dialog)){ //ensure it's the toplevel
		gtk_widget_destroy(dialog);
	}
	g_free(user_data);
}
/* ***************************** HELP BUTTONS ******************************* */
/*    ---------------------------   TO DO ----------------------------------  */
void
on_color_selection_help_button_clicked (GtkButton * button,
					gpointer user_data)
{
}

void
on_dialog_search_help_button_clicked (GtkButton * button, gpointer user_data)
{
}


void
on_dialog_replace_help_button_clicked (GtkButton * button, gpointer user_data)
{
}
