/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "cssedwindow.h"
#include "document.h"
#include "cssdialogs-callbacks.h"
#include "cssdialogs-interface.h"
#include "support.h"
#include "utils.h"

//#define DEBUG

void on_angle_property_dialog_ok_button_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *angle_property_dialog;
	GtkWidget *entry_angle;
	GtkWidget *combo_entry_units;
	gchar *previous_string;
	gchar *output_string;
	gchar *angle;
	gchar *units;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	entry_angle = lookup_widget (GTK_WIDGET (button), "entry_angle");
	combo_entry_units =
		lookup_widget (GTK_WIDGET (button), "combo_entry_units");
	angle_property_dialog =
		lookup_widget (GTK_WIDGET (button), "angle_property_dialog");

#ifdef DEBUG
	g_print ("Entry angle: %x , combo units: %x\n", entry_angle,
		 combo_entry_units);
#endif

	angle = gtk_editable_get_chars (GTK_EDITABLE (entry_angle), 0, -1);
	units = gtk_editable_get_chars (GTK_EDITABLE (combo_entry_units), 0,
					-1);

#ifdef DEBUG
	g_print ("User_data %x, data %x, window %x,doc %x\n", user_data, data,
		 window, tmpdoc);
	g_print ("Previous string: %s\n", data->previous_string);
#endif
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	if (strlen (angle) > 0)
	{
		document_add_text (tmpdoc, angle);
	}
	else
	{
		document_add_text (tmpdoc, "0");
	}
	document_add_text (tmpdoc, units);
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (angle_property_dialog);
}

void on_background_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *background_property_dialog;
	GtkWidget *colorselection;
	GtkWidget *checkbutton_bg_color;
	GtkWidget *combo_entry_bg_color;
	GtkWidget *checkbutton_bg_position;
	GtkWidget *radiobutton_bg_pos_perclen;
	GtkWidget *radiobutton_bg_pos_fixed;
	GtkWidget *spinbutton_upper_left;
	GtkWidget *combo_entry_bg_perclen_upper_left;
	GtkWidget *combo_entry_bg_pos_fixed;
	GtkWidget *spinbutton_lower_right;
	GtkWidget *combo_entry_lower_right;
	GtkWidget *checkbutton_bg_image;
	GtkWidget *combo_entry_bg_image;
	GtkWidget *entry_uri;
	GtkWidget *checkbutton_bg_repeat;
	GtkWidget *combo_entry_bg_repeat;
	GtkWidget *checkbutton_bg_attachment;
	GtkWidget *combo_entry_bg_attachment;
	gchar *previous_string;
	gchar *output_string;
	gchar *colorstr;
	GdkColor gdkcolor;
	gchar *position;
	gchar *position_upper_left;
	gchar *position_lower_right;
	gchar *position_upper_left_unit;
	gint nposition_uper_left;
	gint nposition_lower_right;
	gchar *imagestr;
	gchar *repeatstr;
	gchar *attachmentstr;
	gboolean bimage;
	gboolean bcolor;
	gboolean bposition;
	gboolean bposperclen;
	gboolean brepeat;
	gboolean battachment;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	background_property_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "background_property_dialog");

	checkbutton_bg_image =
		lookup_widget (GTK_WIDGET (button), "checkbutton_bg_image");
	bimage = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					       (checkbutton_bg_image));
	checkbutton_bg_color =
		lookup_widget (GTK_WIDGET (button), "checkbutton_bg_color");
	bcolor = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					       (checkbutton_bg_color));
	checkbutton_bg_position =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_bg_position");
	bposition =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_bg_position));
	checkbutton_bg_repeat =
		lookup_widget (GTK_WIDGET (button), "checkbutton_bg_repeat");
	brepeat =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_bg_repeat));
	checkbutton_bg_attachment =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_bg_attachment");
	battachment =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_bg_attachment));
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	if (!bimage && !bcolor && !bposition && !brepeat && !battachment)
	{
		// background none then
		document_add_text (tmpdoc, "none");
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		document_write_error_to_program_output(tmpdoc,"border dialog: no value selected, defaulting to none");
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy (background_property_dialog);
		return;
	}

	// image
	if (bimage)
	{
		entry_uri = lookup_widget (GTK_WIDGET (button), "entry_uri");
		imagestr =
			gtk_editable_get_chars (GTK_EDITABLE (entry_uri), 0,
						-1);
		document_add_text (tmpdoc, "url(");
		document_add_text (tmpdoc, imagestr);
		document_add_text (tmpdoc, ") ");
		g_free (imagestr);
	}

	// bgcolor
	if (bcolor)
	{
		combo_entry_bg_color =
			lookup_widget (GTK_WIDGET (button),
				       "combo_entry_bg_color");
		colorstr =
			gtk_editable_get_chars (GTK_EDITABLE
						(combo_entry_bg_color), 0,
						-1);

		if (strcmp (colorstr, "from color selector") == 0)
		{
			g_free (colorstr);
			colorselection =
				lookup_widget (GTK_WIDGET (button),
					       "colorselection");
			gtk_color_selection_get_current_color
				(GTK_COLOR_SELECTION (colorselection),
				 &gdkcolor);
			colorstr =
				gdk_color_to_rgb_triplet_string (&gdkcolor);
		}
		document_add_text (tmpdoc, colorstr);
		document_add_text (tmpdoc, " ");
		g_free (colorstr);
	}
	//position
	if (bposition)
	{
		// perclen
		radiobutton_bg_pos_perclen =
			lookup_widget (GTK_WIDGET (button),
				       "radiobutton_bg_pos_perclen");
		bposperclen =
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						      (radiobutton_bg_pos_perclen));
		if (bposperclen)
		{
			spinbutton_upper_left =
				lookup_widget (GTK_WIDGET (button),
					       "spinbutton_upper_left");
			spinbutton_lower_right =
				lookup_widget (GTK_WIDGET (button),
					       "spinbutton_lower_right");
			combo_entry_bg_perclen_upper_left =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_unit");

			position_upper_left_unit =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_bg_perclen_upper_left),
							0, -1);

			nposition_uper_left =
				gtk_spin_button_get_value_as_int
				(GTK_SPIN_BUTTON (spinbutton_upper_left));
			nposition_lower_right =
				gtk_spin_button_get_value_as_int
				(GTK_SPIN_BUTTON (spinbutton_lower_right));

			position_upper_left =
				g_strdup_printf ("%d%s", nposition_uper_left,
						 position_upper_left_unit);
			position_lower_right =
				g_strdup_printf ("%d%s",
						 nposition_lower_right,
						 position_upper_left_unit);
			//g_strdup_printf
			document_add_text (tmpdoc, position_upper_left);
			document_add_text (tmpdoc, " ");
			document_add_text (tmpdoc, position_lower_right);
			document_add_text (tmpdoc, " ");

			g_free (position_upper_left_unit);
			g_free (position_lower_right);
			g_free (position_upper_left);
		}
		else
		{
			combo_entry_bg_pos_fixed =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_bg_pos_fixed");
			position =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_bg_pos_fixed),
							-0, -1);
			document_add_text (tmpdoc, position);
			document_add_text (tmpdoc, " ");
			g_free (position);
		}
	}
	//repeat
	if (brepeat)
	{
		combo_entry_bg_repeat =
			lookup_widget (GTK_WIDGET (button),
				       "combo_entry_bg_repeat");
		repeatstr =
			gtk_editable_get_chars (GTK_EDITABLE
						(combo_entry_bg_repeat), -0,
						-1);
		document_add_text (tmpdoc, repeatstr);
		document_add_text (tmpdoc, " ");
		g_free (repeatstr);
	}
	//attachment
	if (battachment)
	{
		combo_entry_bg_attachment =
			lookup_widget (GTK_WIDGET (button),
				       "combo_entry_bg_attachment");
		attachmentstr =
			gtk_editable_get_chars (GTK_EDITABLE
						(combo_entry_bg_attachment),
						-0, -1);
		document_add_text (tmpdoc, attachmentstr);
		document_add_text (tmpdoc, " ");
		g_free (attachmentstr);
	}
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (background_property_dialog);
}

void
on_border_properties_okbutton_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *border_properties_dialog;
	GtkWidget *checkbutton_border_length;
	GtkWidget *combo_entry_border_length_selector;
	GtkWidget *spinbutton_len;
	GtkWidget *combo_entry_len;
	GtkWidget *checkbutton_border_style;
	GtkWidget *combo_entry_border_style;
	GtkWidget *checkbutton_border_color;
	GtkWidget *combo_entry_border_color;
	GtkWidget *colorselection;
	GdkColor gdkcolor;
	gboolean blength;
	gboolean bstyle;
	gboolean bcolor;
	gchar *strlength;
	gchar *strunit;
	gchar *strlenunit;
	gchar *strstyle;
	gchar *strcolor;
	gint len;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	border_properties_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "border_properties_dialog");
	checkbutton_border_length =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_length");
	checkbutton_border_style =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_style");
	checkbutton_border_color =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_color");

	blength =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_border_length));
	bstyle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					       (checkbutton_border_style));
	bcolor = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					       (checkbutton_border_color));
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	if (!blength && !bstyle && !bcolor)
	{
		document_add_text (tmpdoc, "none");
		document_write_error_to_program_output(tmpdoc,"border dialog: no value selected defaulting to \"none\"");
	}
	else
	{
		// length
		if (blength)
		{
			combo_entry_border_length_selector =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_length_selector");
			strlength =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_length_selector),
							-0, -1);

			if (strcmp (strlength, "Use length") == 0)
			{
				spinbutton_len =
					lookup_widget (GTK_WIDGET (button),
						       "spinbutton_len");
				len = gtk_spin_button_get_value_as_int
					(GTK_SPIN_BUTTON (spinbutton_len));
				combo_entry_len =
					lookup_widget (GTK_WIDGET (button),
						       "combo_entry_len");
				strunit =
					gtk_editable_get_chars (GTK_EDITABLE
								(combo_entry_len),
								-0, -1);
				strlenunit =
					g_strdup_printf ("%d%s ", len,
							 strunit);
				document_add_text (tmpdoc, strlenunit);
				g_free (strlenunit);
				g_free (strunit);
			}
			else
			{
				document_add_text (tmpdoc, strlength);
				document_add_text (tmpdoc, " ");
			}
			g_free (strlength);
		}
		// style
		if (bstyle)
		{
			combo_entry_border_style =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_style");
			strstyle =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_style),
							-0, -1);
			document_add_text (tmpdoc, strstyle);
			document_add_text (tmpdoc, " ");
			g_free (strstyle);
		}
		// color
		if (bcolor)
		{
			combo_entry_border_color =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_color");
			strcolor =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_color),
							-0, -1);

			if (strcmp (strcolor, "from color selector") == 0)
			{
				g_free (strcolor);
				colorselection =
					lookup_widget (GTK_WIDGET (button),
						       "colorselection");
				gtk_color_selection_get_current_color
					(GTK_COLOR_SELECTION (colorselection),
					 &gdkcolor);
				strcolor =
					gdk_color_to_rgb_triplet_string
					(&gdkcolor);
				document_add_text (tmpdoc, strcolor);
			}
			else
			{
				document_add_text (tmpdoc, strcolor);
			}
			g_free (strcolor);
		}
	}
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (border_properties_dialog);
}

void on_border_width_properties_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *border_width_properties_dialog;
	GtkWidget *checkbutton_border_width_top;
	GtkWidget *combo_entry_border_width_top_units;
	GtkWidget *combo_entry_border_width_top_selector;
	GtkWidget *spinbutton_border_width_top;
	GtkWidget *checkbutton_border_width_right;
	GtkWidget *combo_entry_border_width_right_units;
	GtkWidget *combo_entry_border_width_right_selector;
	GtkWidget *spinbutton_border_width_right;
	GtkWidget *checkbutton_border_width_bottom;
	GtkWidget *combo_entry_border_width_bottom_units;
	GtkWidget *combo_entry_border_width_bottom_selector;
	GtkWidget *spinbutton_border_width_bottom;
	GtkWidget *checkbutton_border_width_left;
	GtkWidget *combo_entry_border_width_left_units;
	GtkWidget *combo_entry_border_width_left_selector;
	GtkWidget *spinbutton_border_width_left;
	gboolean btop;
	gboolean bright;
	gboolean bbottom;
	gboolean bleft;
	gchar *strout_top;
	gchar *strlength_top;
	gchar *strunit_top;
	gchar *strout_right;
	gchar *strlength_right;
	gchar *strunit_right;
	gchar *strout_bottom;
	gchar *strlength_bottom;
	gchar *strunit_bottom;
	gchar *strout_left;
	gchar *strlength_left;
	gchar *strunit_left;
	gint nlength;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	border_width_properties_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "border_width_properties_dialog");
	checkbutton_border_width_top =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_width_top");
	checkbutton_border_width_right =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_width_right");
	checkbutton_border_width_bottom =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_width_bottom");
	checkbutton_border_width_left =
		lookup_widget (GTK_WIDGET (button),
			       "checkbutton_border_width_left");

	btop = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					     (checkbutton_border_width_top));
	bright = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					       (checkbutton_border_width_right));
	bbottom =
		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_border_width_bottom));
	bleft = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
					      (checkbutton_border_width_left));
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	if (!btop && !bright && !bbottom && !bleft)
	{
		document_add_text (tmpdoc, "none");
		document_write_error_to_program_output(tmpdoc,"border width single dialog: no value selected defaulting to \"none\"");
	}
	else
	{
		// TOP
		if (btop)
		{
			combo_entry_border_width_top_selector =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_width_top_selector");
			strlength_top =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_width_top_selector),
							-0, -1);
			if (strcmp (strlength_top, "Use length") == 0)
			{
				combo_entry_border_width_top_units =
					lookup_widget (GTK_WIDGET (button),
						       "combo_entry_border_width_top_units");
				strunit_top =
					gtk_editable_get_chars (GTK_EDITABLE
								(combo_entry_border_width_top_units),
								-0, -1);
				spinbutton_border_width_top =
					lookup_widget (GTK_WIDGET (button),
						       "spinbutton_border_width_top");
				nlength =
					gtk_spin_button_get_value_as_int
					(GTK_SPIN_BUTTON
					 (spinbutton_border_width_top));
				strout_top =
					g_strdup_printf ("%d%s", nlength,
							 strunit_top);
				document_add_text (tmpdoc, strout_top);
				document_add_text (tmpdoc, " ");
				g_free (strout_top);
				g_free (strunit_top);
			}
			else
			{
				document_add_text (tmpdoc, strlength_top);
				document_add_text (tmpdoc, " ");
			}
			g_free (strlength_top);
		}
		// RIGHT
		if (bright)
		{
			combo_entry_border_width_right_selector =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_width_right_selector");
			strlength_right =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_width_right_selector),
							-0, -1);
			if (strcmp (strlength_right, "Use length") == 0)
			{
				combo_entry_border_width_right_units =
					lookup_widget (GTK_WIDGET (button),
						       "combo_entry_border_width_right_units");
				strunit_right =
					gtk_editable_get_chars (GTK_EDITABLE
								(combo_entry_border_width_right_units),
								-0, -1);
				spinbutton_border_width_right =
					lookup_widget (GTK_WIDGET (button),
						       "spinbutton_border_width_right");
				nlength =
					gtk_spin_button_get_value_as_int
					(GTK_SPIN_BUTTON
					 (spinbutton_border_width_right));
				strout_right =
					g_strdup_printf ("%d%s", nlength,
							 strunit_right);
				document_add_text (tmpdoc, strout_right);
				document_add_text (tmpdoc, " ");
				g_free (strout_right);
				g_free (strunit_right);
			}
			else
			{
				document_add_text (tmpdoc, strlength_right);
				document_add_text (tmpdoc, " ");
			}
			g_free (strlength_right);
		}
		// BOTTOM
		if (bbottom)
		{
			combo_entry_border_width_bottom_selector =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_width_bottom_selector");
			strlength_bottom =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_width_bottom_selector),
							-0, -1);
			if (strcmp (strlength_bottom, "Use length") == 0)
			{
				combo_entry_border_width_bottom_units =
					lookup_widget (GTK_WIDGET (button),
						       "combo_entry_border_width_bottom_units");
				strunit_bottom =
					gtk_editable_get_chars (GTK_EDITABLE
								(combo_entry_border_width_bottom_units),
								-0, -1);
				spinbutton_border_width_bottom =
					lookup_widget (GTK_WIDGET (button),
						       "spinbutton_border_width_bottom");
				nlength =
					gtk_spin_button_get_value_as_int
					(GTK_SPIN_BUTTON
					 (spinbutton_border_width_bottom));
				strout_bottom =
					g_strdup_printf ("%d%s", nlength,
							 strunit_bottom);
				document_add_text (tmpdoc, strout_bottom);
				document_add_text (tmpdoc, " ");
				g_free (strout_bottom);
				g_free (strunit_bottom);
			}
			else
			{
				document_add_text (tmpdoc, strlength_bottom);
				document_add_text (tmpdoc, " ");
			}
			g_free (strlength_bottom);
		}

		// LEFT
		if (bleft)
		{
			combo_entry_border_width_left_selector =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_border_width_left_selector");
			strlength_left =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_border_width_left_selector),
							-0, -1);
			if (strcmp (strlength_left, "Use length") == 0)
			{
				combo_entry_border_width_left_units =
					lookup_widget (GTK_WIDGET (button),
						       "combo_entry_border_width_left_units");
				strunit_left =
					gtk_editable_get_chars (GTK_EDITABLE
								(combo_entry_border_width_left_units),
								-0, -1);
				spinbutton_border_width_left =
					lookup_widget (GTK_WIDGET (button),
						       "spinbutton_border_width_left");
				nlength =
					gtk_spin_button_get_value_as_int
					(GTK_SPIN_BUTTON
					 (spinbutton_border_width_left));
				strout_left =
					g_strdup_printf ("%d%s", nlength,
							 strunit_left);
				document_add_text (tmpdoc, strout_left);
				document_add_text (tmpdoc, " ");
				g_free (strout_left);
				g_free (strunit_left);
			}
			else
			{
				document_add_text (tmpdoc, strlength_left);
				document_add_text (tmpdoc, " ");
			}
			g_free (strlength_left);
		}
	}
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (border_width_properties_dialog);
}

void on_border_width_single_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *border_width_single_properties_dialog;
	GtkWidget *combo_entry_selector;
	GtkWidget *spinbutton_length;
	GtkWidget *combo_entry_units;
	gchar *strwidth;
	gchar *strunit;
	gchar *strout;
	gint len;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	border_width_single_properties_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "border_width_single_properties_dialog");
	combo_entry_selector =
		lookup_widget (GTK_WIDGET (button), "combo_entry_selector");
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	strwidth =
		gtk_editable_get_chars (GTK_EDITABLE (combo_entry_selector),
					0, -1);
	if (strcmp (strwidth, "Use percentage or length") == 0)
	{
		spinbutton_length =
			lookup_widget (GTK_WIDGET (button),
				       "spinbutton_length");
		combo_entry_units =
			lookup_widget (GTK_WIDGET (button),
				       "combo_entry_units");
		len = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							(spinbutton_length));
		strunit =
			gtk_editable_get_chars (GTK_EDITABLE
						(combo_entry_units), 0, -1);
		strout = g_strdup_printf ("%d%s", len, strunit);
		document_add_text (tmpdoc, strout);
		g_free (strout);
		g_free (strunit);
	}
	else
	{
		document_add_text (tmpdoc, strwidth);
	}
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (strwidth);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (border_width_single_properties_dialog);
}

/* This add counters, and needs three callbacks as various counters can be     
	 as various counters can be added to the list.                             
* ************************************************************************** */
void on_button_counter_increment_reset_add_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *spinbutton;
	GtkWidget *entry;
	GtkTreeIter iter;
	gchar *strident;
	gint increment;

	spinbutton =
		lookup_widget (GTK_WIDGET (button),
			       "spinbutton_counter_increment_reset_step");
	entry = lookup_widget (GTK_WIDGET (button),
			       "entry_counter_increment_reset_identifyer");

	strident = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
	increment =
		gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
						  (spinbutton));

	store = (GtkListStore *) user_data;
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, 0, strident, 1, increment, -1);

	g_free (strident);
}


void on_button_counter_increment_reset_del_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}

void on_dialog_counter_increment_reset_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *counter_increment_reset_dialog;
	GtkTreeModel *list_store;
	GtkWidget *spinbutton;
	GtkWidget *entry;
	GtkWidget *treeview;
	GtkTreeIter iter;
	gboolean valid;
	gchar *strout;
	gchar *str_data;
	gint int_data;
	gchar *int_str;
	gchar *strident;
	gint increment;	
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	spinbutton =
		lookup_widget (GTK_WIDGET (button),
			       "spinbutton_counter_increment_reset_step");
	entry = lookup_widget (GTK_WIDGET (button),
			       "entry_counter_increment_reset_identifyer");

	counter_increment_reset_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "counter_increment_reset_dialog");
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	valid = gtk_tree_model_get_iter_first (list_store, &iter);

	if (valid)
	{			// there are values in the store
		// parser the list
		while (valid)
		{
			gtk_tree_model_get (list_store, &iter,
					    0, &str_data, 1, &int_data, -1);

			document_add_text (tmpdoc, str_data);

			// avoid defaults counter-reset to 0 and counter increment in one
			if (!
			    ((int_data == 0)
			     &&
			     (strcmp ("counter-reset", data->previous_string)
			      == 0)) && !((int_data == 1)
					  &&
					  (strcmp
					   ("counter-increment",
					    data->previous_string) == 0)))
			{
				int_str = g_strdup_printf (" %d ", int_data);
				document_add_text (tmpdoc, int_str);
				g_free (int_str);
			}

			g_free (str_data);
			valid = gtk_tree_model_iter_next (list_store, &iter);
		}
	}
	else
	{
		spinbutton =
			lookup_widget (GTK_WIDGET (button),
				       "spinbutton_counter_increment_reset_step");
		entry = lookup_widget (GTK_WIDGET (button),
				       "entry_counter_increment_reset_identifyer");

		strident =
			gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
		increment =
			gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							  (spinbutton));

		document_add_text (tmpdoc, strident);
		// avoid defaults counter-reset to 0 and counter increment in one
		if (!
		    ((increment == 0)
		     && (strcmp ("counter-reset", data->previous_string) ==
			 0)) && !((increment == 1)
				  &&
				  (strcmp
				   ("counter-increment",
				    data->previous_string) == 0)))
		{
			strout = g_strdup_printf (" %d", increment);
			document_add_text (tmpdoc, strout);
			g_free (strout);
		}

		g_free (strident);
	}
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (counter_increment_reset_dialog);
}
void on_string_properties_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *string_properties_dialog;
	GtkWidget *entry_string_property_dialog;
	gchar *string;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	string_properties_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "string_properties_dialog");
	entry_string_property_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "entry_string_property_dialog");

	string = gtk_editable_get_chars (GTK_EDITABLE
					 (entry_string_property_dialog), 0,
					 -1);

	if (strlen (string) > 0)
	{
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		document_add_text (tmpdoc, "\"");
		document_add_text (tmpdoc, string);
		document_add_text (tmpdoc, "\"");
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy (string_properties_dialog);
	}else{
		document_write_error_to_program_output(tmpdoc,"string dialog: refusing to add empty values.");
	}
	g_free (string);
}


void
on_uri_value_dialog_okbutton_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *uri_value_dialog;
	GtkWidget *entry_uri_value_dialog;
	gchar *string;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	uri_value_dialog =
		lookup_widget (GTK_WIDGET (button), "uri_value_dialog");
	entry_uri_value_dialog =
		lookup_widget (GTK_WIDGET (button), "entry_uri_value_dialog");

	string = gtk_editable_get_chars (GTK_EDITABLE
					 (entry_uri_value_dialog), 0, -1);

	if (strlen (string) > 0)
	{
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ":");
		}
		document_add_text (tmpdoc, " url( ");
		document_add_text (tmpdoc, string);
		document_add_text (tmpdoc, " )");
		if( add_previous ){
			document_add_text (tmpdoc, "\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy (uri_value_dialog);
	}else{
		document_write_error_to_program_output(tmpdoc, "uri dialog: rejecting to add empty values");
	}
	g_free (string);
}

void
on_attr_value_dialog_okbutton_clicked (GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *attr_value_dialog;
	GtkWidget *entry_attribute_value_dialog;
	gchar *string;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	attr_value_dialog =
		lookup_widget (GTK_WIDGET (button), "attr_value_dialog");
	entry_attribute_value_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "entry_attribute_value_dialog");

	string = gtk_editable_get_chars (GTK_EDITABLE
					 (entry_attribute_value_dialog), 0,
					 -1);

	if (strlen (string) > 0)
	{
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ":");
		}
		document_add_text (tmpdoc, " attr( ");
		document_add_text (tmpdoc, string);
		document_add_text (tmpdoc, " )");
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy (attr_value_dialog);
	}

	g_free (string);
}

void on_counter_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *counter_property_dialog;
	GtkWidget *entry_counter_identifier;
	GtkWidget *checkbutton_after;
	GtkWidget *entry_string_after;
	GtkWidget *checkbutton_before;
	GtkWidget *entry_string_before;
	GtkWidget *checkbutton_list_style_tyle;
	GtkWidget *combo_entry_list_style_type;
	gchar *stridentifier;
	gchar *strafter;
	gchar *strbefore;
	gchar *strstyle;
	gboolean bafter;
	gboolean bbefore;
	gboolean bstyle;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	counter_property_dialog =
		lookup_widget (GTK_WIDGET (button),
			       "counter_property_dialog");

	entry_counter_identifier =
		lookup_widget (GTK_WIDGET (button),
			       "entry_counter_identifier");
	stridentifier =
		gtk_editable_get_chars (GTK_EDITABLE
					(entry_counter_identifier), 0, -1);

	if (strlen (stridentifier) > 0)
	{
		checkbutton_after =
			lookup_widget (GTK_WIDGET (button),
				       "checkbutton_after");
		checkbutton_before =
			lookup_widget (GTK_WIDGET (button),
				       "checkbutton_before");
		checkbutton_list_style_tyle =
			lookup_widget (GTK_WIDGET (button),
				       "checkbutton_list_style_tyle");

		bafter = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						       (checkbutton_after));
		bbefore =
			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						      (checkbutton_before));
		bstyle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
						       (checkbutton_list_style_tyle));
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		if (bafter)
		{
			entry_string_after =
				lookup_widget (GTK_WIDGET (button),
					       "entry_string_after");
			strafter =
				gtk_editable_get_chars (GTK_EDITABLE
							(entry_string_after),
							0, -1);
			document_add_text (tmpdoc, " \"");
			document_add_text (tmpdoc, strafter);
			document_add_text (tmpdoc, "\" ");
			g_free (strafter);
		}

		// add the identifier
		document_add_text (tmpdoc, " counter( ");
		document_add_text (tmpdoc, stridentifier);

		if (bstyle)
		{
			combo_entry_list_style_type =
				lookup_widget (GTK_WIDGET (button),
					       "combo_entry_list_style_type");
			strstyle =
				gtk_editable_get_chars (GTK_EDITABLE
							(combo_entry_list_style_type),
							0, -1);
			document_add_text (tmpdoc, ", ");
			document_add_text (tmpdoc, strstyle);
			g_free (strstyle);
		}

		document_add_text (tmpdoc, " )");

		if (bbefore)
		{
			entry_string_before =
				lookup_widget (GTK_WIDGET (button),
					       "entry_string_before");
			strbefore =
				gtk_editable_get_chars (GTK_EDITABLE
							(entry_string_before),
							0, -1);
			document_add_text (tmpdoc, " \"");
			document_add_text (tmpdoc, strbefore);
			document_add_text (tmpdoc, "\" ");
			g_free (strbefore);
		}
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy (counter_property_dialog);
	}else{
		document_write_error_to_program_output(tmpdoc,"counter dialog: rejecting to add empty identifier. Value required."); 
	}
	g_free (stridentifier);
}

/* This needs three callbacks                                                
 * ************************************************************************* */
void on_font_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *font_property_dialog;
	GtkWidget *radiobutton_system_fonts;
	GtkWidget *combo_entry_system_fonts;
	GtkWidget *checkbutton_size;
	GtkWidget *combo_entry_font_size_selector;
	GtkWidget *spinbutton_font_size;
	GtkWidget *combo_entry_font_size_units;
	GtkWidget *checkbutton_line_height;
	GtkWidget *spinbutton_line_height;
	GtkWidget *combo_entry_line_height_selector_and_units;
	GtkWidget *checkbutton_style;
	GtkWidget *combo_entry_style;
	GtkWidget *checkbutton_variant;
	GtkWidget *combo_entry_variant;
	GtkWidget *checkbutton_weight;
	GtkWidget *combo_entry_weight;
	GtkWidget *checkbutton_font_family;
	GtkWidget *combo_entry_font_family;
	GtkWidget *checkbutton_generic_family;
	GtkWidget *combo_entry_generic_family;
	GtkWidget *checkbutton_use_font_set;
	GtkWidget *treeview;
	
	GtkTreeModel *list_store;
	GtkTreeIter iter;
	gboolean valid;

	gboolean bsystemfonts;
	gboolean bsize;
	gboolean bheight;
	gboolean bstyle;
	gboolean bvariant;
	gboolean bweight;
	gboolean bfontfamily;
	gboolean bgenericfamily;
	gboolean bfontset;
	
	gchar *strfont;
	gchar* strfullsize;	
	gchar *strsize;
	gchar *strsize_units;
	gint nsize;
	gchar *strlheight;
	gchar* strfullheight;
	gint nheight;
	gchar* strstyle;
	gchar* strvariant;
	gchar* strweight;
	
	gchar* str_set_font;
	gchar* str_set_generic;
	
	gchar* str_font_family;
	gchar* str_gen_family;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	font_property_dialog =
		lookup_widget (GTK_WIDGET (button), "font_property_dialog");

	radiobutton_system_fonts =
		lookup_widget (GTK_WIDGET (button),
			       "radiobutton_system_fonts");
				   
	bsystemfonts = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (radiobutton_system_fonts));

	//radiobutton_typographical_notation = lookup_widget(GTK_WIDGET(button),"");    
	if( bsystemfonts ){
		combo_entry_system_fonts =
			lookup_widget (GTK_WIDGET (button),
			       "combo_entry_system_fonts");
		
		strfont = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_system_fonts),
					0,-1);
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		document_add_text (tmpdoc, strfont);
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (strfont);
	}else {
		checkbutton_size =
			lookup_widget (GTK_WIDGET (button), "checkbutton_size");
		checkbutton_line_height =
			lookup_widget (GTK_WIDGET (button), "checkbutton_line_height");
		checkbutton_style =
			lookup_widget (GTK_WIDGET (button), "checkbutton_style");
		checkbutton_variant =
			lookup_widget (GTK_WIDGET (button), "checkbutton_variant");
		checkbutton_weight =
			lookup_widget (GTK_WIDGET (button), "checkbutton_weight");
		checkbutton_font_family =
			lookup_widget (GTK_WIDGET (button),"checkbutton_font_family");
		checkbutton_generic_family =
			lookup_widget (GTK_WIDGET (button), "checkbutton_generic_family");
		checkbutton_use_font_set =
			lookup_widget (GTK_WIDGET (button), "checkbutton_use_font_set");
		
		bsize = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_size));
		bheight = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_line_height));
		bstyle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_style));
		bvariant = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_variant));
		bweight = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_weight));
		bfontfamily = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_font_family));
		bgenericfamily = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_generic_family));
		bfontset = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				      (checkbutton_use_font_set));
					  
		if( bsize || bheight || bstyle || bvariant || bweight || bfontfamily ||
			bgenericfamily || bfontset)
		{ 
			if( add_previous ){
				document_add_text (tmpdoc, (gchar *) data->previous_string);
				document_add_text (tmpdoc, ": ");
			}	
			// font size
			if( bsize ){
				combo_entry_font_size_selector =
				lookup_widget (GTK_WIDGET (button),
							   "combo_entry_font_size_selector");
				
				strsize = gtk_editable_get_chars(
							GTK_EDITABLE(combo_entry_font_size_selector),
							0,-1);
				if( strcmp(strsize,"use percentage or length") == 0){
					spinbutton_font_size =
						lookup_widget (GTK_WIDGET (button), "spinbutton_font_size");
					combo_entry_font_size_units =
						lookup_widget (GTK_WIDGET (button),
								"combo_entry_font_size_units");	
	
					nsize = gtk_spin_button_get_value_as_int(
								GTK_SPIN_BUTTON(spinbutton_font_size));
						
					strsize_units = gtk_editable_get_chars(
								GTK_EDITABLE(combo_entry_font_size_units),0,-1);			
					strfullsize = g_strdup_printf("%d%s ", nsize, strsize_units);
					
					document_add_text(tmpdoc, strfullsize);
					
					g_free( strfullsize );
					g_free(	strsize_units );
									
				}else{
					document_add_text (tmpdoc, strsize);
				}
				document_add_text (tmpdoc, " ");
				g_free(strsize);
			}
			
			// line height "use number"
			if( bheight ){
				combo_entry_line_height_selector_and_units =
					lookup_widget (GTK_WIDGET (button),
							"combo_entry_line_height_selector_and_units");
				strlheight = gtk_editable_get_chars(
					GTK_EDITABLE(combo_entry_line_height_selector_and_units),0,-1);
				if( strcmp(strsize,"use number") == 0){
					spinbutton_line_height =
						lookup_widget (GTK_WIDGET (button), 
						"spinbutton_line_height");
					nheight = gtk_spin_button_get_value_as_int(
								GTK_SPIN_BUTTON(spinbutton_line_height));				
					strfullheight = g_strdup_printf("%d", nheight);				
					g_free(strfullheight);				
				}else{
					document_add_text (tmpdoc, strlheight);		
				}
	
				document_add_text (tmpdoc, " ");
				g_free(strlheight);
			}
			
			// style
			if( bstyle ){
				combo_entry_style =
					lookup_widget (GTK_WIDGET (button), "combo_entry_style");
				strstyle = gtk_editable_get_chars(
					GTK_EDITABLE(combo_entry_style),0,-1);
				document_add_text (tmpdoc, strstyle);	
				document_add_text (tmpdoc, " ");
				g_free(strstyle);			
			}		
			// variant		
			if( bvariant ){
				combo_entry_variant =
					lookup_widget (GTK_WIDGET (button), "combo_entry_variant");
				strvariant = gtk_editable_get_chars(
					GTK_EDITABLE(combo_entry_variant),0,-1);
				document_add_text (tmpdoc, strvariant);	
				document_add_text (tmpdoc, " ");
				g_free(strvariant);
			}
			// weight	
			if( bweight ){
				combo_entry_weight =
					lookup_widget (GTK_WIDGET (button), "combo_entry_weight");
				strweight = gtk_editable_get_chars(
					GTK_EDITABLE(combo_entry_weight),0,-1);
				document_add_text (tmpdoc, strweight);	
				document_add_text (tmpdoc, " ");
				g_free(strweight);
			}
			// font set
			if ( bfontset ){
				treeview = lookup_widget (GTK_WIDGET (button), "treeview");
				list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
				valid = gtk_tree_model_get_iter_first (list_store, &iter);
	
				if (valid)
				{	// there are values in the store
					// parser the list
					while (valid)
					{
						gtk_tree_model_get (list_store, &iter,
									0, &str_set_font, 1, &str_set_generic, -1);
			
						if( strlen(str_set_font) > 0){
							document_add_text (tmpdoc, "\"");
							document_add_text (tmpdoc, str_set_font);
							document_add_text (tmpdoc, "\"");
						}
						
						if( ( strlen(str_set_generic) > 0 ) && 
							( strlen(str_set_font)    > 0 )  ){
							document_add_text (tmpdoc, " , ");
						}
						
						if( strlen(str_set_generic) > 0){
							document_add_text (tmpdoc, str_set_generic);
						}					
						
						g_free (str_set_font);		
						g_free (str_set_generic);
						valid = gtk_tree_model_iter_next (list_store, &iter);
						if( valid ){
							document_add_text (tmpdoc, " , ");
						}else{
							document_add_text (tmpdoc, " ");
						}
					}
				}
			}else{
				// font family			
				if( bfontfamily ){
					combo_entry_font_family =
						lookup_widget (GTK_WIDGET (button),
							   "combo_entry_font_family");
					str_font_family = gtk_editable_get_chars(
							GTK_EDITABLE(combo_entry_font_family),0,-1);
					document_add_text (tmpdoc, "\"");
					document_add_text (tmpdoc, str_font_family);
					document_add_text (tmpdoc, "\"");
					g_free(str_font_family);		
				}
				
				if( bgenericfamily ){
					if (bfontfamily) { document_add_text (tmpdoc, " , "); }
					combo_entry_generic_family =
						lookup_widget (GTK_WIDGET (button),
							   "combo_entry_generic_family");
					str_gen_family = gtk_editable_get_chars(
							GTK_EDITABLE(combo_entry_font_family),0,-1);
					document_add_text (tmpdoc, str_gen_family);	
					g_free(str_gen_family);
				}	
			}	
			if( add_previous ){
				document_add_text (tmpdoc, ";\n\t");
			}
		}
	}
		
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy (font_property_dialog);
}

void
on_button_font_set_add_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *combo_entry_font_family;
	GtkWidget *combo_entry_generic_family;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *strgf;
	gchar *strff;

	combo_entry_font_family =
		lookup_widget (GTK_WIDGET (button),
			       "combo_entry_font_family");
	combo_entry_generic_family = lookup_widget (GTK_WIDGET (button),
						    "combo_entry_generic_family");

	strgf = gtk_editable_get_chars (GTK_EDITABLE
					(combo_entry_font_family), 0, -1);
	strff = gtk_editable_get_chars (GTK_EDITABLE
					(combo_entry_generic_family), 0, -1);

	store = (GtkListStore *) user_data;
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, 0, strgf, 1, strff, -1);

	g_free (strgf);
	g_free (strff);
}


void
on_button_font_set_del_clicked (GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}


/* This needs three callbacks                                                
 * ************************************************************************* */
void
on_font_family_dialog_okbutton_clicked
	(GtkButton * button,gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *font_family_dialog;
	GtkWidget *radiobutton_system_fonts;
	GtkWidget *combo_entry_system_fonts;	
	GtkWidget *checkbutton_font_family;
	GtkWidget *combo_entry_font_family;
	GtkWidget *checkbutton_generic_family;
	GtkWidget *combo_entry_generic_family;
	GtkWidget *checkbutton_font_set;
	GtkWidget *treeview;	
	GtkTreeModel *list_store;
	GtkTreeIter iter;
	gboolean valid;	
	gboolean bsystemfonts;
	gboolean bfontfamily;
	gboolean bgenericfamily;
	gboolean bfontset;	
	gchar* str_set_font;
	gchar* str_set_generic;	
	gchar* str_font_family;
	gchar* str_gen_family;
		
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	font_family_dialog = lookup_widget(GTK_WIDGET(button),"font_family_dialog");
	radiobutton_system_fonts = lookup_widget(GTK_WIDGET(button),"radiobutton_system_fonts");		
	checkbutton_font_family = lookup_widget(GTK_WIDGET(button),"checkbutton_font_family");	
	checkbutton_generic_family = lookup_widget(GTK_WIDGET(button),"checkbutton_generic_family");
	checkbutton_font_set = lookup_widget(GTK_WIDGET(button),"checkbutton_font_set");
	
	bsystemfonts = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton_system_fonts));	
	bfontfamily = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_font_family));
	bgenericfamily = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_generic_family));
	bfontset = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_font_set));	
	
	if( bsystemfonts ){
		combo_entry_system_fonts = lookup_widget(GTK_WIDGET(button),"combo_entry_system_fonts");
		str_font_family = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_system_fonts),0,-1);
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
		document_add_text (tmpdoc, str_font_family);
		document_add_text (tmpdoc, ";\n\t");
		g_free(str_font_family);
	}else{
		if( bfontset ){
			treeview = lookup_widget (GTK_WIDGET (button), "treeview");
			list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
			valid = gtk_tree_model_get_iter_first (list_store, &iter);

			if (valid)
			{	// there are values in the store
				// parser the list
				if( add_previous ){
					document_add_text (tmpdoc, (gchar *) data->previous_string);
					document_add_text (tmpdoc, ": ");
				}
				while (valid)
				{
					gtk_tree_model_get (list_store, &iter,
								0, &str_set_font, 1, &str_set_generic, -1);
		
					if( strlen(str_set_font) > 0){
						document_add_text (tmpdoc, "\"");
						document_add_text (tmpdoc, str_set_font);
						document_add_text (tmpdoc, "\"");
					}
					
					if( ( strlen(str_set_generic) > 0 ) && 
						( strlen(str_set_font)    > 0 )  ){
						document_add_text (tmpdoc, " , ");
					}
					
					if( strlen(str_set_generic) > 0){
						document_add_text (tmpdoc, str_set_generic);
					}					
					
					g_free (str_set_font);		
					g_free (str_set_generic);
					valid = gtk_tree_model_iter_next (list_store, &iter);
					if( valid ){
						document_add_text (tmpdoc, " , ");
					}else{
						document_add_text (tmpdoc, " ");
					}
				}
				if( add_previous ){
					document_add_text (tmpdoc, ";\n\t");
				}
			}else { // not valid nothing in the list
				document_write_error_to_program_output( tmpdoc,"font family dialog: rejecting to add empty values.");
			}
		}else {		
			if( bfontfamily || bgenericfamily){
				combo_entry_font_family = lookup_widget(GTK_WIDGET(button),"combo_entry_font_family");
				combo_entry_generic_family = lookup_widget(GTK_WIDGET(button),"combo_entry_generic_family");
				str_font_family = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_font_family),0,-1);
				str_gen_family = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_generic_family),0,-1);	
				if( add_previous ){
					document_add_text (tmpdoc, (gchar *) data->previous_string);
					document_add_text (tmpdoc, ":");
				}
				if( bfontfamily ){
					document_add_text (tmpdoc, "\"");
					document_add_text (tmpdoc, str_font_family);
					document_add_text (tmpdoc, "\"");
				}
				if( bfontfamily && bgenericfamily){
					document_add_text (tmpdoc, " , ");
				}
				if( bgenericfamily ){
					document_add_text (tmpdoc, str_gen_family);
				}
				if( add_previous ){
					document_add_text (tmpdoc, ";\n\t");	
				}	
				g_free(str_font_family);
				g_free(str_gen_family);
			}else{
				document_write_error_to_program_output( tmpdoc,"font family dialog: rejecting to add empty values.");
			}
		}
	}	
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(font_family_dialog);
}

void on_button_font_family_dialog_add_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkWidget *combo_entry_font_family;
	GtkWidget *combo_entry_generic_family;
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *strgf;
	gchar *strff;

	combo_entry_font_family =
		lookup_widget (GTK_WIDGET (button),
			       "combo_entry_font_family");
	combo_entry_generic_family = lookup_widget (GTK_WIDGET (button),
						    "combo_entry_generic_family");

	strgf = gtk_editable_get_chars (GTK_EDITABLE
					(combo_entry_font_family), 0, -1);
	strff = gtk_editable_get_chars (GTK_EDITABLE
					(combo_entry_generic_family), 0, -1);

	store = (GtkListStore *) user_data;
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, 0, strgf, 1, strff, -1);

	g_free (strgf);
	g_free (strff);
}


void on_button_font_family_dialog_del_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}
void 
on_button_font_fam_dialog_set_toggled
	(GtkToggleButton *togglebutton,  gpointer user_data)
{
	GtkWidget* radiobutton;
	gboolean isactive;
	
	radiobutton = GTK_WIDGET(user_data);
	isactive = gtk_toggle_button_get_active(togglebutton);
	if( isactive )
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiobutton), TRUE);
}
void
on_font_size_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *font_size_dialog;
	GtkWidget *combo_entry_font_size;
	GtkWidget *spinbutton_font_size;
	GtkWidget *combo_entry_font_size_units;
	gchar* strsize;
	gchar* strformated;
	gchar* strunit;
	gint nsize;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	font_size_dialog = lookup_widget(GTK_WIDGET(button),"font_size_dialog");
	
	combo_entry_font_size = lookup_widget(GTK_WIDGET(button),"combo_entry_font_size");	
	strsize = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_font_size),0,-1);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	if( strcmp("use percentage or length",strsize) == 0 ){
		spinbutton_font_size = lookup_widget(GTK_WIDGET(button),"spinbutton_font_size");
		combo_entry_font_size_units = lookup_widget(GTK_WIDGET(button),"combo_entry_font_size_units");
		strunit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_font_size_units),0,-1);
		nsize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_font_size));
		strformated = g_strdup_printf("%d%s",nsize,strunit);
		document_add_text (tmpdoc, strformated);
		g_free(strunit);
		g_free(strformated);
	}else{
		document_add_text (tmpdoc, strsize);
	}
	if( add_previous ){
		document_add_text (tmpdoc, " ;\n\t");
	}	
	g_free(strsize);	
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(font_size_dialog);
}

void on_frecuency_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *frecuency_property_dialog;
	GtkWidget *spinbutton_frecuency;
	GtkWidget *combo_entry_frecuency;
	gint nfreq;
	gchar* strfrequency;
	gchar* unit;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	frecuency_property_dialog = lookup_widget(GTK_WIDGET(button),"frecuency_property_dialog");
	spinbutton_frecuency = lookup_widget(GTK_WIDGET(button),"spinbutton_frecuency");
	combo_entry_frecuency = lookup_widget(GTK_WIDGET(button),"combo_entry_frecuency");
	
	unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_frecuency),0,-1);
	nfreq = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
				(spinbutton_frecuency));
				
	strfrequency = g_strdup_printf("%d%s",nfreq,unit);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	document_add_text (tmpdoc,strfrequency);
	if( add_previous ){
		document_add_text (tmpdoc,";\n\t");
	}
	g_free(strfrequency);
	g_free(unit);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(frecuency_property_dialog);
}


void on_identifier_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *identifier_value_dialog;
	GtkWidget *entry_identifier;
	gchar* identifier;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	identifier_value_dialog = lookup_widget(GTK_WIDGET(button),"identifier_value_dialog");
	entry_identifier = lookup_widget(GTK_WIDGET(button),"entry_identifier");
	identifier = gtk_editable_get_chars(GTK_EDITABLE(entry_identifier),0,-1);
	
	if( strlen(identifier) > 0){
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		document_add_text (tmpdoc, identifier);
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(identifier_value_dialog);
	}
	g_free(	identifier );
}
void on_integer_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *integer_value_dialog;
	GtkWidget *spinbutton_integer;
	gchar* integer;
	gint ninteger;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	integer_value_dialog = lookup_widget(GTK_WIDGET(button),"integer_value_dialog");
	spinbutton_integer = lookup_widget(GTK_WIDGET(button),"spinbutton_integer");	
	ninteger = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_integer));
	if( add_previous ){
		integer = g_strdup_printf("%s: %d;\n\t", (gchar *) data->previous_string,
				ninteger );
	}else{
		integer = g_strdup_printf(" %d", ninteger );
	}
	document_add_text (tmpdoc,integer);
	g_free (integer);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(integer_value_dialog);
}

void on_length_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *length_value_dialog;
	GtkWidget *spinbutton_length;
	GtkWidget *combo_entry_length;
	gchar* strlength;
	gchar* strunit;
	gint nlength;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	length_value_dialog = lookup_widget(GTK_WIDGET(button),"length_value_dialog");
	spinbutton_length = lookup_widget(GTK_WIDGET(button),"spinbutton_length");
	combo_entry_length = lookup_widget(GTK_WIDGET(button),"combo_entry_length");
	strunit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_length),0,-1);
	nlength = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_length));
	
	if( add_previous ){
		strlength = g_strdup_printf("%s: %d%s;\n\t",(gchar *) data->previous_string,
					nlength,strunit);
	}else{
		strlength = g_strdup_printf(" %d%s", nlength,strunit);
	}		
	
	document_add_text (tmpdoc, strlength);
	g_free (strunit);
	g_free (strlength);	
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(length_value_dialog);
}

void
on_list_style_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *list_style_dialog;
	GtkWidget *checkbutton_type;
	GtkWidget *combo_entry_type;
	GtkWidget *checkbutton_position;
	GtkWidget *combo_entry_position;
	GtkWidget *checkbutton_image;
	GtkWidget *entry_image;
	gboolean bpos;
	gboolean btype;
	gboolean bimage;
	gchar* strytype;
	gchar* strpos;
	gchar* strimage;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	list_style_dialog = lookup_widget(GTK_WIDGET(button),"list_style_dialog");
	checkbutton_type = lookup_widget(GTK_WIDGET(button),"checkbutton_type");	
	checkbutton_position = lookup_widget(GTK_WIDGET(button),"checkbutton_position"); 
	checkbutton_image = lookup_widget(GTK_WIDGET(button),"checkbutton_image");
	
	btype = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_type));
	bpos = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_position));
	bimage = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_image));

	if( btype || bpos || bimage ){
		if( btype || bpos ){
			if( add_previous ){
				document_add_text (tmpdoc, (gchar *) data->previous_string);
				document_add_text (tmpdoc, ": ");
			}
			if( btype ){
				combo_entry_type = lookup_widget(GTK_WIDGET(button),"combo_entry_type");
				strytype = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_type),
							0,-1);
				document_add_text (tmpdoc, strytype);
				g_free (strytype);
			}
			
			if( bpos ){
				combo_entry_position = lookup_widget(GTK_WIDGET(button),"combo_entry_position");
				strpos = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_position),
							0,-1);
				if( btype ) document_add_text (tmpdoc, " ");
				document_add_text (tmpdoc, strpos);
				g_free (strpos);
			}
			if(!bimage && add_previous){
				 document_add_text (tmpdoc,";\n\t");
			}			
		}
		if( bimage ){
			entry_image = lookup_widget(GTK_WIDGET(button),"entry_image");
			strimage = gtk_editable_get_chars(GTK_EDITABLE(entry_image),
							0,-1);
			if( strlen(strimage) > 0) {
				if( !btype && !bpos ){
					if( add_previous ){
						document_add_text (tmpdoc, (gchar *) data->previous_string);
						document_add_text (tmpdoc, ":");
					}
					document_add_text (tmpdoc, " url(");
					document_add_text (tmpdoc, strimage);
					document_add_text (tmpdoc, ")");
					if( add_previous ){
						document_add_text (tmpdoc,";\n\t");
					}
				}else{
					document_add_text (tmpdoc, "url(");
					document_add_text (tmpdoc, strimage);
					document_add_text (tmpdoc, ")");
					if( add_previous ){
						document_add_text (tmpdoc,";\n\t");
					}					
				}
			}else{
				if( ( btype || bpos ) && add_previous ){
					document_add_text (tmpdoc, ";\n\t");
				}
				document_write_error_to_program_output( tmpdoc,"list style dialog: image is selected but empty, ignoring ...");
			}
			g_free (strimage);
		}
		
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(list_style_dialog);		
		return;
	}
	document_write_error_to_program_output( tmpdoc,"list style dialog: rejecting to add empty values");
}

void on_margin_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *margin_property_dialog;
	GtkWidget *checkbutton_top;
	GtkWidget *spinbutton_top;
	GtkWidget *combo_entry_top;
	GtkWidget *checkbutton_right;
	GtkWidget *spinbutton_right;
	GtkWidget *combo_entry_right;
	GtkWidget *checkbutton_bottom;
	GtkWidget *spinbutton_bottom;
	GtkWidget *combo_entry_bottom;
	GtkWidget *checkbutton_left;
	GtkWidget *spinbutton_left;
	GtkWidget *combo_entry_left;
	gboolean b_top;
	gboolean b_right;
	gboolean b_bottom;
	gboolean b_left;
	gchar* strtop;
	gchar* strtop_unit;
	gint ntop;
	gchar* strright;
	gchar* strright_unit;	
	gint nright;
	gchar* strbottom;
	gchar* strbottom_unit;	
	gint nbottom;
	gchar* strleft;
	gchar* strleft_unit;	
	gint nleft;
	gint count;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	margin_property_dialog = lookup_widget(GTK_WIDGET(button),"margin_property_dialog");
	
	checkbutton_top = lookup_widget(GTK_WIDGET(button),"checkbutton_top");
	checkbutton_right = lookup_widget(GTK_WIDGET(button),"checkbutton_right");
	checkbutton_bottom = lookup_widget(GTK_WIDGET(button),"checkbutton_bottom");
	checkbutton_left = lookup_widget(GTK_WIDGET(button),"checkbutton_left");
	
	b_top = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_top));
	b_right = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_right));
	b_bottom = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_bottom));
	b_left = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_left));
	
	
	if( !b_top && !b_right && !b_bottom && !b_left ){
		document_write_error_to_program_output(tmpdoc,"margin property dialog: rejecting to add empty values.");
	}else{
		count = 0;
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		if( b_top ){
			count++;
			spinbutton_top = lookup_widget(GTK_WIDGET(button),"spinbutton_top");
			combo_entry_top = lookup_widget(GTK_WIDGET(button),"combo_entry_top");
			ntop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_top));
			strtop_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_top),0,-1);
			strtop = g_strdup_printf("%d%s ",ntop,strtop_unit);
			document_add_text (tmpdoc, strtop);
			g_free(strtop);
			g_free(strtop_unit);			
		}

		if( b_right ){
			count++;
			spinbutton_right = lookup_widget(GTK_WIDGET(button),"spinbutton_right");
			combo_entry_right = lookup_widget(GTK_WIDGET(button),"combo_entry_right");
			nright = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_right));
			strright_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_right),0,-1);	
			strright = g_strdup_printf("%d%s ",nright,strright_unit);
			document_add_text (tmpdoc, strright);
			g_free(strright);
			g_free(strright_unit);			
		}
		if( b_bottom ){
			count++;
			spinbutton_bottom = lookup_widget(GTK_WIDGET(button),"spinbutton_bottom");
			combo_entry_bottom = lookup_widget(GTK_WIDGET(button),"combo_entry_bottom");
			nbottom =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_bottom));
			strbottom_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_bottom),0,-1);	
			strbottom = g_strdup_printf("%d%s ",nbottom,strbottom_unit);
			document_add_text (tmpdoc, strbottom);
			g_free(strbottom);
			g_free(strbottom_unit);			
		}
		if( b_left ){
			count++;
			spinbutton_left = lookup_widget(GTK_WIDGET(button),"spinbutton_left");
			combo_entry_left = lookup_widget(GTK_WIDGET(button),"combo_entry_left");
			nleft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_left));
			strleft_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_left),0,-1);
			strleft = g_strdup_printf("%d%s ",nleft,strleft_unit);
			document_add_text (tmpdoc, strleft);
			g_free(strleft);
			g_free(strleft_unit);			
		}	
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		if( count == 1){
			document_write_error_to_program_output(tmpdoc,"margin: only one value used. will be applied to all margins.");
		}else if( count == 2 ){
			document_write_error_to_program_output(tmpdoc,"margin: two values used. first will be applied to top/bottom, second to left/right.");
		}else if(count == 3 ){
			document_write_error_to_program_output(tmpdoc,"margin: three values used. first will be used to top, second to left/right and third to bottom.");
		}		
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(margin_property_dialog);
	}
}


void on_margin_width_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *margin_width_value_dialog;
	GtkWidget *spinbutton_margin_width;
	GtkWidget *combo_entry_margin_width_units;
	gchar* strwidth;
	gchar* strunit;
	gint nwidth;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	margin_width_value_dialog = lookup_widget(GTK_WIDGET(button), "margin_width_value_dialog");
	spinbutton_margin_width = lookup_widget(GTK_WIDGET(button), "spinbutton_margin_width");
	combo_entry_margin_width_units = lookup_widget(GTK_WIDGET(button), "combo_entry_margin_width_units");	

	nwidth = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_margin_width));
	strunit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_margin_width_units),0,-1);
	strwidth = g_strdup_printf("%d%s",nwidth,strunit);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}	
	document_add_text (tmpdoc, strwidth);
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}	
	g_free (strwidth);
	g_free (strunit);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(margin_width_value_dialog);
}


void on_number_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *number_value_dialog;
	GtkWidget *spinbutton_number;
	gfloat number;
	gchar* strnumber;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	number_value_dialog = lookup_widget(GTK_WIDGET(button),"number_value_dialog");
	spinbutton_number = lookup_widget(GTK_WIDGET(button),"spinbutton_number");
	
	number = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_number));
	if( add_previous ){
		strnumber = g_strdup_printf("%s: %0.2f;\n\t",(gchar *) data->previous_string,
					number);
	}else{
		strnumber = g_strdup_printf(" %0.2f", number);
	}
	document_add_text (tmpdoc, strnumber);
	
	g_free(strnumber);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(number_value_dialog);
}

void on_padding_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *padding_property_dialog;
	GtkWidget *checkbutton_top;
	GtkWidget *spinbutton_top;
	GtkWidget *combo_entry_top;
	GtkWidget *checkbutton_left;
	GtkWidget *spinbutton_left;
	GtkWidget *combo_entry_left;
	GtkWidget *checkbutton_bottom;
	GtkWidget *spinbutton_bottom;
	GtkWidget *combo_entry_bottom;
	GtkWidget *checkbutton_right;
	GtkWidget *spinbutton_right;
	GtkWidget *combo_entry_right;
	gboolean b_top;
	gboolean b_left;
	gboolean b_bottom;
	gboolean b_right;
	gint ntop;
	gint nleft;
	gint nbottom;
	gint nright;
	gchar* strtop;
	gchar* strleft;
	gchar* strbottom;
	gchar* strright;
	gchar* strtop_unit;
	gchar* strleft_unit;
	gchar* strbottom_unit;
	gchar* strright_unit;	
	gint count;
		
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	padding_property_dialog = lookup_widget(GTK_WIDGET(button), "padding_property_dialog");
	
	checkbutton_top = lookup_widget(GTK_WIDGET(button), "checkbutton_top");
	checkbutton_left = lookup_widget(GTK_WIDGET(button), "checkbutton_left");
	checkbutton_bottom = lookup_widget(GTK_WIDGET(button), "checkbutton_bottom");
	checkbutton_right = lookup_widget(GTK_WIDGET(button), "checkbutton_right");

	b_top = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_top));
	b_left = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_left));
	b_bottom = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_bottom));
	b_right = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_right));	
	
	if( !b_top && !b_right && !b_bottom && !b_left ){
		document_write_error_to_program_output(tmpdoc,_("padding property dialog: no values in dialog, defaulting to cero.") );
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		document_add_text (tmpdoc, "0");
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
	}else{
		count = 0;
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		if( b_top ){
			count++;
			spinbutton_top  = lookup_widget(GTK_WIDGET(button), "spinbutton_top");
			combo_entry_top = lookup_widget(GTK_WIDGET(button), "combo_entry_top");
			ntop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_top));
			strtop_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_top),0,-1);
			strtop = g_strdup_printf("%d%s ",ntop,strtop_unit);
			document_add_text (tmpdoc, strtop);
			g_free(strtop);
			g_free(strtop_unit);			
		}
		if( b_right ){
			count++;
			spinbutton_right = lookup_widget(GTK_WIDGET(button),"spinbutton_right");
			combo_entry_right = lookup_widget(GTK_WIDGET(button),"combo_entry_right");
			nright = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_right));
			strright_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_right),0,-1);	
			strright = g_strdup_printf("%d%s ",nright,strright_unit);
			document_add_text (tmpdoc, strright);
			g_free(strright);
			g_free(strright_unit);			
		}
		if( b_bottom ){
			count++;
			spinbutton_bottom = lookup_widget(GTK_WIDGET(button),"spinbutton_bottom");
			combo_entry_bottom = lookup_widget(GTK_WIDGET(button),"combo_entry_bottom");
			nbottom =  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_bottom));
			strbottom_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_bottom),0,-1);	
			strbottom = g_strdup_printf("%d%s ",nbottom,strbottom_unit);
			document_add_text (tmpdoc, strbottom);
			g_free(strbottom);
			g_free(strbottom_unit);			
		}
		if( b_left ){
			count++;
			spinbutton_left = lookup_widget(GTK_WIDGET(button),"spinbutton_left");
			combo_entry_left = lookup_widget(GTK_WIDGET(button),"combo_entry_left");
			nleft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_left));
			strleft_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_left),0,-1);
			strleft = g_strdup_printf("%d%s ",nleft,strleft_unit);
			document_add_text (tmpdoc, strleft);
			g_free(strleft);
			g_free(strleft_unit);			
		}	
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		if( count == 1){
			document_write_error_to_program_output(tmpdoc,"padding: only one value used. will be applied to all sides.");
		}else if( count == 2 ){
			document_write_error_to_program_output(tmpdoc,"padding: two values used. first will be applied to top/bottom, second to left/right.");
		}else if(count == 3 ){
			document_write_error_to_program_output(tmpdoc,"padding: three values used. first will be used to top, second to left/right and third to bottom.");
		}		
	}
	
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(padding_property_dialog);
}
void on_outline_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *outline_property_dialog;
	GtkWidget *checkbutton_length;
	GtkWidget *combo_entry_length_selector;
	GtkWidget *spinbutton_length;
	GtkWidget *combo_entry_length_units;
	GtkWidget *checkbutton_style;
	GtkWidget *combo_entry_style;
	GtkWidget *checkbutton_color;
	GtkWidget *combo_entry_color;
	GtkWidget *colorselection;	
	gboolean b_length;
	gboolean b_style;
	gboolean b_color;
	gint nlength;
	gchar* strlength_units;
	gchar* strlength;
	gchar* strlength_and_unit;	
	gchar* strstyle;	
	gchar* strcolor;
	gchar* gdk_color_to_string;
	GdkColor color; 
		
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	outline_property_dialog = lookup_widget(GTK_WIDGET(button),"outline_property_dialog");
	checkbutton_length  = lookup_widget(GTK_WIDGET(button),"checkbutton_length");
	checkbutton_style = lookup_widget(GTK_WIDGET(button),"checkbutton_style");
	checkbutton_color = lookup_widget(GTK_WIDGET(button),"checkbutton_color");
	
	b_length = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_length)); 
	b_style = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_style));
	b_color = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_color));	
	
	if ( !b_length && !b_style && !b_color ){
		document_write_error_to_program_output(tmpdoc,"outline dialog: rejecting to add empty values.\n");
	}else{		
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		if( b_length ){
			combo_entry_length_selector = lookup_widget(GTK_WIDGET(button),"combo_entry_length_selector") ;	
			strlength = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_length_selector),
								0,-1);
			if( strcmp(strlength,"use percentage or length") == 0){
				spinbutton_length  = lookup_widget(GTK_WIDGET(button),"spinbutton_length");
				combo_entry_length_units = lookup_widget(GTK_WIDGET(button),"combo_entry_length_units");
				nlength = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_length));
				strlength_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_length_units),
								0,-1);
				strlength_and_unit = g_strdup_printf("%d%s",nlength,strlength_units);
				document_add_text(tmpdoc,strlength_and_unit);
				g_free(strlength_units);
				g_free(strlength_and_unit);
			}else{
				document_add_text(tmpdoc,strlength);
			}			
			g_free(strlength);
			if( b_style || b_color ){
				document_add_text(tmpdoc," ");
			}			
		}
		if( b_style ){
			combo_entry_style = lookup_widget(GTK_WIDGET(button),"combo_entry_style");	
			strstyle = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_style),0,-1);
			document_add_text(tmpdoc,strstyle);
			if( b_color ){
				document_add_text(tmpdoc, " ");
			}
			g_free(strstyle);
		}		
		if( b_color ){
			combo_entry_color = lookup_widget(GTK_WIDGET(button),"combo_entry_color");
			strcolor = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_color),0,-1);
			if( strcmp(strcolor,"from color selector") == 0 ){
				colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");
				gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(colorselection),&color);
				gdk_color_to_string = gdk_color_to_rgb_triplet_string(&color);
				document_add_text(tmpdoc, gdk_color_to_string );
				g_free(gdk_color_to_string);
			}else{
				document_add_text(tmpdoc,strcolor);
			}		
			g_free(strcolor);
		}		
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(outline_property_dialog);
	}
}
void on_pause_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *pause_property_dialog;
	GtkWidget *checkbutton_after;
	GtkWidget *spinbutton_after;
	GtkWidget *combo_entry_after;
	GtkWidget *checkbutton_before;
	GtkWidget *spinbutton_before;
	GtkWidget *combo_entry_before;
	gboolean b_after;
	gboolean b_before;
	gchar* strafter;
	gchar* strafter_unit;
	gint nafter;
	gchar* strbefore;
	gchar* strbefore_unit;
	gint nbefore;	
	
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	pause_property_dialog = lookup_widget(GTK_WIDGET(button),"pause_property_dialog");
	checkbutton_after = lookup_widget(GTK_WIDGET(button),"checkbutton_after");
	checkbutton_before = lookup_widget(GTK_WIDGET(button),"checkbutton_before");
	
	b_after = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_after));
	b_before = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_before));
	
	if(!b_after && !b_before){
		document_write_error_to_program_output(tmpdoc,"pause dialog: rejecting to add empty values");
	}else{	
		if( b_after ){
			spinbutton_after = lookup_widget(GTK_WIDGET(button),"spinbutton_after");
			combo_entry_after = lookup_widget(GTK_WIDGET(button),"combo_entry_after");
			strafter_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_after),0,-1);
			nafter = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_after));
			strafter = g_strdup_printf("%d%s",nafter,strafter_unit);
			document_add_text(tmpdoc,strafter);
			if( b_before ){
				document_add_text(tmpdoc," ");
			}
			g_free(strafter_unit);
			g_free(strafter);
		}
		if( b_before ){
			if( !b_after ){
				document_add_text(tmpdoc," 0ms ");
				document_write_error_to_program_output(tmpdoc,"pause dialog: only pause before selected, defaulting pause-after to 0 miliseconds.");
			}
			spinbutton_before = lookup_widget(GTK_WIDGET(button),"spinbutton_before");
			combo_entry_before = lookup_widget(GTK_WIDGET(button),"combo_entry_before");			
			strbefore_unit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_before),0,-1);
			nbefore = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_before));
			strbefore = g_strdup_printf("%d%s",nbefore,strbefore_unit);
			document_add_text(tmpdoc,strbefore);
			g_free(strbefore_unit);
			g_free(strbefore);
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(pause_property_dialog);
	}
}

void on_percentage_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *percentage_value_dialog;
	GtkWidget *spinbutton_percentage;
	gchar* strpercentage;
	gint npercentage;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	percentage_value_dialog = lookup_widget(GTK_WIDGET(button),"percentage_value_dialog");
	spinbutton_percentage = lookup_widget(GTK_WIDGET(button),"spinbutton_percentage");
	npercentage = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_percentage));
	strpercentage = g_strdup_printf(" %d",npercentage);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}	
	document_add_text (tmpdoc, strpercentage);
	document_add_text (tmpdoc, "%");	
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	
	g_free (strpercentage);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(percentage_value_dialog);	
}
void on_percentage_length_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *percentage_length_value_dialog;
	GtkWidget *spinbutton_perclen;
	GtkWidget *combo_entry_perclen_units;
	gchar* strpercentage;
	gchar* strunit;
	gint npercentage;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	percentage_length_value_dialog = lookup_widget(GTK_WIDGET(button),"percentage_length_value_dialog");
	spinbutton_perclen = lookup_widget(GTK_WIDGET(button),"spinbutton_perclen");
	combo_entry_perclen_units = lookup_widget(GTK_WIDGET(button),"combo_entry_perclen_units");
	
	npercentage = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_perclen));
	strunit = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_perclen_units),0,-1);
	strpercentage = g_strdup_printf(" %d%s",npercentage,strunit);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}	
	
	document_add_text (tmpdoc, strpercentage);
	
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	
	g_free (strpercentage);
	g_free (strunit);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(percentage_length_value_dialog);
}

void on_play_during_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *play_during_property_dialog;
	GtkWidget *entry_uri;
	GtkWidget *checkbutton_repeat;
	GtkWidget *checkbutton_mix;	
	gchar* struri;
	gboolean b_repeat;
	gboolean b_mix;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	play_during_property_dialog = lookup_widget(GTK_WIDGET(button),"play_during_property_dialog");
	entry_uri = lookup_widget(GTK_WIDGET(button),"entry_uri");
	
	struri = gtk_editable_get_chars(GTK_EDITABLE(entry_uri),0,-1);
	
	if( strlen(struri) > 0){		
		checkbutton_repeat = lookup_widget(GTK_WIDGET(button),"checkbutton_repeat");
		checkbutton_mix = lookup_widget(GTK_WIDGET(button),"checkbutton_mix");
		b_repeat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_repeat));
		b_mix = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_mix));
		
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ":");
		}	
		document_add_text (tmpdoc, " url( ");
		document_add_text (tmpdoc,struri);
		document_add_text (tmpdoc, " )");		
		
		if( b_repeat ) document_add_text (tmpdoc, " repeat");
		if( b_mix ) document_add_text (tmpdoc, " mix");
			
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(play_during_property_dialog);		
	}else{
		document_write_error_to_program_output(tmpdoc,"play during dialog: value \"uri\" required, rejecting to add invalid values");
	}	
	g_free(struri);
}
/* this needs three callbacks */
void on_button_quotes_property_dialog_add_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkWidget *entry_open;
	GtkWidget *entry_close;
	
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *stroq;
	gchar *strcq;
	
	store = (GtkListStore *) user_data;
	
	entry_open =
		lookup_widget (GTK_WIDGET (button),
			       "entry_open");
	entry_close = lookup_widget (GTK_WIDGET (button),
						    "entry_close");

	stroq = gtk_editable_get_chars (GTK_EDITABLE
					(entry_open), 0, -1);
	strcq = gtk_editable_get_chars (GTK_EDITABLE
					(entry_close), 0, -1);

	if( (strlen(stroq) >0 ) && (strlen(strcq)>0) ){
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, stroq, 1, strcq, -1);
	}else{ 	// this will default to the same open and close quotes if only 
		   	// one value is added. nothing will be added to the store if both
			// values are empty.
		if(	strlen(stroq) >0 ){
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, stroq, 1, stroq, -1);
		}			
		if(	strlen(strcq) >0 ){
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, strcq, 1, strcq, -1);
		}
	}

	g_free (stroq);
	g_free (strcq);
}


void on_button_quotes_property_dialog_del_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}


void on_quotes_property_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *treeview;	
	GtkTreeModel *list_store;
	GtkTreeIter iter;
	GtkWidget *quotes_property_dialog;
	gboolean valid;	
	gchar* stropen;
	gchar* strclose;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	quotes_property_dialog = lookup_widget (GTK_WIDGET (button), "quotes_property_dialog");
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	valid = gtk_tree_model_get_iter_first (list_store, &iter);

	if (valid)
	{	// there are values in the store
		// parse the list
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		while (valid)
		{
			gtk_tree_model_get (list_store, &iter,
						0, &stropen, 1, &strclose, -1);

			// the callback on_button_quotes_property_dialog_add_clicked()
			// ensures that no empty values can be passed.
			if( strcmp(stropen,"\"") == 0 ){
				document_add_text (tmpdoc, " '");
				document_add_text (tmpdoc, stropen);
				document_add_text (tmpdoc, "'");
			}else{
				document_add_text (tmpdoc, " \"");
				document_add_text (tmpdoc, stropen);
				document_add_text (tmpdoc, "\"");					
			}
		

			if( strcmp(strclose,"\"") == 0 ){
				document_add_text (tmpdoc, " '");
				document_add_text (tmpdoc, strclose);
				document_add_text (tmpdoc, "'");
			}else{
				document_add_text (tmpdoc, " \"");
				document_add_text (tmpdoc, strclose);
				document_add_text (tmpdoc, "\"");					
			}
	
			
			g_free (stropen);		
			g_free (strclose);
			valid = gtk_tree_model_iter_next (list_store, &iter);
			if( valid ){
				document_add_text (tmpdoc, " , ");
			}
		}		
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		
	}else { // not valid nothing in the list
		document_write_error_to_program_output( tmpdoc,"quotes dialog: nothing in the list, rejecting to add empty values.");
	}
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(quotes_property_dialog);
}


/* end quotes */
void
on_shape_value_dialog_okbutton_clicked (GtkButton * button,
					gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *shape_value_dialog;
	GtkWidget *spinbutton_top;
	GtkWidget *combo_entry_top;
	GtkWidget *spinbutton_right;
	GtkWidget *combo_entry_right;
	GtkWidget *spinbutton_bottom;
	GtkWidget *combo_entry_bottom;
	GtkWidget *spinbutton_left;
	GtkWidget *combo_entry_left;
	gchar* strshape;
	gchar* str_top_units;
	gchar* str_left_units;
	gchar* str_bottom_units;
	gchar* str_right_units;
	gint ntop;
	gint nleft;
	gint nbottom;
	gint nright;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	shape_value_dialog = lookup_widget(GTK_WIDGET(button),"shape_value_dialog");
	spinbutton_top = lookup_widget(GTK_WIDGET(button),"spinbutton_top");
	combo_entry_top = lookup_widget(GTK_WIDGET(button),"combo_entry_top");
	spinbutton_right = lookup_widget(GTK_WIDGET(button),"spinbutton_right");
	combo_entry_right = lookup_widget(GTK_WIDGET(button),"combo_entry_right");
	spinbutton_bottom = lookup_widget(GTK_WIDGET(button),"spinbutton_bottom");
	combo_entry_bottom = lookup_widget(GTK_WIDGET(button),"combo_entry_bottom");
	spinbutton_left = lookup_widget(GTK_WIDGET(button),"spinbutton_left");
	combo_entry_left = lookup_widget(GTK_WIDGET(button),"combo_entry_left");
	
	str_top_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_top),0,-1);
	str_left_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_right),0,-1);
	str_bottom_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_bottom),0,-1);
	str_right_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_left),0,-1);
	
	ntop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_top));
	nleft = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_left));
	nbottom = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_bottom));
	nright = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_right));
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	strshape = g_strdup_printf("rect( %d%s, %d%s, %d%s, %d%s )",
			ntop,str_top_units,nright,str_right_units,
			nbottom,str_bottom_units,nleft,str_left_units);
	
	document_add_text (tmpdoc, strshape);
	
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}
	
	g_free (strshape);	
	g_free (str_top_units);
	g_free (str_left_units);
	g_free (str_bottom_units);
	g_free (str_right_units);

	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(shape_value_dialog);
}



void on_size_length_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *size_length_value_dialog;
	GtkWidget *spinbutton_width;
	GtkWidget *combo_entry_width;
	GtkWidget *spinbutton_height;
	GtkWidget *combo_entry_height;
	gchar* strsize;
	gchar* strwidth_units;
	gchar* strheight_units;	
	gfloat nwidth;
	gfloat nheight;

	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	size_length_value_dialog = lookup_widget(GTK_WIDGET(button),"size_length_value_dialog");
	spinbutton_width = lookup_widget(GTK_WIDGET(button),"spinbutton_width");
	combo_entry_width = lookup_widget(GTK_WIDGET(button),"combo_entry_width");
	spinbutton_height = lookup_widget(GTK_WIDGET(button),"spinbutton_height");
	combo_entry_height = lookup_widget(GTK_WIDGET(button),"combo_entry_height");

	strwidth_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_width ),0,-1);
	strheight_units  = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_height ),0,-1) ;	
	nwidth = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_width));
	nheight = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_height));	
	
	strsize = g_strdup_printf("%0.2f%s %0.2f%s",nwidth,strwidth_units,nheight,strheight_units);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	document_add_text (tmpdoc, strsize);	
	
	if( add_previous ){
		document_add_text (tmpdoc, "\n\t");
	}
	
	g_free (strwidth_units);
	g_free (strheight_units);
	g_free (strsize);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(size_length_value_dialog);
}


/* needs three callbacks */
/* **************  I'm working here right now  ****************************** */
void on_button_text_shadow_value_dialog_add_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkTreeIter iter;
	GtkWidget *spinbutton_right;
	GtkWidget *combo_entry_right;
	GtkWidget *spinbutton_below;
	GtkWidget *combo_entry_below;	
	GtkWidget *checkbutton_use_color;	
	GtkWidget *checkbutton_blur;
	GtkWidget *spinbutton_blur;
	GtkWidget *combo_entry_blur;
	GtkWidget *combo_entry_use_color;
	GtkWidget *colorselection;	
	GtkWidget *checkbutton_use_shadow_list;
	gchar* str_right_units;
	gchar* str_right;
	gfloat nright;	
	gchar* str_below_units;
	gchar* str_below;
	gfloat nbelow;	
	gchar* str_blur_units;
	gchar* str_blur;
	gfloat nblur;	
	gboolean b_blur;	
	gchar* strcolor;
	gchar* strrgbcolor;
	GdkColor gdkcolor;
	gboolean b_color;
	
	enum {
		COL_RIGHT,
		COL_BELOW,
		COL_BLUR,
		COL_COLOR,
		N_COLS 
	};
	// if the're adding valus to the list is for using it so check the
	// use shadow list on bahalve of the user.
	checkbutton_use_shadow_list = lookup_widget(GTK_WIDGET(button),"checkbutton_use_shadow_list");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_use_shadow_list),TRUE);
	
	store = (GtkListStore *) user_data;
	
	spinbutton_right = lookup_widget(GTK_WIDGET(button),"spinbutton_right");
	combo_entry_right = lookup_widget(GTK_WIDGET(button),"combo_entry_right");
	spinbutton_below = lookup_widget(GTK_WIDGET(button),"spinbutton_below");
	combo_entry_below = lookup_widget(GTK_WIDGET(button),"combo_entry_below");	
	checkbutton_blur = lookup_widget(GTK_WIDGET(button),"checkbutton_blur");
	checkbutton_use_color = lookup_widget(GTK_WIDGET(button),"checkbutton_use_color");

	b_blur = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_blur));
	b_color = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_use_color));
	
	nright = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_right));
	str_right_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_right),0,-1);
	str_right = g_strdup_printf("%0.2f%s",nright,str_right_units);
	
	nbelow = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_below));
	str_below_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_below),0,-1);
	str_below = g_strdup_printf("%0.2f%s",nbelow ,str_below_units);
	
	if( b_blur ){
		spinbutton_blur = lookup_widget(GTK_WIDGET(button),"spinbutton_blur");
		combo_entry_blur = lookup_widget(GTK_WIDGET(button),"combo_entry_blur");
		nblur  = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_blur));	
		str_blur_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_blur),0,-1);
		str_blur= g_strdup_printf("%0.2f%s",nblur ,str_blur_units);
	}else{
		str_blur = g_strdup("");
	}
	
	if( b_color ){
		combo_entry_use_color = lookup_widget(GTK_WIDGET(button),"combo_entry_use_color");
		colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");	
		strcolor = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_use_color),0,-1);
		if( strcmp(strcolor,"from color selector") == 0){
			gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(colorselection),
				&gdkcolor);
			strrgbcolor = gdk_color_to_rgb_triplet_string(&gdkcolor);
		}else{
			strrgbcolor = g_strdup(strcolor);
		}
	}

	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter, COL_RIGHT, str_right, COL_BELOW, str_below, 
									COL_BLUR, str_blur, COL_COLOR, strrgbcolor,-1);
	
	g_free(strcolor);
	g_free( strrgbcolor );
	g_free( str_blur_units );
	g_free( str_blur );
	g_free( str_right_units );
	g_free( str_below_units );
	g_free( str_below );
	g_free( str_right );
}


void on_button_text_shadow_value_dialog_del_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}


void on_text_shadow_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkTreeModel *list_store;
	GtkTreeIter iter;
	GtkWidget *text_shadow_value_dialog;
	GtkWidget *spinbutton_right;
	GtkWidget *combo_entry_right;
	GtkWidget *spinbutton_below;
	GtkWidget *combo_entry_below;
	GtkWidget *checkbutton_blur;
	GtkWidget *spinbutton_blur;
	GtkWidget *combo_entry_blur;
	GtkWidget *checkbutton_use_color;
	GtkWidget *combo_entry_use_color;
	GtkWidget *colorselection;
	GtkWidget *checkbutton_use_shadow_list;
	GtkWidget *treeview;
	gboolean b_use_list;
	gchar* str_right_units;
	gchar* str_right;
	gfloat nright;	
	gchar* str_below_units;
	gchar* str_below;
	gfloat nbelow;	
	gchar* str_blur_units;
	gchar* str_blur;
	gfloat nblur;	
	gboolean b_blur;	
	gchar* strcolor;
	gchar* strrgbcolor;
	GdkColor gdkcolor;
	gboolean b_color;
	gboolean valid;
	
	enum {
		COL_RIGHT,
		COL_BELOW,
		COL_BLUR,
		COL_COLOR,
		N_COLS 
	};
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	text_shadow_value_dialog = lookup_widget(GTK_WIDGET(button),"text_shadow_value_dialog");
	checkbutton_use_shadow_list = lookup_widget(GTK_WIDGET(button),"checkbutton_use_shadow_list");

	b_use_list = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_use_shadow_list));
	
	if( b_use_list ){
		treeview = lookup_widget(GTK_WIDGET(button),"treeview");
		document_write_error_to_program_output(tmpdoc,"text shadow dialog: using list values, ignoring other dialog choices.");
		list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
		valid = gtk_tree_model_get_iter_first (list_store, &iter);
	
		if (valid)
		{	// there are values in the store
			// parse the list
			if( add_previous ){
				document_add_text (tmpdoc, (gchar *) data->previous_string);
				document_add_text (tmpdoc, ": ");
			}
			while (valid)
			{
				gtk_tree_model_get (list_store, &iter,
							COL_RIGHT, &str_right, 
							COL_BELOW, &str_below, 
							COL_BLUR, &str_blur, 
							COL_COLOR, &strcolor, 	-1);
				
				document_add_text(tmpdoc,str_right);
				document_add_text(tmpdoc," ");
				document_add_text(tmpdoc,str_below);
				
				if ( str_blur != NULL ){
					if(  strlen(str_blur) > 0 ){
						document_add_text(tmpdoc," ");
						document_add_text(tmpdoc,str_blur);	
					}
					g_free (str_blur);	
				}
				
				if( strcolor != NULL ){
					if( strlen(strcolor) > 0  ){
						document_add_text(tmpdoc," ");
						document_add_text(tmpdoc,strcolor);	
					}					
				}
				
				g_free (str_right);		
				g_free (str_below);
			
				valid = gtk_tree_model_iter_next (list_store, &iter);
				if( valid ){
					document_add_text (tmpdoc, " , ");
				}
			}		
			if( add_previous ){
				document_add_text (tmpdoc, ";\n\t");
			}
			
			g_free (data->previous_string);
			g_free (user_data);
			gtk_widget_destroy(text_shadow_value_dialog);			
		}else { // nothing in the list
			document_write_error_to_program_output( tmpdoc,"text shadow dialog: shadow list selected but nothing in the list. Rejecting to add empty values.");
		}		
	}else{ // don't use list
		checkbutton_blur = lookup_widget(GTK_WIDGET(button),"checkbutton_blur");	
		checkbutton_use_color = lookup_widget(GTK_WIDGET(button),"checkbutton_use_color");
		
		b_blur = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_blur));
		b_color = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_use_color));
		
		spinbutton_right = lookup_widget(GTK_WIDGET(button),"spinbutton_right");
		combo_entry_right = lookup_widget(GTK_WIDGET(button),"combo_entry_right");
		spinbutton_below = lookup_widget(GTK_WIDGET(button),"spinbutton_below");
		combo_entry_below = lookup_widget(GTK_WIDGET(button),"combo_entry_below");	
		spinbutton_blur = lookup_widget(GTK_WIDGET(button),"spinbutton_blur");
		combo_entry_blur = lookup_widget(GTK_WIDGET(button),"combo_entry_blur");	
		combo_entry_use_color = lookup_widget(GTK_WIDGET(button),"combo_entry_use_color");
		colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");
		
		nright = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_right));
		str_right_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_right),0,-1);
		str_right = g_strdup_printf("%0.2f%s",nright,str_right_units);
		
		nbelow = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_below));
		str_below_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_below),0,-1);
		str_below = g_strdup_printf("%0.2f%s",nbelow ,str_below_units);
		
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		
		document_add_text(tmpdoc,str_right);
		document_add_text(tmpdoc," ");
		document_add_text(tmpdoc,str_below);
				
		if( b_blur ){
			spinbutton_blur = lookup_widget(GTK_WIDGET(button),"spinbutton_blur");
			combo_entry_blur = lookup_widget(GTK_WIDGET(button),"combo_entry_blur");
			nblur  = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_blur));	
			str_blur_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_blur),0,-1);
			str_blur= g_strdup_printf("%0.2f%s",nblur ,str_blur_units);
			document_add_text(tmpdoc," ");
			document_add_text(tmpdoc,str_blur);
		}else{
			str_blur = g_strdup("");
		}
		
		if( b_color ){
			combo_entry_use_color = lookup_widget(GTK_WIDGET(button),"combo_entry_use_color");
			colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");	
			strcolor = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_use_color),0,-1);
			if( strcmp(strcolor,"from color selector") == 0){
				gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(colorselection),
					&gdkcolor);
				strrgbcolor = gdk_color_to_rgb_triplet_string(&gdkcolor);
			}else{
				strrgbcolor = g_strdup(strcolor);
			}
			document_add_text(tmpdoc," ");
			document_add_text(tmpdoc,strrgbcolor);
		}
		
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		
		g_free(strcolor);
		g_free( strrgbcolor );
		g_free( str_blur_units );
		g_free( str_blur );
		g_free( str_right_units );
		g_free( str_below_units );
		g_free( str_below );
		g_free( str_right );
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(text_shadow_value_dialog);
	}	
}
// end text shadow
/* needs three callbacks */
void on_button_voice_family_value_dialog_add_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkWidget *entry_specific_voice;	
	GtkWidget *combo_entry_generic;
	GtkWidget *checkbutton_voice_set;	
	GtkListStore *store;
	GtkTreeIter iter;
	gchar *str_gen;
	gchar *str_spec;
	
	enum {
		COL_GENERIC,
		COL_SPECIFIC,
		N_COLS
	};
	
	store = (GtkListStore *) user_data;
	
	entry_specific_voice =
		lookup_widget (GTK_WIDGET (button),
			       "entry_specific_voice");
	combo_entry_generic = lookup_widget (GTK_WIDGET (button),
						    "combo_entry_generic");
	
	checkbutton_voice_set = lookup_widget (GTK_WIDGET (button),
						    "checkbutton_voice_set");
	
	str_spec = gtk_editable_get_chars (GTK_EDITABLE
					(entry_specific_voice), 0, -1);
	str_gen = gtk_editable_get_chars (GTK_EDITABLE
					(combo_entry_generic), 0, -1);
	// add only if there is at least one valid value
	if( (strlen(str_spec) > 0) || (strlen(str_gen) > 0) ){ 
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, COL_GENERIC, str_gen, COL_SPECIFIC, str_spec, -1);
		// set the "use list" toggle active as they are adding values to the list
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_voice_set),TRUE);
	}
	g_free (str_spec);
	g_free (str_gen);
}
	
void on_button_voice_family_value_dialog_del_clicked
	(GtkButton * button, gpointer user_data)
{
	GtkListStore *store;
	GtkWidget *treeview;
	GtkTreeSelection *selection;
	gboolean isselected;
	GtkTreeIter iter;

	store = (GtkListStore *) user_data;
	treeview = lookup_widget (GTK_WIDGET (button), "treeview");
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
	isselected = gtk_tree_selection_get_selected (selection, NULL, &iter);

	if (isselected)
	{
		gtk_list_store_remove (store, &iter);
	}
}
		
void on_voice_family_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *voice_family_value_dialog;
	GtkWidget *combo_entry_generic;
	GtkWidget *entry_specific_voice;
	GtkWidget *checkbutton_voice_set;
	GtkWidget *treeview;
	GtkTreeModel *list_store;
	GtkTreeIter iter;
	gboolean valid;
	gboolean b_set;
	gchar* str_generic;
	gchar* str_specific;
	
	enum {
		COL_GENERIC,
		COL_SPECIFIC,
		N_COLS
	};
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	voice_family_value_dialog = lookup_widget(GTK_WIDGET(button),"voice_family_value_dialog");
	checkbutton_voice_set = lookup_widget(GTK_WIDGET(button),"checkbutton_voice_set");
	
	b_set = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_voice_set));
	
	if( b_set ){
		treeview = lookup_widget(GTK_WIDGET(button),"treeview");
		list_store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
		valid = gtk_tree_model_get_iter_first (list_store, &iter);
	
		if (valid)
		{	// there are values in the store
			// parse the list
			if( add_previous ){
				document_add_text (tmpdoc, (gchar *) data->previous_string);
				document_add_text (tmpdoc, ": ");
			}
			while (valid)
			{
				gtk_tree_model_get (list_store, &iter,
							COL_GENERIC, &str_generic, 
							COL_SPECIFIC, &str_specific, 	-1);				
				
				if ( str_specific != NULL ){
					if( strlen(str_specific) > 0  ){
						document_add_text(tmpdoc," \"");
						document_add_text(tmpdoc,str_specific);	
						document_add_text(tmpdoc,"\"");
						if( (strlen(str_generic) > 0 ) && (str_generic != NULL) ){
							document_add_text(tmpdoc,",");
						}	
					}						
				}
				
				if( str_generic != NULL ){
					if( strlen(str_generic) > 0  ){
						document_add_text(tmpdoc," ");
						document_add_text(tmpdoc,str_generic);	
					}					
				}
				
				g_free (str_generic);		
				g_free (str_specific);
			
				valid = gtk_tree_model_iter_next (list_store, &iter);
				if( valid ){
					document_add_text (tmpdoc, ",");
				}
			}		
			if( add_previous ){
				document_add_text (tmpdoc, ";\n\t");
			}
			
			g_free (data->previous_string);
			g_free (user_data);
			gtk_widget_destroy(voice_family_value_dialog);
			
		}else { // nothing in the list
			document_write_error_to_program_output( tmpdoc,"voice family dialog: voice list selected but nothing in the list. Rejecting to add empty values.");
		}	
	}else{
		combo_entry_generic = lookup_widget(GTK_WIDGET(button),"combo_entry_generic");
		entry_specific_voice = lookup_widget(GTK_WIDGET(button),"entry_specific_voice");
		
		str_generic = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_generic),0,-1);
		str_specific = gtk_editable_get_chars(GTK_EDITABLE(entry_specific_voice),0,-1);
		
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
			
		if( (  strlen(str_generic) > 0 ) || ( strlen(str_specific) > 0  ) ){
			if( add_previous ){
				document_add_text (tmpdoc, (gchar *) data->previous_string);
				document_add_text (tmpdoc, ": ");
			}			
		}else{
			document_write_error_to_program_output(tmpdoc,"voice family dialog: rejecting to add empty values.");
			g_free (str_generic);		
			g_free (str_specific);
			return;
		}		

		if( strlen(str_specific) > 0  ){
			document_add_text(tmpdoc," \"");
			document_add_text(tmpdoc,str_specific);	
			document_add_text(tmpdoc,"\"");
			
			if( (strlen(str_generic) > 0 ) && (str_generic != NULL) ){
				document_add_text(tmpdoc,",");
			}
		}
		if(  strlen(str_generic) > 0 ){
			document_add_text(tmpdoc," ");
			document_add_text(tmpdoc,str_generic);	
		}	

		if( (  strlen(str_generic) > 0 ) || ( strlen(str_specific) > 0  ) ){
			if( add_previous ){
				document_add_text (tmpdoc, ";\n\t");
			}			
		}
		
		g_free (str_generic);		
		g_free (str_specific);
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(voice_family_value_dialog);			
	}
}
// end voice family
void on_border_width_style_color_value_dialog_okbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *border_width_style_color_value_dialog;
	GtkWidget *checkbutton_length;
	GtkWidget *combo_entry_length;
	GtkWidget *spinbutton_length;
	GtkWidget *combo_entry_length_units;
	GtkWidget *checkbutton_color;
	GtkWidget *combo_entry_color;
	GtkWidget *colorselection;
	GtkWidget *checkbutton_style;
	GtkWidget *combo_entry_style;
	gboolean b_length;
	gboolean b_style;
	gboolean b_color;
	gchar* str_length_units;
	gchar* str_length;
	gchar* str_legth_and_units;
	gint nlength;
	gchar* strstyle;
	gchar* str_color;
	gchar* str_rgb_color;
	GdkColor gdkcolor;
	
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;

	border_width_style_color_value_dialog = 
			lookup_widget(GTK_WIDGET(button),
			"border_width_style_color_value_dialog");
	checkbutton_length = lookup_widget(GTK_WIDGET(button),"checkbutton_length");
	checkbutton_color = lookup_widget(GTK_WIDGET(button),"checkbutton_color");
	checkbutton_style = lookup_widget(GTK_WIDGET(button),"checkbutton_style");
	
	b_length = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_length));
	b_style = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_style));
	b_color =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_color));
	
	if( !b_length && !b_style && !b_color ){
		document_write_error_to_program_output(tmpdoc,"width style and color dialog: nothing selected. rejectiong to add empty values.");
	}else{	
		if( add_previous ){
			document_add_text (tmpdoc, (gchar *) data->previous_string);
			document_add_text (tmpdoc, ": ");
		}
		
		if( b_length ){
			
			combo_entry_length = lookup_widget(GTK_WIDGET(button),"combo_entry_length");

			str_length	= gtk_editable_get_chars(GTK_EDITABLE(combo_entry_length),0,-1);
			if( strcmp(str_length,"Use length") == 0 ){
				spinbutton_length = lookup_widget(GTK_WIDGET(button),"spinbutton_length");
				combo_entry_length_units = lookup_widget(GTK_WIDGET(button),"combo_entry_length_units");
				
				str_length_units = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_length_units),0,-1);
				nlength = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_length));
				
				str_legth_and_units = g_strdup_printf("%d%s", nlength, str_length_units);
				
				document_add_text(tmpdoc,str_legth_and_units);
				
				if( b_style || b_color ){
					document_add_text(tmpdoc," ");
				}				
				
				g_free(str_length_units);
				g_free(str_legth_and_units);		
				
			}else{
				document_add_text(tmpdoc,str_length);
				if( b_style || b_color ){
					document_add_text(tmpdoc," ");
				}
			}
			g_free( str_length );
		}
		
		if( b_style ){
			combo_entry_style = lookup_widget(GTK_WIDGET(button),"combo_entry_style");	
			strstyle = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_style),0,-1);	
			document_add_text(tmpdoc,strstyle);
			if(  b_color ){
				document_add_text(tmpdoc," ");
			}	
			g_free(strstyle);		
		}
		
		if( b_color ){
			combo_entry_color = lookup_widget(GTK_WIDGET(button),"combo_entry_color");	
			str_color= gtk_editable_get_chars(GTK_EDITABLE(combo_entry_color),0,-1);
			
			if( strcmp(str_color,"from color selector") == 0 ){
				colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");
				gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(colorselection),&gdkcolor);				
				str_rgb_color = gdk_color_to_rgb_triplet_string(&gdkcolor);
				document_add_text(tmpdoc,str_rgb_color);
				g_free(str_rgb_color);
			}else{
				document_add_text(tmpdoc,str_color);
			}
			g_free(	str_color );
		}
		
		if( add_previous ){
			document_add_text (tmpdoc, ";\n\t");
		}
		g_free (data->previous_string);
		g_free (user_data);
		gtk_widget_destroy(border_width_style_color_value_dialog);
	}
}

void
on_color_selector_dialog_okbutton_clicked
	(GtkButton       *button,  gpointer         user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *color_selector_dialog;
	GtkWidget *radiobutton_named;
	GtkWidget *combo_entry_named;
	GtkWidget *colorselection;
	gboolean b_use_named_color;
	GdkColor gdkcolor;
	gchar* str_color;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	color_selector_dialog = lookup_widget(GTK_WIDGET(button),"color_selector_dialog");
	radiobutton_named = lookup_widget(GTK_WIDGET(button),"radiobutton_named");
	combo_entry_named = lookup_widget(GTK_WIDGET(button),"combo_entry_named");
	colorselection = lookup_widget(GTK_WIDGET(button),"colorselection");
	
	b_use_named_color = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton_named));
	
	if( b_use_named_color ){
		str_color = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_named),0,-1);
	}else{
		gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(colorselection),
			&gdkcolor );
		
		str_color = gdk_color_to_rgb_triplet_string(&gdkcolor);
	}
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}
	
	document_add_text (tmpdoc, str_color);
	
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}	
	g_free (str_color);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(color_selector_dialog);
}
void
on_border_style_dialog_okbutton_clicked
       (GtkButton       *button,   gpointer         user_data)
{
	Tcssed_cssdialog_data *data;
	Tcssed_window *window;
	Tcssed_doc *tmpdoc;
	gboolean add_previous;
	GtkWidget *border_style_dialog;
	GtkWidget *combo_entry_style;
	gchar* str_style;
	
	data = (Tcssed_cssdialog_data *) user_data;
	window = CSSED_WINDOW (data->main_window);
	tmpdoc = document_get_current (window);
	add_previous = data->add_property_string;
	
	border_style_dialog = lookup_widget(GTK_WIDGET(button),"border_style_dialog");
	combo_entry_style = lookup_widget(GTK_WIDGET(button),"combo_entry_style");
	
	str_style = gtk_editable_get_chars(GTK_EDITABLE(combo_entry_style),0,-1);
	
	if( add_previous ){
		document_add_text (tmpdoc, (gchar *) data->previous_string);
		document_add_text (tmpdoc, ": ");
	}	
	
	document_add_text (tmpdoc, str_style);
	
	if( add_previous ){
		document_add_text (tmpdoc, ";\n\t");
	}	
	
	g_free (str_style);
	g_free (data->previous_string);
	g_free (user_data);
	gtk_widget_destroy(border_style_dialog);
}
/* ---------------- CALLED BY ALL DIALOGS ----------------------------------  */
/* this will be called on all cancel button click events                      */
void
on_css_dialog_cancel_button_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *dialog;
	Tcssed_cssdialog_data *data;

	data = (Tcssed_cssdialog_data *) user_data;

	dialog = gtk_widget_get_toplevel (GTK_WIDGET (button));
	gtk_widget_destroy (dialog);
	g_free (data->previous_string);
	g_free (user_data);
}
/* this is called when any dialog is closed                                   */
void
on_css_dialog_close (GtkDialog *dialog, gpointer  user_data)
{
	Tcssed_cssdialog_data *data;

	data = (Tcssed_cssdialog_data *) user_data;

	g_free (data->previous_string);
	g_free (user_data);
}
/* *************************** HELP BUTTONS ********************************* */
void on_angle_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{


}
void on_background_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_border_properties_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_border_with_properties_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_border_width_single_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_dialog_counter_increment_reset_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_string_properties_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void
on_uri_value_dialog_helpbutton_clicked (GtkButton * button,
					gpointer user_data)
{

}
void on_attr_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_counter_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_font_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_font_family_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void
on_font_size_dialog_helpbutton_clicked (GtkButton * button,
					gpointer user_data)
{

}
void
on_dialog_frecuency_helpbutton_clicked (GtkButton * button,
					gpointer user_data)
{

}
void
on_dialog_integer_helpbutton_clicked (GtkButton * button, gpointer user_data)
{

}
void on_length_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_list_style_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_margin_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_margin_width_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_number_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_padding_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_outline_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_pause_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_percentage_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_percentage_length_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_play_during_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_quotes_property_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_shape_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_size_lenght_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void on_border_width_style_color_value_dialog_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{

}
void
on_color_selector_dialog_helpbutton_clicked
	(GtkButton       *button, gpointer         user_data)
{

}
void on_identifier_value_helpbutton_clicked
	(GtkButton * button, gpointer user_data)
{
	
}

void
on_border_style_dialog_helpbutton_clicked
      (GtkButton       *button,    gpointer         user_data)
{
	
}
