/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef PLAT_GTK
#   define PLAT_GTK 1
#endif

//#define DEBUG 
//#define DEBUG_CONTROL_NOTIFICATIONS 

#include <gtk/gtk.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libgen.h>	

#include "cssedwindow.h"
#include "callbacks.h"
#include "cssdialogs-interface.h"
#include "interface.h"
#include "support.h"
#include "document.h"

#include <Scintilla.h>
#include <SciLexer.h>
#include <ScintillaWidget.h>

/* some prototipes will be moved to sciwrappers.h and functions to sciwrappers.c*/
void 				sci_set_css_style 			(ScintillaObject * sci);
void 				on_editor_notification		(GtkWidget* editor,gint scn, gpointer lscn, gpointer user_data);
ScintillaObject* 	sci_get_from_document		(Tcssed_doc* doc);
ScintillaObject* 	sci_get_from_notebook_page	(GtkNotebook* notebook, gint 		page_number);
void 				sci_set_line_numbers		(ScintillaObject* sci,  gboolean 	set );
void 				sci_set_keyworkds 			(ScintillaObject* sci,  gchar* 		keywords);
void 				sci_add_text 				(ScintillaObject* sci,  gchar* 		text);
void 				sci_undo					(ScintillaObject* sci);
void 				sci_redo					(ScintillaObject* sci);
void 				sci_empty_undo_buffer		(ScintillaObject* sci);
void 				sci_end_undo_action			(ScintillaObject* sci);
void 				sci_start_undo_action		(ScintillaObject* sci);
void				sci_set_visible_eols		(ScintillaObject* sci, gboolean set );
gboolean			sci_get_visible_eols		(ScintillaObject* sci);
void				sci_set_lines_wrapped		(ScintillaObject* sci, gboolean set );
gboolean 			sci_get_lines_wrapped		(ScintillaObject* sci);
void				sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set );
gboolean 			sci_get_visible_white_spaces(ScintillaObject* sci);
gint				sci_get_eol_mode			(ScintillaObject* sci);
void 				sci_convert_eols			(ScintillaObject* sci, gint eolmode);
void 				sci_set_eol_mode			(ScintillaObject* sci, gint eolmode);
void 				sci_zoom_in					(ScintillaObject* sci);
void 				sci_zoom_out				(ScintillaObject* sci);
void 				sci_zoom_off				(ScintillaObject* sci);
void				sci_set_marker_at_line		(ScintillaObject* sci, gint line_number, gboolean set, gint marker );
gboolean 			sci_is_marker_set_at_line	(ScintillaObject* sci, gint line, gint marker);
gboolean 			sci_marker_next				(ScintillaObject* sci, gint line, gint marker_mask);
gboolean 			sci_marker_prev				(ScintillaObject* sci, gint line, gint marker_mask);

gint 				sci_get_line_from_position	(ScintillaObject* sci, gint position);
gint 				sci_get_position_from_line	(ScintillaObject* sci, gint line );
gint 				sci_get_current_position	(ScintillaObject* sci);
void 				sci_set_current_position	(ScintillaObject* sci, gint position);
void 				sci_set_current_line		(ScintillaObject* sci, gint line);

void 				sci_cut						(ScintillaObject* sci);
void 				sci_copy					(ScintillaObject* sci);
void 				sci_paste					(ScintillaObject* sci);

gint				sci_get_selection_start		(ScintillaObject* sci);
gint				sci_get_selection_end		(ScintillaObject* sci);
void 				sci_replace_sel				(ScintillaObject* sci, gchar* text);

gint				sci_get_length				(ScintillaObject* sci);
void				sci_get_text				(ScintillaObject* sci,gint len,gchar* text);

void				sci_get_line				(ScintillaObject* sci, gint line, gchar* text);
gint 				sci_line_length				(ScintillaObject* sci, gint line);

void 				sci_get_xy_from_position	(ScintillaObject* sci,gint pos, gint* x, gint* y);
/* create an empty doc with an scintilla editor attached 
	fill the struct fields.
*/
Tcssed_doc *
cssed_create_new_doc ( Tcssed_window* window )
{
	GtkWidget *editor;
	ScintillaObject *sci;
	Tcssed_doc *newdoc;
	gint sciid;

	newdoc = g_malloc (sizeof (Tcssed_doc));

	editor = scintilla_new ();
	sci = SCINTILLA (editor);
	sciid = get_new_sci_number ();
	scintilla_set_id (sci, sciid);

#ifdef DEBUG
	g_print ("document.c:cssed_create_new_doc() - created doc\n\teditor\t%x\n\tsiid\t%d\n", editor, sciid);
	g_print ("document.c:cssed_create_new_doc() - in list %d\n\n", g_list_length (window->document_list));
#endif
	g_signal_connect( editor, "sci-notify", G_CALLBACK(on_editor_notification), (gpointer) window );
	gtk_widget_set_usize (editor, 500, 300);
	newdoc->editor = editor;					// scintilla editor
	newdoc->scid = sciid;						// scintilla id
	newdoc->notebook = window->notebook_docs;	// main notebook where this doc is attached
	newdoc->filename = NULL;    				// full path to file if exists
	newdoc->modified = FALSE;					// fresh document, so unmodified
	newdoc->filetype = CSSED_FILETYPE_CSS;		// one of the supported filetype css, html, text, bynary or unknow
	newdoc->autocompletion = FALSE;				// no autocompletion at start, user must choose
	newdoc->window = window;
	
	document_set_css_style( newdoc );

	return newdoc;
}

/*  create an empty doc with an scintilla editor and
	attach to the notebook passed as argument, puts the
	string title on notebook label.
	FIXME I nedd to ass the doc to the doclist as it's attached
*/
void
create_and_attach_new_doc (Tcssed_window* window , const gchar * title)
{
	Tcssed_doc *newdoc;
	GtkWidget *label;

	newdoc = cssed_create_new_doc (window);
	if (newdoc == NULL)
	{ // FIXME say something to user
#ifdef DEBUG
		g_print ("document.c:create_and_attach_new_doc() - passed null doc !!!\n\n");
#endif
	}
	else
	{
#ifdef DEBUG
		g_print ("document.c:create_and_attach_new_doc() - doc passed %x\n", newdoc);
		g_print ("document.c:create_and_attach_new_doc() - in list %d\n\n", g_list_length (window->document_list));
#endif
		window->document_list = g_list_append (window->document_list, newdoc);
		gtk_widget_show (newdoc->editor);
		label = gtk_label_new (title);
		gtk_notebook_append_page ((GtkNotebook *) window->notebook_docs,  newdoc->editor, label);
		gtk_widget_queue_draw (GTK_WIDGET (window->notebook_docs));
	}
}

void
create_and_attach_new_named_doc_from_fileselector_cb (GtkButton * button, gpointer user_data)
{
	Tcssed_doc *newdoc;
	GtkWidget *label;
	GtkWidget *notebook;
	GtkWidget *file_selector;
	Tcssed_fileselector_data* data;
	Tcssed_window* window;			
	const gchar *selected_filename;
	gint pnum;
	struct stat _stat;
	gint filedes;
	gint _retval;
	void *map;
	gchar *ptr;
	gchar *block_end;
	gchar* line_start;
	gchar* line_buffer;
	gint line_size;
	gchar* file_name;
	gchar* filenamecp;
	gchar* errormsg;

	data = (Tcssed_fileselector_data*) user_data;
	window = (Tcssed_window *) data->main_window;
	notebook = GTK_WIDGET (data->notebook);
	file_selector = GTK_WIDGET (data->fileselector);

	selected_filename =	gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
	gtk_widget_destroy (gtk_widget_get_toplevel (file_selector));	

#ifdef DEBUG
	g_print ("document.c - create_and_attach_new_named_doc_from_fileselector_cb\n");
	g_print ("\tSelected filename: %s\n\n", selected_filename);
#endif

	if (!g_file_test
	    (selected_filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
	{
		errormsg = g_strdup_printf("File %s\ndon't exist or is not a regular file.",selected_filename);
		cssed_error_message (errormsg,"Error opening file");
		g_free(errormsg);
		g_free (data);
		return;
	}
	_retval = stat (selected_filename, &_stat);
	if (_retval != 0)
	{
		errormsg = g_strdup_printf("Can't get information about file\n%s.",selected_filename);
		cssed_error_message (errormsg,"Error opening file");
		g_free(errormsg);
		g_free (data);
		return;
	}
	if ((filedes = open (selected_filename, O_RDWR)) < 0)
	{
		errormsg = g_strdup_printf("Can't open the file\n%s.",selected_filename);
		cssed_error_message (errormsg,"Error opening file");
		g_free(errormsg);
		g_free (data);
		return;
	}
	if ((map =
	     mmap (0, _stat.st_size, PROT_READ, MAP_SHARED, filedes,
		   0)) == MAP_FAILED)
	{
		errormsg = g_strdup_printf("Can not map to memory the file\n%s.",selected_filename);
		cssed_error_message (errormsg,"Error opening file");
		close (filedes);
		g_free(errormsg);
		g_free (data);
		return;
	}

	ptr = map;		// alocate the pointer at start of memory block
	block_end = map + _stat.st_size;	     // end of memory block
	line_start = map;	                     // start of first line

	newdoc = cssed_create_new_doc (window);
	window->document_list = g_list_append (window->document_list, newdoc);
	
	if (newdoc == NULL)
	{
#ifdef DEBUG /* FIXME; say something to user */
		g_print ("document.c create_(...)_from_fileselector_cb() \n\tERR _ Passed null doc !!!\n");
#endif
		close (filedes);
		g_free (data);
		munmap(map, _stat.st_size);
		return;
	}
	else
	{
#ifdef DEBUG
		g_print ("document.c create_(...)_from_fileselector_cb() \n\tDoc passed %x\n", newdoc);
		g_print ("document.c create_(...)_from_fileselector_cb() \n\tin list %d\n", g_list_length (window->document_list));
#endif
		// add the file to the editor 
		line_size = 0;
		while( ptr < block_end ){
			if( ( (char) *ptr == '\n' ) || (ptr == block_end) ){
				// add line
				line_buffer = (char*) calloc( line_size + 1, sizeof(char));
				if( line_buffer == NULL ){ /* FIXME; say something to user */
#ifdef DEBUG
					g_print ("document.c create_(...)_from_fileselector_cb() \n\tCan't reserve mem, calloc returned %x\n", line_buffer);
#endif
					close (filedes);
					g_free (data);
					munmap(map, _stat.st_size);
					return;
				}else{		
					line_buffer = strncpy(line_buffer,line_start , line_size * sizeof(char));
					document_add_text_line( newdoc,(char*)  line_buffer);
					free(line_buffer);
				}
				line_size = 0;
                line_start = ptr + 1;
			}else{
				line_size++;
			}
			ptr++;
		}

		filenamecp = (char*) g_strdup( selected_filename );		
		file_name = basename( filenamecp );
		
		newdoc->notebook = notebook;
		newdoc->filename = filenamecp;
		newdoc->filetype = CSSED_FILETYPE_CSS;
		newdoc->modified = FALSE;
		
		gtk_widget_show (newdoc->editor);
		label = gtk_label_new (file_name);
		gtk_notebook_append_page ((GtkNotebook *) notebook, newdoc->editor, label);
		pnum = gtk_notebook_get_n_pages (GTK_NOTEBOOK(notebook));
		gtk_notebook_set_current_page   (GTK_NOTEBOOK(notebook), --pnum);		
		gtk_widget_queue_draw (GTK_WIDGET (notebook));
	}
	close (filedes);
	g_free (data);
	munmap(map, _stat.st_size);
}

/* create an alone editor used as a callback from create_window
	NEDDS FIX don't use in other created documents. 
	use create_and_attach_new_doc() instead
*/
GtkWidget *
create_initial_blank_editor (Tcssed_window* window)
{
	Tcssed_doc *newdoc;

	newdoc = cssed_create_new_doc (window);
	window->document_list = g_list_append (window->document_list, newdoc);

#ifdef DEBUG
	g_print ("document.c:create_initial_blank_editor ()\n\tInitial editor created\n");
	g_print ("document.c:create_initial_blank_editor ()\n\tin list %d\n", g_list_length (window->document_list));
#endif

	return newdoc->editor;
}

/*  Get diferent - incrementally - integers as sci ids
	so, right now ,we are limited in docs opened in a
	single editing session. Even if you closed them.
	NEEDS FIX
*/
int
get_new_sci_number ()
{
	static int initialized;
	static unsigned int count;

	if (initialized != 1)
	{
		initialized = 1;
		count = 0;
		return ++count;	// id 0 created on startup
	}
	else
	{
		return ++count;
	}
}
/* scroll to the caret 
	when you adds text, to the scintilla control, if the
	edge bound of text is down the editor window, no automatic
	scrolling is performed so you must call this func
	to scroll to the caret before text insertion.
*/
void 
document_scroll_to_caret( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );	
	SSM (sci, SCI_SCROLLCARET, 0, 0);
}

/* document wrapper, calls to sci_set_css_style  
		and sci_set_keywords */
void 
document_set_css_style(Tcssed_doc* doc)
{
	ScintillaObject * sci;
	Tcssed_window* window;
	
	window = CSSED_WINDOW(doc->window);
	
	sci = SCINTILLA( doc->editor );
	sci_set_css_style (sci);

	sci_set_keyworkds (sci, (gchar*) window->keywords_list);
#ifdef DEBUG
	g_print("document.c: document_set_css_style() - Keywords used\n%s\n\n",window->keywords_list);
#endif
}
Tcssed_doc*
document_get_from_editor( Tcssed_window* window, GtkWidget* editor )
{
	GList* tmplist;
	Tcssed_doc* tmpdoc;	
	
	tmplist = g_list_first(window->document_list);
	while(tmplist){	
		tmpdoc = (Tcssed_doc*) tmplist->data;
		if( editor == (GtkWidget*) tmpdoc->editor ){
			return tmpdoc;
		}else{
			tmplist = g_list_next(tmplist);
		}
	}
	// no doc in list, if it reachs here...  :-(
	return NULL;	
}
Tcssed_doc*
document_get_from_notebook_page_num(Tcssed_window* window, gint page_number)
{
	GtkWidget* editor;
	Tcssed_doc* tmpdoc;
	GtkWidget* notebook;
	
	notebook = window->notebook_docs;
	
	editor = gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook), page_number);
	return document_get_from_editor( window, editor );		
}

Tcssed_doc*
document_get_current(Tcssed_window* window)
{
	GtkWidget* notebook;
	Tcssed_doc* tmpdoc;
	gint current_index;
	
	notebook = window->notebook_docs;
	
	current_index =  gtk_notebook_get_current_page   ( GTK_NOTEBOOK(notebook) );
	tmpdoc = document_get_from_notebook_page_num(window, current_index);
	
	return tmpdoc;
}
/* right now it's not needed anymore  
   as it's in the *doc->modified field 
	In the other hand, each char you type
	the modified flag is set so we have a
	memory acces that can be avoided.
	Perhaps in the future, this func  
	will be used again so DON'T DELETE IT */ 
gboolean
document_get_modified( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	gint modified;
	
	sci = (ScintillaObject *) sci_get_from_document(  doc );	
	modified = SSM (sci, SCI_GETMODIFY, 0, 0);
	if( modified == 0) return FALSE;
	else return TRUE;	
}

void
document_set_modified( Tcssed_doc* doc , gboolean modified)
{
	ScintillaObject * sci;
	
	if( modified ){
		doc->modified = TRUE;
	}else{
		sci = (ScintillaObject *) sci_get_from_document(  doc );	
		SSM (sci, SCI_SETSAVEPOINT, 0, 0);
		doc->modified = FALSE;
	}	
}
/*  Detach the doc from the notebook , pops it out the list
	and clear all memory allocated. If last doc in queue it
	must open a clean one.
	Save before it or changes will be lost.
*/
gboolean
document_dispose( Tcssed_doc* doc )
{
	GtkWidget* notebook;
	GtkWidget* editor;
	Tcssed_window* window;	
	GList* tmplist;
	gint document_page_number;
	gint notebook_total_pages;
	gboolean needs_new = FALSE;
	
	notebook = doc->notebook;
	editor = doc->editor;
	
	document_page_number = gtk_notebook_page_num(GTK_NOTEBOOK(notebook),editor);
	
	if( document_page_number == -1 ){ // editor don't exist
		return FALSE;
	}else{
		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), document_page_number);
		notebook_total_pages = gtk_notebook_get_n_pages  (GTK_NOTEBOOK(notebook));
		if( notebook_total_pages == 0 ){
			needs_new = TRUE;
#ifdef DEBUG
			g_print("document.c:document_dispose() - Last notebook page out.\nWill create new one ...\n\n");
#endif	
		}	
		// clear doc allocated mem and pop out the doc list
		if( doc->filename != NULL ){
			g_free(doc->filename);
		}
		window = doc->window;
		tmplist = window->document_list;
		window->document_list = g_list_remove(tmplist,doc);
		g_free(doc);
	}

	if( needs_new )	create_and_attach_new_doc (window,"Untitled");
	return TRUE;
}

/* line numbers visibility */
gboolean 
document_get_line_numbers_visible( Tcssed_doc* doc )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_line_numbers(sci);
}

void 
document_set_line_numbers_visible( Tcssed_doc* doc, gboolean visible )
{
	ScintillaObject* sci;	
	
#ifdef DEBUG
	g_print("Going to set line numbers visibility to %s in doc %x\n",visible?"true":"false",doc);	
#endif	
	sci = sci_get_from_document(doc);
	sci_set_line_numbers(sci,visible);
}

/* line endings visibility */
gboolean
document_get_line_endings_visible( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_visible_eols(sci);	
}
void
document_set_line_endings_visible( Tcssed_doc* doc, gboolean visible)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_visible_eols(sci,visible);	
}
/* line wrapping */
gboolean
document_get_lines_wrapped( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_lines_wrapped(sci);	
}
void
document_set_lines_wrapped( Tcssed_doc* doc, gboolean wrapped)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_lines_wrapped(sci,wrapped);	
}
/* white spaces */
gboolean
document_get_white_spaces_visible( Tcssed_doc* doc)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	return sci_get_visible_white_spaces(sci);	
}
void
document_set_white_spaces_visible( Tcssed_doc* doc, gboolean visible)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);
	sci_set_visible_white_spaces(sci,visible);	
}
/* EOL mode */
gint
document_get_eol_mode( Tcssed_doc* doc )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	return sci_get_eol_mode	(sci);
}

void 
document_set_eol_mode(Tcssed_doc* doc, gint eolmode)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	sci_set_eol_mode(sci, eolmode);
}

void 
document_convert_eols(Tcssed_doc* doc, gint eolmode)
{
	ScintillaObject* sci;	
	sci = sci_get_from_document(doc);	
	sci_convert_eols(sci, eolmode);
}
/* text */
void 
document_add_text( Tcssed_doc* doc, gchar* text )
{
	ScintillaObject* sci;	
	sci = sci_get_from_document( doc );
	sci_add_text(sci,text);
}

void/* adds text to the document editor ending in line feed */
document_add_text_line( Tcssed_doc * doc, char* text )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );

	SSM (sci, SCI_ADDTEXT, strlen(text) , (sptr_t) text );
	SSM (sci, SCI_ADDTEXT, strlen("\n") , (sptr_t) "\n" );
}
void // no wrapper function for this
document_search_text( Tcssed_doc* doc, gchar* text, gint flags )
{
	ScintillaObject* sci;	
	gint selection_start;
	gint selection_end;	
#ifdef DEBUG	
	gint current_pos;
	gint anchor_pos;
#endif	
	
	sci = sci_get_from_document( doc );	
	// look if any selection, if so move the caret out it can
	// it can be selected by a prevous search.
	// right now, cssed can't search into a block of text selected by
	// the user. The user must know that.
	
	selection_start = SSM( sci, SCI_GETSELECTIONSTART, 0, 0);
	selection_end   = SSM( sci, SCI_GETSELECTIONEND, 0, 0);
	
#ifdef DEBUG	
	current_pos		= SSM( sci, SCI_GETCURRENTPOS, 0, 0);
	anchor_pos		= SSM( sci, SCI_GETANCHOR, 0, 0);
	g_print( "document_search_text(): Selection start %d end %d pos %d anchor %d\n",selection_start,selection_end,current_pos,anchor_pos);
#endif

	if( selection_end > selection_start ){ // there's selection
		// move the caret
		SSM( sci, SCI_GOTOPOS, selection_end + 1, 0);
	}	
	// set the anchor
	SSM( sci, SCI_SEARCHANCHOR,0,0);

	if( SSM( sci, SCI_SEARCHNEXT,flags,(sptr_t) text ) != -1 ){
		SSM( sci, SCI_SCROLLCARET,0,0);
	}else{		
		cssed_error_message("The text don't exists or\nthere is no document active.","Text not found");
	}
}

void 
document_undo(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_undo(sci);
/*
	gtk_widget_queue_draw ( GTK_WIDGET(doc->notebook) );	
	gtk_widget_queue_draw ( GTK_WIDGET(doc->editor) );
*/
}
void 
document_redo(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_redo(sci);
}

void 
document_end_undo_action(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_end_undo_action(sci);
}

void 
document_start_undo_action (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_start_undo_action(sci);
}

void 
document_empty_undo_buffer(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_empty_undo_buffer(sci);
}

void 
document_zoom_in(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_in(sci);
}

void 
document_zoom_out(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_out(sci);
}

void 
document_zoom_off(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_zoom_off(sci);
}

void 
document_cut (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_cut (sci);
}
void 
document_copy (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_copy (sci);
}
void 
document_paste (Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_paste (sci);
}

void 
document_set_marker_at_line( Tcssed_doc* doc, gint line, gboolean set, gint marker)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	sci_set_marker_at_line( sci, line, set, marker);
}
gboolean
document_is_marker_set_at_line( Tcssed_doc* doc, gint line, gint marker)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );
	return sci_is_marker_set_at_line( sci, line, marker);
}
gboolean 
document_marker_next(Tcssed_doc* doc, gint line, gint marker_mask)
{
	ScintillaObject* sci;
	gint position;
	
	sci = sci_get_from_document( doc );	
	sci_marker_next( sci, line, marker_mask );
}
	
	
gboolean 
document_marker_prev(Tcssed_doc* doc, gint line, gint marker_mask)
{
	ScintillaObject* sci;
	gint position;
	
	sci = sci_get_from_document( doc );		
	sci_marker_prev( sci, line, marker_mask );
}

gint
document_get_line_from_position( Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_line_from_position(sci, position );
}
gint
document_get_position_from_line( Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_position_from_line(sci, line );
}
gint 
document_get_current_position	(Tcssed_doc* doc )
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	return sci_get_current_position	(sci);
}

void
document_set_current_position	(Tcssed_doc* doc, gint position)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	sci_set_current_position(sci, position);
}

void
document_set_current_line(Tcssed_doc* doc, gint line)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );		
	sci_set_current_line(sci, line);
}
gint
document_get_selection_start(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_start	(sci);
}

gint
document_get_selection_end(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_get_selection_end(sci);
}
void
document_replace_sel(Tcssed_doc* doc, gchar* text)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_replace_sel(sci, text);
}
void
document_clear_sel(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	sci_replace_sel(sci, "");
}

gint
document_get_length	(Tcssed_doc* doc)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_length(sci);	
}

void
document_get_text(Tcssed_doc* doc, gint len, gchar* text)
{
	ScintillaObject* sci;
	
	sci = sci_get_from_document( doc );	
	return sci_get_text(sci,len,text);
}

void 
document_write_error_to_program_output(Tcssed_doc* doc, gchar* error_message)
{
	Tcssed_window *window;

	GtkListStore *store;
	GtkTreeIter iter;
	GtkWidget* output_list;
	GtkTreePath* path;
	
	window = CSSED_WINDOW(doc->window);
	store = window->output_store;

	gtk_list_store_insert_before(store,&iter,NULL);
	gtk_list_store_set (store, &iter, 0, error_message, -1);
	
	output_list = lookup_widget(GTK_WIDGET(doc->notebook),"output_list");
	
	path = gtk_tree_model_get_path ((GtkTreeModel*) store,&iter);
	gtk_tree_view_scroll_to_cell ((GtkTreeView*) output_list,path,NULL,FALSE,0,0);
}
gboolean
document_current_pos_is_into_selector(Tcssed_doc* doc)
{
	ScintillaObject *sci;

	gint position;
	gint open_brace;
	gint close_brace;
	gchar *buffer;



	sci = (ScintillaObject *) sci_get_from_document (doc);
	position = document_get_current_position (doc);


	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	open_brace = SSM (sci, SCI_SEARCHPREV, 0, (sptr_t) "{");

#ifdef DEBUG
	if (open_brace == -1)
		g_print ("No open brace before\n");
	else
		g_print ("Open brace at %d\n", open_brace);
#endif

	SSM (sci, SCI_SEARCHANCHOR, 0, 0);
	close_brace = SSM (sci, SCI_SEARCHNEXT, 0, (sptr_t) "}");

#ifdef DEBUG
	if (close_brace == -1)
		g_print ("No close brace after\n");
	else
		g_print ("Close brace at %d\n", close_brace);
#endif

	if (close_brace < position)
	{
		document_set_current_position(doc,position);
		return FALSE;
	}
	else
	{
		if ((open_brace != -1) && (close_brace != -1))
		{	
			document_set_current_position(doc,position);
			return TRUE;
		}else{
			document_set_current_position(doc,position);
			return FALSE;
		}
	}	
}
gint 
document_get_line_length(Tcssed_doc* doc, gint line)
{
	ScintillaObject *sci;

	sci = (ScintillaObject *) sci_get_from_document (doc);
	
	return sci_line_length(sci,line);
}

gint 
document_get_text_line(Tcssed_doc* doc, gint line,gchar* buffer)
{
	ScintillaObject *sci;

	sci = (ScintillaObject *) sci_get_from_document (doc);	
	sci_get_line(sci, line, buffer);
}
void  // not in document.h
document_get_xy_from_position(Tcssed_doc* doc,gint pos, gint* x, gint* y)
{
	ScintillaObject *sci;

	sci = (ScintillaObject *) sci_get_from_document (doc);	
	sci_get_xy_from_position(sci, pos, x, y);	
}
void   // not in document.h
document_gtk_popmenu_position_func(GtkMenu *menu, gint *x, gint *y,
				gboolean *push_in,  gpointer user_data)
{
	Tcssed_point* point;
	Tcssed_window* window;
	Tcssed_doc* doc;
	GtkWidget* notebook;
	GtkWidget* editor;
	GtkWidget* window_widget;
	GdkWindow* gdkwindow;
	gint new_x;
	gint new_y;
	gint root_x;
	gint root_y;
		
	point = (Tcssed_point*) user_data;
	window = CSSED_WINDOW(point->main_window);
	doc = document_get_current(window);
	notebook = (GtkWidget*) window->notebook_docs;
	editor = GTK_WIDGET(doc->editor);
	window_widget = gtk_widget_get_toplevel(notebook);
	gdkwindow = gtk_widget_get_parent_window (editor);	
#ifdef DEBUG
	g_print("document_gtk_popmenu_position_func:\n\tis notebook %d\n",GTK_IS_NOTEBOOK(notebook));
#endif
	gtk_widget_translate_coordinates        (editor,
                                             window_widget,
                                             point->x,
                                             point->y,
                                             &new_x,
                                             &new_y);											 
#ifdef DEBUG
	g_print("Translated coordinates from editor to window\n\tx = %d , y = %d\n",
			new_x, new_y);
#endif	
	gdk_window_get_root_origin(gdkwindow, &root_x, &root_y);
#ifdef DEBUG
	g_print("Root window coordinates\n\tx = %d , y = %d\n",
			root_x,root_y);
#endif	
	*x = new_x + root_x + 5;  // +5  add a little padding to right
	*y = new_y + root_y + 30; // +30 add a little padding down
#ifdef DEBUG
	g_print("Menu coordinates\n\tx = %d , y = %d\n",
			*x,*y);
#endif		
	g_free(user_data);
}
/* It will scan the second treeview colum ( in the css definitions treeview )
	trying to find the text passed. If it's found, it builds a popmenu with the
	third column contents and connect the appropiated callbacks to get 
	autocompletion to work */
void // not in document.h
document_pop_submenu_from_keyword_in_treeview(Tcssed_doc* doc,gchar* text)
{
	Tcssed_window* window;
	Tcssed_point* point;
	Tcssed_popmenu_data* popmenu_data;
	GdkColor color;
	GtkWidget* treeview;
	GtkTreeModel* model;
	GtkWidget* menu;
	GtkWidget* menu_items;
	GtkTreeIter iter;
	gboolean valid;
	gboolean validchild;
	gchar* keyword;	
	gchar* value;
	gchar* path_str;
	GList* menu_item_list = NULL;
	gint nrow;
	gint x;
	gint y;
	
	window = CSSED_WINDOW(doc->window);
	treeview = window->tree_view_css_properties;
	
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	
	if ( model == NULL ){
		return;
	}
	valid = gtk_tree_model_get_iter_from_string(model,&iter,"0:0");
	if(!valid){
		return;
	}
	nrow = 0;
	while(valid){
		gtk_tree_model_get (model, &iter, 1, &keyword, -1);
#ifdef DEBUG
        g_print ("scanning %s\n", keyword);
#endif
		if( strcmp(keyword,text)==0 ){
#ifdef DEBUG			
			g_print("found !!\n");
#endif			
			path_str = g_strdup_printf("0:%d:0",nrow);
			valid =  gtk_tree_model_get_iter_from_string(model,&iter,path_str);
#ifdef DEBUG			
			g_print("trying to get child on %s\n",path_str);
#endif			
			g_free(path_str);
			if(!valid ){
#ifdef DEBUG				
				g_print("can't get child.\n");
#endif					
				g_free(keyword);
				return;
			}
#ifdef DEBUG			
			g_print("got 3rd child!\n");
#endif				
			menu = gtk_menu_new ();
			// ensure that will receive the hide event
			//gtk_widget_add_events( menu, GDK_VISIBILITY_NOTIFY_MASK  );		
			while( valid ){	
				popmenu_data = g_malloc(sizeof(	Tcssed_popmenu_data ));
				gtk_tree_model_get (model, &iter, 2, &value, -1);
				menu_items = gtk_menu_item_new_with_label (value);
				// ensure that will receive the hide event
				//gtk_widget_add_events( menu_items, GDK_VISIBILITY_NOTIFY_MASK );
#ifdef DEBUG				
				g_print("\t* %s\n",value);
#endif					
				popmenu_data->main_window = window;
				popmenu_data->document = document_get_current(window);
				popmenu_data->attribute = g_strdup(value);
				popmenu_data->property = g_strdup(keyword);
				popmenu_data->add_property_string = FALSE;
				
				menu_item_list = g_list_append(menu_item_list, popmenu_data);
				gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_items);
				
				// callback will call fire_dialog_from_strings_or_write_output()
				g_signal_connect (G_OBJECT (menu_items), "activate",
                                      G_CALLBACK (popmenu_autocomp_activate_cb), 
                                      popmenu_data );				
								  
				gtk_widget_show (menu_items);
				g_free(value);
				valid = gtk_tree_model_iter_next (model,&iter);				
			}
			
			// this will dispose all memory allocated parsing the list 
			// passed on unmap event (event->type ==  GDK_UNMAP).
			g_signal_connect (G_OBJECT (menu), "event-after",
                               G_CALLBACK (popmenu_autocomp_event_after_cb), 
                               menu_item_list );			
#ifdef DEBUG
			g_print("Sending list %p as user_data for callback\n",menu_item_list);
#endif		
			document_get_xy_from_position(doc,document_get_current_position(doc),&x,&y);
#ifdef DEBUG	
			g_print("Current point into editor x=%d,y=%d\n",x,y);
#endif				
			point = g_malloc(sizeof(Tcssed_point));
			point->main_window = window;
			point->x = x;
			point->y = y;
			gtk_menu_popup (GTK_MENU(menu),NULL, NULL,
										 document_gtk_popmenu_position_func  ,
                                         (gpointer) point,// freed by callback
                                         0,
                                         gtk_get_current_event_time());
			g_free(keyword);
			return;				
		}
		nrow++;
        g_free (keyword);
		valid = gtk_tree_model_iter_next (model,&iter);
     }
}

void 
document_show_autocompletion_list(Tcssed_doc* doc, gint identifier, gchar* list)
{
	ScintillaObject* sci;
	sci = sci_get_from_document( doc );	
	SSM(sci,SCI_AUTOCSHOW,0, (sptr_t) list); 
}

// look if it's a fixed string or a dialog
// dialogs are marked on the XML properties file
// between parenthesis:
// (color) (angle) (background) (border), etc ...
// more to come ....    33 new arrived !!   :P
void 
fire_dialog_from_strings_or_write_output(Tcssed_window* window, 
										Tcssed_doc* tmpdoc, 
										gchar* attribute, 
										gchar* property , 
										gboolean add_previous)
{
	gchar *line_to_insert;
	gchar* strdialog;
	GtkWidget *dialog;

	// FIXME use a hash table or something quicker than this
	if( attribute[0] == '(' ){ // if it don't start with ( is not a dialog
		if (strcmp (attribute, "(color)") == 0)
		{   // this must change to the new color selection dialog FIXME
			dialog = create_color_selector_dialog (window,
								  g_strdup (property) ,  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(angle)") == 0)
		{
			dialog = create_angle_property_dialog (window,
								   g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(background)") == 0)
		{
			dialog = create_background_property_dialog (window,
									g_strdup
									(property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border)") == 0)
		{
			dialog = create_border_properties_dialog (window,
								  g_strdup
								  (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border-width)") == 0)
		{
			dialog = create_border_width_properties_dialog (window,
									g_strdup
									(property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(border-width-single)") == 0)
		{
			dialog = create_border_width_single_properties_dialog (window,
										   g_strdup
										   (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(increment-reset)") == 0)
		{
			dialog = create_counter_increment_reset_dialog (window,
									g_strdup
									(property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(string)") == 0)
		{
			dialog = create_string_properties_dialog (window,
								  g_strdup
								  (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(uri)") == 0)
		{
			dialog = create_uri_value_dialog (window,
							  g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(attr)") == 0)
		{
			dialog = create_attr_value_dialog (window,
							   g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(counter)") == 0)
		{
			dialog = create_counter_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font)") == 0)
		{
			dialog = create_font_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font-family)") == 0)
		{
			dialog = create_font_family_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(font-size)") == 0)
		{
			dialog = create_font_size_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(frequency)") == 0)
		{
			dialog = create_frecuency_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(identifier)") == 0)
		{
			dialog = create_identifier_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(integer)") == 0)
		{
			dialog = create_integer_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(length)") == 0)
		{
			dialog = create_length_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(list-style)") == 0)
		{
			dialog = create_list_style_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(margin)") == 0)
		{
			dialog = create_margin_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}
		else if (strcmp (attribute, "(margin-width)") == 0)
		{
			dialog = create_margin_width_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}	
		else if (strcmp (attribute, "(number)") == 0)
		{
			dialog = create_number_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);
		}	
		else if (strcmp (attribute, "(padding)") == 0)
		{
			dialog = create_padding_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(outline)") == 0)
		{
			dialog = create_outline_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(pause)") == 0)
		{
			dialog = create_pause_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(percentage)") == 0)
		{
			dialog = create_percentage_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}			
		else if (strcmp (attribute, "(percentage-length)") == 0)
		{
			dialog = create_percentage_length_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}		
		else if (strcmp (attribute, "(play-during)") == 0)
		{
			dialog = create_play_during_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(quotes)") == 0)
		{
			dialog = create_quotes_property_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(shape)") == 0)
		{
			dialog = create_shape_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(size-length)") == 0)
		{
			dialog = create_size_length_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(text-shadow)") == 0)
		{
			dialog = create_text_shadow_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(voice-family)") == 0)
		{
			dialog = create_voice_family_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}	
		else if (strcmp (attribute, "(width-style-color)") == 0)
		{
			dialog = create_border_width_style_color_value_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}
		else if (strcmp (attribute, "(border-style)") == 0)
		{
			dialog = create_border_style_dialog (window,
								 g_strdup (property),  add_previous);
			gtk_widget_show (dialog);			
		}		
		else
		{ // dialog don't exists yet
			strdialog = g_strdup_printf(_("The dialog %s don't exist or is not finished yet"),attribute);
			document_write_error_to_program_output(tmpdoc,strdialog);
			if( add_previous ){
				line_to_insert =
				g_strconcat (property, ": ", attribute, ";",
					     	NULL);
				document_add_text_line (tmpdoc, line_to_insert);
				document_add_text (tmpdoc, "\t");
				g_free (line_to_insert);
			}else{
				document_add_text (tmpdoc, attribute);
			}
			g_free(strdialog);			
		}		
	}
	else
	{// no dialog to show, write the selected strings
		if (tmpdoc != NULL)
		{
			if( add_previous ){
				line_to_insert =
					g_strconcat (property, ":", attribute, ";",
					     NULL);
				document_add_text_line (tmpdoc, line_to_insert);
				document_add_text (tmpdoc, "\t");
				g_free (line_to_insert);
			}else{
				document_add_text (tmpdoc, attribute);
			}
		}
	}
}

// callback func called by all editors when a signals arises
void
on_editor_notification( GtkWidget* editor,gint scn, gpointer lscn, gpointer user_data)
{
	struct SCNotification *nt;
	ScintillaObject* sci;
	Tcssed_doc* tmpdoc;
	Tcssed_window* window;
	// used if lastchar == ':' in Autocompletion mode
	gint linestart;
	gchar* buffer;
	// ////////////
	gint line;
	gchar lastchar;
	gint position;
	gint margin;
	
	window = ( Tcssed_window*) user_data;
	tmpdoc = document_get_from_editor( window, editor );

	nt = lscn;
	switch (nt->nmhdr.code){
		case SCN_STYLENEEDED:	
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_STYLENEEDED\n", editor);
#endif
			break;
		case SCN_CHARADDED:	
			/* set undo and redo operations in a word basis not working yet */
			 lastchar = nt->ch;
			if( lastchar == ' '){
				document_end_undo_action(tmpdoc);
				document_start_undo_action(tmpdoc);	
			}

			if( tmpdoc->autocompletion ){
				// add close brace if open brace is added
				if( lastchar == '{' ){
					document_add_text(tmpdoc,"\n\t\n}\n");
					document_set_current_position(tmpdoc,
						(document_get_current_position(tmpdoc)-3) );
					document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
				}
				if( lastchar == ';'){
					document_add_text(tmpdoc,"\n\t");		
					document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
				}			
	
				if( lastchar == ':'){
					// get if in selector
					if(document_current_pos_is_into_selector(tmpdoc)){
#ifdef DEBUG
						g_print("into a selector\n");
#endif				
						position = document_get_current_position(tmpdoc);
						line = document_get_line_from_position(tmpdoc,position);
						linestart  = document_get_position_from_line(tmpdoc,line);
#ifdef DEBUG					
						g_print("Line %d start %d pos %d\n",line,linestart,position);
#endif					
						if( position > linestart){	
							buffer = g_malloc0(sizeof(gchar)*( position - linestart )+1);					
							document_get_text_line(tmpdoc,line,buffer);
#ifdef DEBUG							
							g_print("Line contains: \"%s\"\n", buffer);	
#endif							
							buffer[position-linestart-1] = '\0';
							g_strstrip( buffer );
#ifdef DEBUG						
							g_print("Cleaned: \"%s\"\n", buffer);
#endif							
							document_pop_submenu_from_keyword_in_treeview(tmpdoc,buffer);
						
							g_free(buffer);	
						}	
					}
				}	
			}
#ifdef DEBUG_CONTROL_NOTIFICATIONS 			
								g_print("received control %x notification SCN_CHARADDED\n", editor);
#endif
			break;
		case SCN_SAVEPOINTREACHED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_SAVEPOINTREACHED\n", editor);
#endif
			break;
		case SCN_SAVEPOINTLEFT:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_SAVEPOINTLEFT\n", editor);
#endif
			break;
		case SCN_MODIFYATTEMPTRO:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_MODIFYATTEMPTRO\n", editor);
#endif
			break;
		case SCN_KEY:
			if( (nt->ch==97) && (nt->modifiers==4) ){ // Alt + A fire autocompletion
				document_show_autocompletion_list(tmpdoc,0,(gchar*) window->keywords_list);
			}
#ifdef DEBUG
			g_print("Pressed char %d modifier %d\n", nt->ch,nt->modifiers);
#endif
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_KEY\n", editor);
#endif
			break;
		case SCN_DOUBLECLICK:   
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_DOUBLECLICK\n", editor);
#endif
			break;
		case SCN_UPDATEUI:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_UPDATEUI\n", editor);
#endif	
			break;
		case SCN_MODIFIED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_MODIFIED\n", editor);
#endif
			if( tmpdoc != NULL ){
				tmpdoc->modified = TRUE;
			}else{
				g_print("Program error, can set the dirty flag on document\nTo cssed it will seems that never changes !!!\n");
			}		
			break;
		case SCN_MACRORECORD:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 						
		g_print("received control %x notification SCN_MACRORECORD\n", editor);
#endif
			break;
		case SCN_MARGINCLICK:
			position = nt->position;
			margin = nt->margin;
		
			line = document_get_line_from_position( tmpdoc, position );

			if( document_is_marker_set_at_line( tmpdoc, line, 0 ) ){
				document_set_marker_at_line(tmpdoc,line,TRUE, 0 );
			}else{
				document_set_marker_at_line(tmpdoc,line,FALSE, 0 );
			}
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_MARGINCLICK\n", editor);
#endif
			break;
		case SCN_NEEDSHOWN:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_NEEDSHOWN\n", editor);
#endif
			break;
		case SCN_PAINTED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_PAINTED\n", editor);
#endif
			break;
		case SCN_USERLISTSELECTION:
			 // only one user list in
			switch( nt->wParam ){
				case 0:{
					g_print("Selected from list 0: %s\n",(gchar*) nt->text);
					}
				break;
				default:
						g_print("Unknow list selected\n");
				break;				
			}
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_USERLISTSELECTION\n", editor);
#endif
			break;
		case SCN_URIDROPPED:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_URIDROPPED\n", editor);
#endif
			break;
		case SCN_DWELLSTART:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_DWELLSTART\n", editor);
#endif
			break;
		case SCN_DWELLEND:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_STYLENEEDED\n", editor);
#endif
			break;
		case SCN_ZOOM:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_ZOOM\n", editor);
#endif
			break;
		case SCN_HOTSPOTCLICK:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_HOTSPOTCLICK\n", editor);
#endif
			break;
		case SCN_HOTSPOTDOUBLECLICK:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_STYLENEEDED\n", editor);
#endif
			break;
		case SCN_CALLTIPCLICK:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received control %x notification SCN_CALLTIPCLICK\n", editor);
#endif
			break;
		default:
#ifdef DEBUG_CONTROL_NOTIFICATIONS 
								g_print("received spurious control %x notification %d\n", editor, nt->nmhdr.code );
#endif
			break;
	}

	return;
}
/* scintilla object related functions */
ScintillaObject*
sci_get_from_notebook_page(GtkNotebook* notebook, gint page_number)
{
	ScintillaObject* sci;
	GtkWidget* editor;
	
	editor = gtk_notebook_get_nth_page (notebook, page_number);
	sci = SCINTILLA(editor);
	return sci;	
}

ScintillaObject*
sci_get_from_document( Tcssed_doc* doc )
{
	ScintillaObject * sci;
	sci = SCINTILLA( doc->editor );	
	return sci;
}

void
sci_set_css_style (ScintillaObject * sci)
{
	SSM (sci, SCI_STYLECLEARALL, 0, 0);
	SSM (sci, SCI_SETLEXER, SCLEX_CSS, 0);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_TAG, 0x000066);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_PSEUDOCLASS, 0xff00ff);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_OPERATOR, 0xff0033);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IDENTIFIER, 0x990000);
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_IDENTIFIER, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_UNKNOWN_IDENTIFIER , 0x0000ff);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_VALUE, 0x009900);
	SSM (sci, SCI_STYLESETBOLD, SCE_CSS_COMMENT, 1);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_COMMENT, 0x999999);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_ID, 0x009999);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_IMPORTANT, 0x6666ff);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DIRECTIVE, 0x660066);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_DOUBLESTRING, 0x660033);
	SSM (sci, SCI_STYLESETFORE, SCE_CSS_SINGLESTRING, 0x660033);
	SSM (sci, SCI_SETTABWIDTH, 4, 0);
	SSM (sci, SCI_SETREADONLY, 0, 0);
	SSM (sci, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
	SSM (sci, SCI_SETMARGINWIDTHN, 0, 24 );
	SSM (sci, SCI_SETMARGINTYPEN, 1, SC_MARGIN_SYMBOL);
	SSM (sci, SCI_SETMARGINWIDTHN, 1, 16 );
	SSM (sci, SCI_SETMARGINSENSITIVEN,1, TRUE);
	SSM (sci, SCI_MARKERDEFINE,0, SC_MARK_ROUNDRECT); // marker 0 for bookmarks	
	SSM (sci,SCI_SETUNDOCOLLECTION,1,0);
	SSM (sci,SCI_SETMODEVENTMASK, SC_MODEVENTMASKALL ,0); 
	//SCI_SETMODEVENTMASK,SC_MODEVENTMASKALL,0); 
	// SC_PERFORMED_UNDO|SC_PERFORMED_REDO|SC_MOD_INSERTTEXT|SC_MOD_DELETETEXT|SC_PERFORMED_USER
	/* will be for folding
 	SSM (sci, SCI_SETMARGINTYPEN, 2, SC_MARGIN_SYMBOL);
	SSM (sci, SCI_SETMARGINWIDTHN, 2, 16 ); 
	*/ 
}

void
sci_set_keyworkds (ScintillaObject * sci, gchar* keywords)
{
	SSM (sci, SCI_SETKEYWORDS, 0, (sptr_t) keywords);
	SSM (sci, SCI_SETKEYWORDS, 1, (sptr_t) "important link hover visited first-letter first-line after before lang");
}
/* line numbers visibility */
void 
sci_set_line_numbers(ScintillaObject * sci, gboolean set )
{
	if( set ){
		SSM (sci, SCI_SETMARGINWIDTHN, 0, 24 );		
	}else{
		SSM (sci, SCI_SETMARGINWIDTHN, 0, 0 );
	}
}

gboolean
sci_get_line_numbers(ScintillaObject * sci)
{
	gint margin_width;
	
	margin_width = SSM(sci, SCI_GETMARGINWIDTHN, 0, 0);
	if( margin_width > 0 ) return TRUE;
	else return FALSE;
}
void
sci_set_visible_eols(ScintillaObject* sci, gboolean set )
{
	SSM(sci,SCI_SETVIEWEOL,set,0);
}

gboolean 
sci_get_visible_eols(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETVIEWEOL,0,0);
}
void
sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
{
	if(set){
		SSM(sci,SCI_SETVIEWWS,SCWS_VISIBLEALWAYS,0);
	}else{
		SSM(sci,SCI_SETVIEWWS,SCWS_INVISIBLE,0);
	}
}
gboolean 
sci_get_visible_white_spaces(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETVIEWWS,0,0);
}
void
sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
{
	if( set ){
		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
	}else{
		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
	}
}
gboolean 
sci_get_lines_wrapped(ScintillaObject* sci)
{
	return SSM( sci, SCI_GETWRAPMODE,0,0);
}
void 
sci_set_eol_mode( ScintillaObject* sci, gint eolmode)
{
	SSM( sci, SCI_SETEOLMODE, eolmode, 0);
}

gint
sci_get_eol_mode( ScintillaObject* sci)
{
	return SSM(sci,SCI_GETEOLMODE,0,0);
}

void 
sci_convert_eols( ScintillaObject* sci, gint eolmode)
{
	SSM( sci, SCI_CONVERTEOLS, eolmode,0);
}


void 
sci_add_text(ScintillaObject* sci, gchar* text)
{
	SSM( sci, SCI_ADDTEXT, strlen(text) , (sptr_t) text);
}

void
sci_undo( ScintillaObject* sci )
{
	if( SSM( sci, SCI_CANUNDO,0,0) ){
		SSM( sci, SCI_UNDO,0,0);		
	}else{ // change this to document function
		cssed_error_message("There is no action in the undo list.","Undo limit reached");
	}	
}

void
sci_redo( ScintillaObject* sci )
{
	if( SSM( sci, SCI_CANREDO,0,0) ){
		SSM( sci, SCI_REDO,0,0);
	}else{// change this to a document function
		cssed_error_message("There is no action in the redo list.","Redo limit reached");
	}
}

void 
sci_start_undo_action( ScintillaObject* sci )
{
	SSM( sci,SCI_BEGINUNDOACTION,0,0 );
}

void 
sci_end_undo_action( ScintillaObject* sci )
{
	SSM( sci, SCI_ENDUNDOACTION,0,0);
}

void 
sci_empty_undo_buffer( ScintillaObject* sci )
{
	SSM( sci, SCI_EMPTYUNDOBUFFER,0,0);
}

void 
sci_zoom_in( ScintillaObject* sci )
{
	SSM( sci, SCI_ZOOMIN,0,0);
}

void 
sci_zoom_out( ScintillaObject* sci )
{
	SSM( sci, SCI_ZOOMOUT,0,0);
}

void 
sci_zoom_off( ScintillaObject* sci )
{
	SSM( sci, SCI_SETZOOM,0,0);
}
void
sci_set_marker_at_line( ScintillaObject* sci, gint line_number, gboolean set, gint marker )
{
	
#ifdef DEBUG
	g_print("Set marker to %d line %d \n",set,line_number);
#endif
	
	if( set ){
		SSM( sci, SCI_MARKERADD,line_number, marker);
	}else{
		SSM( sci, SCI_MARKERDELETE,line_number, marker);
	}
}
gboolean 
sci_is_marker_set_at_line(ScintillaObject* sci, gint line, gint marker)
{
	gint state;
	
	state = SSM( sci, SCI_MARKERGET, line, marker );	
	return(!(state & (1 << marker)));
}

gboolean
sci_marker_next(ScintillaObject* sci, gint line, gint marker_mask)
{
	gint marker_line;

	marker_line = SSM(sci, SCI_MARKERNEXT, line, marker_mask);
#ifdef DEBUG	
	g_print( "Marker next: line %d next marker %d\n",line,marker_line);
#endif
	if( marker_line != -1 ){
		SSM(sci,SCI_GOTOLINE,marker_line,0);
		return TRUE;
	}else{
		return FALSE;
	}
}

gboolean
sci_marker_prev(ScintillaObject* sci, gint line, gint marker_mask)
{
	gint marker_line;
	
	marker_line = SSM(sci, SCI_MARKERPREVIOUS, line, marker_mask);	
#ifdef DEBUG		
	g_print( "Marker previous: line %d marker %d\n",line,marker_line);
#endif	
	if( marker_line != -1 ){
		SSM(sci,SCI_GOTOLINE,marker_line,0);
		return TRUE;
	}else{
		return FALSE;
	}
}

gint 
sci_get_line_from_position(ScintillaObject* sci, gint position )
{
	return SSM(sci, SCI_LINEFROMPOSITION, position, 0);
}

gint 
sci_get_position_from_line(ScintillaObject* sci, gint line )
{
	return SSM(sci, SCI_POSITIONFROMLINE, line, 0);
}
 
gint 
sci_get_current_position(ScintillaObject* sci )
{
	return SSM(sci, SCI_GETCURRENTPOS, 0, 0);
}

void 
sci_set_current_position(ScintillaObject* sci, gint position )
{
	SSM(sci, SCI_GOTOPOS, position, 0);
}

void 
sci_set_current_line(ScintillaObject* sci, gint line )
{
	SSM(sci, SCI_GOTOLINE, line, 0);
}
void 
sci_cut(ScintillaObject* sci)
{
	SSM(sci, SCI_CUT, 0, 0);
}

void 
sci_copy(ScintillaObject* sci)
{
	SSM(sci, SCI_COPY, 0, 0);
}
void 
sci_paste(ScintillaObject* sci)
{
	SSM(sci, SCI_PASTE, 0, 0);
}
gint
sci_get_selection_start(ScintillaObject* sci)
{
	return SSM(sci, SCI_GETSELECTIONSTART,0,0);
}

gint
sci_get_selection_end(ScintillaObject* sci)
{
	return SSM(sci, SCI_GETSELECTIONEND,0,0);
}

void 
sci_replace_sel(ScintillaObject* sci, gchar* text)
{
	SSM(sci, SCI_REPLACESEL,0, (sptr_t) text);
}

gint
sci_get_length(ScintillaObject* sci)
{
	return SSM(sci,SCI_GETLENGTH,0,0);
}
gint 
sci_line_length(ScintillaObject* sci,gint line)
{
	return SSM(sci,SCI_LINELENGTH, line,0);
}
void // will not be null terminated
sci_get_line(ScintillaObject* sci, gint line, gchar* text)
{
	SSM(sci,SCI_GETLINE,line, (sptr_t) text);
}
void // the last char will be null terminated
sci_get_text(ScintillaObject* sci, gint len, gchar* text)
{
	SSM( sci, SCI_GETTEXT, len,(sptr_t) text );
}
void 
sci_get_xy_from_position(ScintillaObject* sci,gint pos, gint* x, gint* y)
{
	*x = SSM(sci, SCI_POINTXFROMPOSITION,0, (int) pos);
 	*y = SSM(sci, SCI_POINTYFROMPOSITION,0, (int) pos);
}
