/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

// uncomment to add a terminal tab to the footer notebook
//#define WITH_TERMINAL

#include "cssedwindow.h"
#include "document.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "xmlparser.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

enum
{
	COL_CSS_VER = 0,
	COL_ATTR,
	COL_VALUE,
	NUM_COLS
};

void set_css_properties_treeview (Tcssed_window * window);

GtkWidget *
create_window (Tcssed_window * windowobj)
{
	GtkWidget *window;
	GtkWidget *vbox_main;
	GtkWidget *alignment1;
	GtkWidget *menubar_main;
	GtkWidget *menuitem_file;
	GtkWidget *menuitem_file_menu;
	GtkWidget *menu_nuevo;
	GtkWidget *menu_abrir;
	GtkWidget *menu_guardar;
	GtkWidget *menu_guardar_como;
	GtkWidget *separatormenuitem1;
	GtkWidget *menu_salir;
	GtkWidget *menuitem_editar;
	GtkWidget *menuitem_editar_menu;
	GtkWidget *menu_cortar;
	GtkWidget *menu_copiar;
	GtkWidget *menu_pegar;
	GtkWidget *menu_borrar;
	////////////////////// dinamic ///
	GtkWidget *menuitem_ver;
	GtkWidget *menu_ver_line_numbers;
	GtkWidget *menuitem_ver_menu;
	GtkWidget *menu_ver_line_endings;
	GtkWidget *menu_ver_white_spaces;
	GtkWidget *menu_ver_lines_wrapped;
	GtkWidget *menu_item_autocompletion;
	GtkWidget *menuitem_eol_mode;
	GtkWidget *menuitem_eol_mode_menu;
	GtkWidget *menuitem_eol_mode_cr;
	GtkWidget *menuitem_eol_mode_lf;
	GtkWidget *menuitem_eol_mode_crlf;
	//////////////////////////////////
	GtkWidget *menuitem_ayuda;
	GtkWidget *menuitem_ayuda_menu;
	GtkWidget *menu_about;
	GtkWidget *toolbar_main;
	GtkWidget *button_abrir;
	GtkWidget *button_nuevo;
	GtkWidget *button_cerrar;
	GtkWidget *button_buscar;
	GtkWidget *button_reemplazar;
	GtkWidget *button_cortar;
	GtkWidget *button_copiar;
	GtkWidget *button_pegar;
	GtkWidget *button_next;
	GtkWidget *button_prev;
	GtkWidget *button_undo;
	GtkWidget *button_redo;
	GtkWidget *button_color;
	GtkWidget *button_guardar;
	GtkWidget *button_guardarcomo;
	//////// zoom //////////////
	GtkWidget *button_zoom_in;
	GtkWidget *button_zoom_out;
	GtkWidget *button_zoom_off;
	////////////////////////////
	GtkWidget *menuitem_test;
	GtkWidget *menuitem_test_menu;
	GtkWidget *menu_test_scan_selector;
	GtkWidget *menu_test_doc_info;
	GtkWidget *menu_test_autocomp;	
	//////////////////////////////////
	GtkWidget *vpaned_main;
	GtkWidget *hpaned_main;
	GtkWidget *notebook_docs;
	GtkWidget *editor;
	GtkWidget *label_notebook_blank;
	GtkWidget *vpaned_css;
	GtkWidget *scrolledwindow_cssc;
	GtkWidget *treeview_css_current;
	GtkWidget *scrolledwindow_cssp;
	GtkWidget *treeview_css_properties;
	GtkTreePath* css_prop_path;
	GtkWidget *notebook_footer;
	////////////////////////
	//GtkWidget *output_text;
	GtkWidget *output_list;
	//////////////////////
	GtkWidget *scrolledwindow_output_text;
	GtkWidget *scrolledwindow_scratch_text;
	GtkWidget *scratch_text;
	GtkWidget *scratch_label;
#ifdef WITH_TERMINAL
	GtkWidget *term_window;
	GtkWidget *label_terminal;
	GtkWidget *scrolled_window_terminal;
	GdkColor terminal_color;
#endif
	GtkWidget *label_notebook_footer;
	GtkAccelGroup *accel_group;
	// set up items to the output list
	GtkListStore *output_store;
	GtkTreeViewColumn *output_column;
	GtkCellRenderer *output_render;
	// set up items for the current selector list
	GtkListStore *current_css_store;
	GtkTreeViewColumn *current_css_property_column;
	GtkTreeViewColumn *current_css_value_column;
	GtkCellRenderer *current_css_render_properties;
	GtkCellRenderer *current_css_render_values;
	// icon stuff
	const gchar* icon_name = "cssed-icon.png";
	GdkPixbuf* icon_pixbuf;
	Tcssed_dyn_menus *menus;

	accel_group = gtk_accel_group_new ();

	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), "cssed");
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	//gtk_window_maximize (GTK_WINDOW (window));
	icon_pixbuf =  create_pixbuf(icon_name);
	if( icon_pixbuf != NULL ){
		gtk_window_set_icon  ( GTK_WINDOW (window), icon_pixbuf);
	}
	
	vbox_main = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox_main);
	gtk_container_add (GTK_CONTAINER (window), vbox_main);

	alignment1 = gtk_alignment_new (0.5, 0.5, 1, 1);
	gtk_widget_show (alignment1);
	gtk_box_pack_start (GTK_BOX (vbox_main), alignment1, FALSE, FALSE, 0);

	menubar_main = gtk_menu_bar_new ();
	gtk_widget_show (menubar_main);
	gtk_container_add (GTK_CONTAINER (alignment1), menubar_main);

	menuitem_file = gtk_menu_item_new_with_mnemonic (_("_File"));
	gtk_widget_show (menuitem_file);
	gtk_container_add (GTK_CONTAINER (menubar_main), menuitem_file);

	menuitem_file_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_file),
				   menuitem_file_menu);

	menu_nuevo =
		gtk_image_menu_item_new_from_stock ("gtk-new", accel_group);
	gtk_widget_show (menu_nuevo);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu), menu_nuevo);

	menu_abrir =
		gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
	gtk_widget_show (menu_abrir);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu), menu_abrir);

	menu_guardar =
		gtk_image_menu_item_new_from_stock ("gtk-save", accel_group);
	gtk_widget_show (menu_guardar);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu), menu_guardar);

	menu_guardar_como =
		gtk_image_menu_item_new_from_stock ("gtk-save-as",
						    accel_group);
	gtk_widget_show (menu_guardar_como);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu),
			   menu_guardar_como);

	separatormenuitem1 = gtk_menu_item_new ();
	gtk_widget_show (separatormenuitem1);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu),
			   separatormenuitem1);
	gtk_widget_set_sensitive (separatormenuitem1, FALSE);

	menu_salir =
		gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
	gtk_widget_show (menu_salir);
	gtk_container_add (GTK_CONTAINER (menuitem_file_menu), menu_salir);

	menuitem_editar = gtk_menu_item_new_with_mnemonic (_("_Edit"));
	gtk_widget_show (menuitem_editar);
	gtk_container_add (GTK_CONTAINER (menubar_main), menuitem_editar);

	menuitem_editar_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_editar),
				   menuitem_editar_menu);

	menu_cortar =
		gtk_image_menu_item_new_from_stock ("gtk-cut", accel_group);
	gtk_widget_show (menu_cortar);
	gtk_container_add (GTK_CONTAINER (menuitem_editar_menu), menu_cortar);

	menu_copiar =
		gtk_image_menu_item_new_from_stock ("gtk-copy", accel_group);
	gtk_widget_show (menu_copiar);
	gtk_container_add (GTK_CONTAINER (menuitem_editar_menu), menu_copiar);

	menu_pegar =
		gtk_image_menu_item_new_from_stock ("gtk-paste", accel_group);
	gtk_widget_show (menu_pegar);
	gtk_container_add (GTK_CONTAINER (menuitem_editar_menu), menu_pegar);

	menu_borrar =
		gtk_image_menu_item_new_from_stock ("gtk-delete",
						    accel_group);
	gtk_widget_show (menu_borrar);
	gtk_container_add (GTK_CONTAINER (menuitem_editar_menu), menu_borrar);

	///////////////// dinamc menu /////////////////////////////////////////
	menuitem_ver = gtk_menu_item_new_with_mnemonic (_("_Document"));
	gtk_widget_show (menuitem_ver);
	gtk_container_add (GTK_CONTAINER (menubar_main), menuitem_ver);

	menuitem_ver_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_ver),
				   menuitem_ver_menu);

	menu_ver_line_numbers =
		gtk_check_menu_item_new_with_mnemonic (_("View line numbers"));
	gtk_widget_show (menu_ver_line_numbers);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menu_ver_line_numbers);

	menu_ver_line_endings =
		gtk_check_menu_item_new_with_mnemonic (_("View line endings"));
	gtk_widget_show (menu_ver_line_endings);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menu_ver_line_endings);

	menu_ver_white_spaces =
		gtk_check_menu_item_new_with_mnemonic (_("View white spaces"));
	gtk_widget_show (menu_ver_white_spaces);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menu_ver_white_spaces);

	menu_ver_lines_wrapped =
		gtk_check_menu_item_new_with_mnemonic (_("View lines wrapped"));
	gtk_widget_show (menu_ver_lines_wrapped);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menu_ver_lines_wrapped);
	
	menu_item_autocompletion =
		gtk_check_menu_item_new_with_mnemonic (_("Set autocompletion"));
	gtk_widget_show (menu_item_autocompletion);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menu_item_autocompletion);		
	// ~~~~~~~~~~~~~~~~~~~~~~~~~ eols ~~~~~~~~~~~~~~~~~~~~
	menuitem_eol_mode = gtk_menu_item_new_with_mnemonic (_("Set EOL mode"));
	gtk_widget_show (menuitem_eol_mode);
	gtk_container_add (GTK_CONTAINER (menuitem_ver_menu),
			   menuitem_eol_mode);

	menuitem_eol_mode_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_eol_mode),
				   menuitem_eol_mode_menu);

	menuitem_eol_mode_cr =
		gtk_menu_item_new_with_mnemonic (_("Carriage return (CR)"));
	gtk_widget_show (menuitem_eol_mode_cr);
	gtk_container_add (GTK_CONTAINER (menuitem_eol_mode_menu),
			   menuitem_eol_mode_cr);

	menuitem_eol_mode_lf =
		gtk_menu_item_new_with_mnemonic (_("Line feed (LF)"));
	gtk_widget_show (menuitem_eol_mode_lf);
	gtk_container_add (GTK_CONTAINER (menuitem_eol_mode_menu),
			   menuitem_eol_mode_lf);

	menuitem_eol_mode_crlf =
		gtk_menu_item_new_with_mnemonic (_
						 ("Carriage return & Line feed(CR/LF)"));
	gtk_widget_show (menuitem_eol_mode_crlf);
	gtk_container_add (GTK_CONTAINER (menuitem_eol_mode_menu),
			   menuitem_eol_mode_crlf);

	/////////////// dynamic menus end /////////////////////////////////////
	menuitem_ayuda = gtk_menu_item_new_with_mnemonic (_("Help"));
	gtk_widget_show (menuitem_ayuda);
	gtk_container_add (GTK_CONTAINER (menubar_main), menuitem_ayuda);

	menuitem_ayuda_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_ayuda),
				   menuitem_ayuda_menu);

	menu_about = gtk_menu_item_new_with_mnemonic (_("_About"));
	gtk_widget_show (menu_about);
	gtk_container_add (GTK_CONTAINER (menuitem_ayuda_menu), menu_about);
	//////////////////////////// testing here //////////////////////////////
	menuitem_test = gtk_menu_item_new_with_mnemonic (_("Test"));
	gtk_widget_show (menuitem_test);
	gtk_container_add (GTK_CONTAINER (menubar_main), menuitem_test);

	menuitem_test_menu = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem_test),
				   menuitem_test_menu);

	menu_test_scan_selector =
		gtk_menu_item_new_with_mnemonic (_("Scan selector"));
	gtk_widget_show (menu_test_scan_selector);
	gtk_container_add (GTK_CONTAINER (menuitem_test_menu),
			   menu_test_scan_selector);


	menu_test_doc_info = gtk_menu_item_new_with_mnemonic (_("Doc Info"));
	gtk_widget_show (menu_test_doc_info);
	gtk_container_add (GTK_CONTAINER (menuitem_test_menu),
			   menu_test_doc_info);
	   
	menu_test_autocomp  = gtk_menu_item_new_with_mnemonic (_("Autocompletion"));
	gtk_widget_show (menu_test_autocomp);
	gtk_container_add (GTK_CONTAINER (menuitem_test_menu),
			   menu_test_autocomp);

	///////////////////////// toolbar //////////////////////////////////////
	toolbar_main = gtk_toolbar_new ();
	gtk_widget_show (toolbar_main);
	gtk_box_pack_start (GTK_BOX (vbox_main), toolbar_main, FALSE, FALSE,
			    0);
	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar_main), GTK_TOOLBAR_ICONS);

	button_abrir =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-open", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_abrir);

	button_nuevo =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-new", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_nuevo);

	button_cerrar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-close", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_cerrar);

	button_guardar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-save", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_guardar);

	button_guardarcomo =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-save-as", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_guardarcomo);

	button_buscar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-find", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_buscar);

	button_reemplazar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-find-and-replace", NULL, NULL,
					  NULL, NULL, -1);
	gtk_widget_show (button_reemplazar);

	button_undo =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-undo", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_undo);

	button_redo =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-redo", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_redo);


	button_cortar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-cut", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_cortar);

	button_copiar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-copy", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_copiar);

	button_pegar =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-paste", NULL, NULL, NULL, NULL,
					  -1);
	gtk_widget_show (button_pegar);

	button_color =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-select-color", NULL, NULL,
					  NULL, NULL, -1);
	gtk_widget_show (button_color);

	// next prev for bookmarks
	button_prev =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-go-back", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_guardarcomo);

	button_next =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-go-forward", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_guardarcomo);
	// adding zoom in toolbar //////////////////////////////////////////////////
	button_zoom_in =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-zoom-in", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_zoom_in);

	button_zoom_out =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-zoom-out", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_zoom_out);

	button_zoom_off =
		gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar_main),
					  "gtk-zoom-100", NULL, NULL, NULL,
					  NULL, -1);
	gtk_widget_show (button_zoom_off);

	// /////////////////////////////////////////////////////////////////////////

	vpaned_main = gtk_vpaned_new ();
	gtk_widget_show (vpaned_main);
	gtk_box_pack_start (GTK_BOX (vbox_main), vpaned_main, TRUE, TRUE, 0);
	gtk_paned_set_position (GTK_PANED (vpaned_main), 300);

	hpaned_main = gtk_hpaned_new ();
	gtk_widget_show (hpaned_main);
	gtk_paned_pack1 (GTK_PANED (vpaned_main), hpaned_main, TRUE, TRUE);
	gtk_paned_set_position (GTK_PANED (hpaned_main), 450);
	//gtk_container_set_resize_mode (GTK_CONTAINER(hpaned_main),GTK_RESIZE_IMMEDIATE);
	//////////////// notebook_docs ///////////////////////////////////////
	notebook_docs = gtk_notebook_new ();
	gtk_widget_show (notebook_docs);
	gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook_docs), TRUE);
	gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook_docs));
	gtk_paned_pack1 (GTK_PANED (hpaned_main), notebook_docs, FALSE, TRUE);

	///////////// set win object setting before creating any doc ///////////
	windowobj->notebook_docs = notebook_docs;
	windowobj->notebook_utils = notebook_footer;
	menus = windowobj->dynamic_menus;

	menus->menu_item_line_numbers = menu_ver_line_numbers;
	menus->menu_item_endings = menu_ver_line_endings;
	menus->menu_item_white_spaces = menu_ver_white_spaces;
	menus->menu_item_lines_wrapped = menu_ver_lines_wrapped;
	menus->menu_item_eol_mode_cr = menuitem_eol_mode_cr;
	menus->menu_item_eol_mode_lf = menuitem_eol_mode_lf;
	menus->menu_item_eol_mode_crlf = menuitem_eol_mode_crlf;
	menus->menu_item_autocompletion = menu_item_autocompletion;
	////////////////////////////////////////////////////////////////////////
	// we need to fill the window->keyword list before creating an editor, 
	// if we don't do that the initial editor will have
	// no keywords to style so first of all we create and fill
	// the tree, will be show later.
	treeview_css_properties = gtk_tree_view_new ();
	// this func will fill the treeview
	// we must fill the window->treeview_css_properties before 
	// trying to fill the tree
	windowobj->tree_view_css_properties = treeview_css_properties;
	set_css_properties_treeview (windowobj);
	// the same can be applied to the currenct CSS properties treeview
	treeview_css_current = gtk_tree_view_new ();
	windowobj->tree_view_css_current = treeview_css_current;
	// also to the program output list
	output_store = gtk_list_store_new (1, G_TYPE_STRING);
	windowobj->output_store = output_store;
	// and also for the current css selector list 
	current_css_store =
		gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
	windowobj->css_store = current_css_store;

	// now we can create and stylised editor
	editor = create_initial_blank_editor (windowobj);

	gtk_widget_show (editor);
	gtk_container_add (GTK_CONTAINER (notebook_docs), editor);
	GTK_WIDGET_SET_FLAGS (editor, GTK_CAN_FOCUS);
	GTK_WIDGET_SET_FLAGS (editor, GTK_CAN_DEFAULT);

	label_notebook_blank = gtk_label_new (_("Untitled"));
	gtk_widget_show (label_notebook_blank);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_docs),
				    gtk_notebook_get_nth_page (GTK_NOTEBOOK
							       (notebook_docs),
							       0),
				    label_notebook_blank);
	gtk_label_set_justify (GTK_LABEL (label_notebook_blank),
			       GTK_JUSTIFY_LEFT);

	vpaned_css = gtk_vpaned_new ();
	gtk_widget_show (vpaned_css);
	gtk_paned_pack2 (GTK_PANED (hpaned_main), vpaned_css, TRUE, TRUE);
	gtk_paned_set_position (GTK_PANED (vpaned_css), 150);

	scrolledwindow_cssc = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_cssc);
	gtk_paned_pack1 (GTK_PANED (vpaned_css), scrolledwindow_cssc, FALSE,
			 TRUE);

	// Current selector list is added here to a container
	gtk_widget_show (treeview_css_current);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_cssc),
			   treeview_css_current);
	// current css selector list 
	current_css_render_properties = gtk_cell_renderer_text_new ();
	current_css_render_values = gtk_cell_renderer_text_new ();
	current_css_property_column =
		gtk_tree_view_column_new_with_attributes ("Property",
							  current_css_render_properties,
							  "text", 0, NULL);
	current_css_value_column =
		gtk_tree_view_column_new_with_attributes ("Value",
							  current_css_render_values,
							  "text", 1, NULL);
	gtk_tree_view_insert_column (GTK_TREE_VIEW (treeview_css_current),
				     current_css_property_column, 0);
	gtk_tree_view_insert_column (GTK_TREE_VIEW (treeview_css_current),
				     current_css_value_column, 1);
	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview_css_current),
				 GTK_TREE_MODEL (current_css_store));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW
					   (treeview_css_current), TRUE);

	scrolledwindow_cssp = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_cssp);
	gtk_paned_pack2 (GTK_PANED (vpaned_css), scrolledwindow_cssp, TRUE,
			 TRUE);
	gtk_widget_set_size_request (scrolledwindow_cssp, 100, 100);

	// HERE THE TREE VIEW IS ADDED to a container
	gtk_widget_show (treeview_css_properties);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_cssp),
			   treeview_css_properties);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview_css_properties),
				      TRUE);
	/* expand the tree on path "0" */
	css_prop_path = gtk_tree_path_new_first();
	gtk_tree_view_expand_row (GTK_TREE_VIEW (treeview_css_properties),
                              css_prop_path, FALSE);
	// Footer notebook
	notebook_footer = gtk_notebook_new ();
	gtk_widget_show (notebook_footer);
	gtk_paned_pack2 (GTK_PANED (vpaned_main), notebook_footer, TRUE,
			 TRUE);

	scrolledwindow_output_text = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_output_text);
	// output list
	output_list = gtk_tree_view_new ();
	gtk_widget_show (output_list);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_output_text),
			   output_list);
	gtk_container_add (GTK_CONTAINER (notebook_footer),
			   scrolledwindow_output_text);
	output_render = gtk_cell_renderer_text_new ();
	output_column =
		gtk_tree_view_column_new_with_attributes ("", output_render,
							  "text", 0, NULL);
	gtk_tree_view_insert_column (GTK_TREE_VIEW (output_list),
				     output_column, 0);
	gtk_tree_view_set_model (GTK_TREE_VIEW (output_list),
				 GTK_TREE_MODEL (output_store));
	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (output_list),
					   FALSE);

	scrolledwindow_scratch_text = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow_scratch_text);
	scratch_text = gtk_text_view_new ();
	gtk_text_view_set_editable (GTK_TEXT_VIEW (scratch_text), TRUE);
	gtk_widget_show (scratch_text);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_scratch_text),
			   scratch_text);

	scratch_label = gtk_label_new ("Scratch - pad");
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook_footer),
				  scrolledwindow_scratch_text, scratch_label);
	gtk_widget_queue_draw (GTK_WIDGET (notebook_footer));


	label_notebook_footer = gtk_label_new (_("Program output"));
	gtk_widget_show (label_notebook_footer);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook_footer),
				    gtk_notebook_get_nth_page (GTK_NOTEBOOK
							       (notebook_footer),
							       0),
				    label_notebook_footer);
	gtk_label_set_justify (GTK_LABEL (label_notebook_footer),
			       GTK_JUSTIFY_LEFT);

#ifdef WITH_TERMINAL
	scrolled_window_terminal = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolled_window_terminal);
	term_window = (GtkWidget *) vte_terminal_new ();
	vte_terminal_set_emulation (term_window, "xterm");
	vte_terminal_set_mouse_autohide (term_window, TRUE);
	/*
	 * gdk_color_parse ("Black", &terminal_color);
	 * vte_terminal_set_color_foreground (term_window, &terminal_color);
	 * gdk_color_parse ("White", &terminal_color);
	 * vte_terminal_set_color_background (term_window, &terminal_color);
	 */
	vte_terminal_set_default_colors (term_window);
	vte_terminal_set_cursor_blinks (term_window, TRUE);
	vte_terminal_set_scroll_on_output (term_window, TRUE);
	vte_terminal_set_scroll_on_keystroke (term_window, TRUE);
	vte_terminal_reset (term_window, TRUE, TRUE);
	vte_terminal_fork_command (term_window, "/bin/bash", NULL, NULL, "~/",
				   0, 0, 0);
	gtk_widget_show (term_window);
	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					       (scrolled_window_terminal),
					       term_window);
	label_terminal = gtk_label_new (_("Terminal"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook_footer),
				  scrolled_window_terminal, label_terminal);
#endif

			
	g_signal_connect ((gpointer) window, "destroy",
			  G_CALLBACK (on_window_destroy), NULL);
	g_signal_connect ((gpointer) menu_nuevo, "activate",
			  G_CALLBACK (on_menu_nuevo_activate), windowobj);
	g_signal_connect ((gpointer) menu_abrir, "activate",
			  G_CALLBACK (on_menu_abrir_activate), windowobj);
	g_signal_connect ((gpointer) menu_guardar, "activate",
			  G_CALLBACK (on_menu_guardar_activate), windowobj);
	g_signal_connect ((gpointer) menu_guardar_como, "activate",
			  G_CALLBACK (on_menu_guardar_como_activate),
			  windowobj);
	g_signal_connect ((gpointer) menu_salir, "activate",
			  G_CALLBACK (on_menu_salir_activate), windowobj);
	g_signal_connect ((gpointer) menu_cortar, "activate",
			  G_CALLBACK (on_menu_cortar_activate), windowobj);
	g_signal_connect ((gpointer) menu_copiar, "activate",
			  G_CALLBACK (on_menu_copiar_activate), windowobj);
	g_signal_connect ((gpointer) menu_pegar, "activate",
			  G_CALLBACK (on_menu_pegar_activate), windowobj);
	g_signal_connect ((gpointer) menu_borrar, "activate",
			  G_CALLBACK (on_menu_borrar_activate), windowobj);
	g_signal_connect ((gpointer) menu_about, "activate",
			  G_CALLBACK (on_menu_about_activate), NULL);
	// toolbar buttons
	g_signal_connect ((gpointer) button_abrir, "clicked",
			  G_CALLBACK (on_button_abrir_clicked), windowobj);
	g_signal_connect ((gpointer) button_nuevo, "clicked",
			  G_CALLBACK (on_button_nuevo_clicked), windowobj);
	g_signal_connect ((gpointer) button_cerrar, "clicked",
			  G_CALLBACK (on_button_cerrar_clicked), windowobj);
	g_signal_connect ((gpointer) button_buscar, "clicked",
			  G_CALLBACK (on_button_buscar_clicked), windowobj);
	g_signal_connect ((gpointer) button_reemplazar, "clicked",
			  G_CALLBACK (on_button_reemplazar_clicked),
			  windowobj);
	g_signal_connect ((gpointer) button_color, "clicked",
			  G_CALLBACK (on_button_color_clicked), windowobj);
	g_signal_connect ((gpointer) button_guardar, "clicked",
			  G_CALLBACK (on_button_guardar_clicked), windowobj);
	g_signal_connect ((gpointer) button_guardarcomo, "clicked",
			  G_CALLBACK (on_button_guardarcomo_clicked),
			  windowobj);

	g_signal_connect ((gpointer) button_copiar, "clicked",
			  G_CALLBACK (on_button_copiar_clicked), windowobj);
	g_signal_connect ((gpointer) button_pegar, "clicked",
			  G_CALLBACK (on_button_pegar_clicked), windowobj);
	g_signal_connect ((gpointer) button_cortar, "clicked",
			  G_CALLBACK (on_button_cortar_clicked), windowobj);

	g_signal_connect ((gpointer) button_next, "clicked",
			  G_CALLBACK (on_button_next_clicked), windowobj);
	g_signal_connect ((gpointer) button_prev, "clicked",
			  G_CALLBACK (on_button_prev_clicked), windowobj);

	g_signal_connect ((gpointer) button_undo, "clicked",
			  G_CALLBACK (on_button_undo_clicked), windowobj);
	g_signal_connect ((gpointer) button_redo, "clicked",
			  G_CALLBACK (on_button_redo_clicked), windowobj);
	/* zooming callbacks */
	g_signal_connect ((gpointer) button_zoom_in, "clicked",
			  G_CALLBACK (on_button_zoom_in_clicked), windowobj);
	g_signal_connect ((gpointer) button_zoom_out, "clicked",
			  G_CALLBACK (on_button_zoom_out_clicked), windowobj);
	g_signal_connect ((gpointer) button_zoom_off, "clicked",
			  G_CALLBACK (on_button_zoom_off_clicked), windowobj);
	/* notebook */
	g_signal_connect ((gpointer) notebook_docs, "switch-page",
			  G_CALLBACK (on_notebook_docs_switch_page),
			  windowobj);
	// main view menu, this will change child on a document basis reflecting the opened document state
	g_signal_connect ((gpointer) menuitem_ver, "activate",
			  G_CALLBACK (on_menu_view_activate), windowobj);
	// dynamic menus set the callback func used when document specific menu items are toggled
	g_signal_connect ((gpointer) menu_ver_line_numbers, "toggled",
			  G_CALLBACK (on_menu_line_numbers_toggled),
			  windowobj);
	g_signal_connect ((gpointer) menu_ver_line_endings, "toggled",
			  G_CALLBACK (on_menu_line_endings_toggled),
			  windowobj);
	g_signal_connect ((gpointer) menu_ver_lines_wrapped, "toggled",
			  G_CALLBACK (on_menu_line_wrapping_toggled),
			  windowobj);
	g_signal_connect ((gpointer) menu_ver_white_spaces, "toggled",
			  G_CALLBACK (on_menu_ver_white_spaces_toggled),
			  windowobj);

	g_signal_connect ((gpointer) menu_item_autocompletion, "toggled",
			  G_CALLBACK (on_menu_item_autocompletion_toggled),
			  windowobj);	  
/*
	g_signal_connect ((gpointer) menu_item_autocompletion, "activate",
			  G_CALLBACK (on_menu_item_autocompletion_activate),
			  windowobj);		
*/			  
	g_signal_connect ((gpointer) menuitem_eol_mode_cr, "activate",
			  G_CALLBACK (on_menu_eol_mode_cr_activate),
			  windowobj);
	g_signal_connect ((gpointer) menuitem_eol_mode_lf, "activate",
			  G_CALLBACK (on_menu_eol_mode_lf_activate),
			  windowobj);
	g_signal_connect ((gpointer) menuitem_eol_mode_crlf, "activate",
			  G_CALLBACK (on_menu_eol_mode_crlf_activate),
			  windowobj);
	// selector scan menu, it's a test right now
	g_signal_connect ((gpointer) menu_test_scan_selector, "activate",
			  G_CALLBACK (on_menu_scan_selector_activate),
			  windowobj);
	// doc info not in lookup_widget list
	g_signal_connect ((gpointer) menu_test_doc_info, "activate",
			  G_CALLBACK (on_menu_doc_info_activate), windowobj);

	// autocompletion,  not in lookup_widget list
	g_signal_connect ((gpointer) menu_test_autocomp, "activate",
			  G_CALLBACK (on_menu_autocompletion_activate), windowobj);	

	// treeview callbacks 
	g_signal_connect ((gpointer) treeview_css_properties, "row-activated",
			  G_CALLBACK
			  (on_treeview_css_properties_row_activated),
			  windowobj);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (window, window, "window");
	GLADE_HOOKUP_OBJECT (window, vbox_main, "vbox_main");
	GLADE_HOOKUP_OBJECT (window, alignment1, "alignment1");
	GLADE_HOOKUP_OBJECT (window, menubar_main, "menubar_main");
	GLADE_HOOKUP_OBJECT (window, menuitem_file, "menuitem_file");
	GLADE_HOOKUP_OBJECT (window, menuitem_file_menu,
			     "menuitem_file_menu");
	GLADE_HOOKUP_OBJECT (window, menu_nuevo, "menu_nuevo");
	GLADE_HOOKUP_OBJECT (window, menu_abrir, "menu_abrir");
	GLADE_HOOKUP_OBJECT (window, menu_guardar, "menu_guardar");
	GLADE_HOOKUP_OBJECT (window, menu_guardar_como, "menu_guardar_como");
	GLADE_HOOKUP_OBJECT (window, separatormenuitem1,
			     "separatormenuitem1");
	GLADE_HOOKUP_OBJECT (window, menu_salir, "menu_salir");
	GLADE_HOOKUP_OBJECT (window, menuitem_editar, "menuitem_editar");
	GLADE_HOOKUP_OBJECT (window, menuitem_editar_menu,
			     "menuitem_editar_menu");
	GLADE_HOOKUP_OBJECT (window, menu_cortar, "menu_cortar");
	GLADE_HOOKUP_OBJECT (window, menu_copiar, "menu_copiar");
	GLADE_HOOKUP_OBJECT (window, menu_pegar, "menu_pegar");
	GLADE_HOOKUP_OBJECT (window, menu_borrar, "menu_borrar");
	GLADE_HOOKUP_OBJECT (window, menuitem_ver, "menuitem_ver");
	GLADE_HOOKUP_OBJECT (window, menuitem_ayuda, "menuitem_ayuda");
	GLADE_HOOKUP_OBJECT (window, menuitem_ayuda_menu,
			     "menuitem_ayuda_menu");
	GLADE_HOOKUP_OBJECT (window, menu_about, "menu_about");
	GLADE_HOOKUP_OBJECT (window, toolbar_main, "toolbar_main");
	GLADE_HOOKUP_OBJECT (window, button_abrir, "button_abrir");
	GLADE_HOOKUP_OBJECT (window, button_nuevo, "button_nuevo");
	GLADE_HOOKUP_OBJECT (window, button_cerrar, "button_cerrar");
	GLADE_HOOKUP_OBJECT (window, button_buscar, "button_buscar");
	GLADE_HOOKUP_OBJECT (window, button_reemplazar, "button_reemplazar");
	GLADE_HOOKUP_OBJECT (window, button_copiar, "button_copiar");
	GLADE_HOOKUP_OBJECT (window, button_pegar, "button_pegar");
	GLADE_HOOKUP_OBJECT (window, button_color, "button_color");
	GLADE_HOOKUP_OBJECT (window, button_guardar, "button_guardar");
	GLADE_HOOKUP_OBJECT (window, button_guardarcomo,
			     "button_guardarcomo");
	GLADE_HOOKUP_OBJECT (window, vpaned_main, "vpaned_main");
	GLADE_HOOKUP_OBJECT (window, hpaned_main, "hpaned_main");
	GLADE_HOOKUP_OBJECT (window, notebook_docs, "notebook_docs");
	GLADE_HOOKUP_OBJECT (window, editor, "editor");
	GLADE_HOOKUP_OBJECT (window, label_notebook_blank,
			     "label_notebook_blank");
	GLADE_HOOKUP_OBJECT (window, vpaned_css, "vpaned_css");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_cssc,
			     "scrolledwindow_cssc");
	GLADE_HOOKUP_OBJECT (window, treeview_css_current,
			     "treeview_css_current");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_cssp,
			     "scrolledwindow_cssp");
	GLADE_HOOKUP_OBJECT (window, treeview_css_properties,
			     "treeview_css_properties");
	GLADE_HOOKUP_OBJECT (window, notebook_footer, "notebook_footer");
	GLADE_HOOKUP_OBJECT (window, output_list, "output_list");
	GLADE_HOOKUP_OBJECT (window, label_notebook_footer,
			     "label_notebook_footer");
	GLADE_HOOKUP_OBJECT (window, scrolledwindow_scratch_text,
			     "scrolledwindow_scratch_text");
	GLADE_HOOKUP_OBJECT (window, notebook_footer, "notebook_footer");
	GLADE_HOOKUP_OBJECT (window, scratch_text, "scratch_text");
	GLADE_HOOKUP_OBJECT (window, scratch_label, "scratch_label");

	gtk_widget_grab_focus (editor);
	gtk_widget_grab_default (editor);
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

	return window;
}

/* 	This file selection related functions will be merged on one 
	FIXME - This is just a unused template the used ones are below

GtkWidget *
create_fileselection (void)
{
	GtkWidget *fileselection;
	GtkWidget *fileselection_ok_button;
	GtkWidget *fileselection_cancel_button;

	fileselection = gtk_file_selection_new (_("Seleccione Archivo"));
	gtk_container_set_border_width (GTK_CONTAINER (fileselection), 10);
	gtk_window_set_position (GTK_WINDOW (fileselection),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (fileselection), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (fileselection), TRUE);

	fileselection_ok_button =
		GTK_FILE_SELECTION (fileselection)->ok_button;
	gtk_widget_show (fileselection_ok_button);
	GTK_WIDGET_SET_FLAGS (fileselection_ok_button, GTK_CAN_DEFAULT);

	fileselection_cancel_button =
		GTK_FILE_SELECTION (fileselection)->cancel_button;
	gtk_widget_show (fileselection_cancel_button);
	GTK_WIDGET_SET_FLAGS (fileselection_cancel_button, GTK_CAN_DEFAULT);

	g_signal_connect_swapped ((gpointer) fileselection, "close",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (fileselection));
				  
	g_signal_connect_swapped ((gpointer) fileselection, "destroy",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (fileselection));
	g_signal_connect_swapped ((gpointer) fileselection_cancel_button,
				  "clicked", G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (fileselection));

	// Store pointers to all widgets, for use by lookup_widget(). 
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection, fileselection,
				    "fileselection");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection, fileselection_ok_button,
				    "fileselection_ok_button");
	GLADE_HOOKUP_OBJECT_NO_REF (fileselection,
				    fileselection_cancel_button,
				    "fileselection_cancel_button");

	return fileselection;
}
*/
GtkWidget *
create_save_fileselection (Tcssed_window * window)
{
	GtkWidget *save_fileselection;
	GtkWidget *save_fileselection_ok_button;
	GtkWidget *save_fileselection_cancel_button;
	Tcssed_fileselector_data *data;

	save_fileselection = gtk_file_selection_new (_("Seleccione Archivo"));
	gtk_container_set_border_width (GTK_CONTAINER (save_fileselection),
					10);
	gtk_window_set_position (GTK_WINDOW (save_fileselection),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (save_fileselection), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (save_fileselection),
					    TRUE);

	save_fileselection_ok_button =
		GTK_FILE_SELECTION (save_fileselection)->ok_button;
	gtk_widget_show (save_fileselection_ok_button);
	GTK_WIDGET_SET_FLAGS (save_fileselection_ok_button, GTK_CAN_DEFAULT);

	save_fileselection_cancel_button =
		GTK_FILE_SELECTION (save_fileselection)->cancel_button;
	gtk_widget_show (save_fileselection_cancel_button);
	GTK_WIDGET_SET_FLAGS (save_fileselection_cancel_button,
			      GTK_CAN_DEFAULT);

	data = g_malloc (sizeof (Tcssed_fileselector_data));
	data->notebook = window->notebook_docs;
	data->fileselector = save_fileselection;
	data->main_window = window;

	g_signal_connect ((gpointer) save_fileselection, "close",
				  G_CALLBACK (file_selector_close),
				  data);
	g_signal_connect ((gpointer) save_fileselection_cancel_button,
				  "clicked", G_CALLBACK (file_selector_cancel_button_clicked),
				 data);
	g_signal_connect ((gpointer) save_fileselection_ok_button, "clicked",
		G_CALLBACK (on_save_fileselection_ok_button_clicked), (gpointer) data);	// must be freed by callers
	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (save_fileselection, save_fileselection,
				    "save_fileselection");
	GLADE_HOOKUP_OBJECT_NO_REF (save_fileselection,
				    save_fileselection_ok_button,
				    "save_fileselection_ok_button");
	GLADE_HOOKUP_OBJECT_NO_REF (save_fileselection,
				    save_fileselection_cancel_button,
				    "save_fileselection_cancel_button");

	return save_fileselection;
}

GtkWidget *
create_save_as_fileselection (Tcssed_window * window)
{
	GtkWidget *save_as_fileselection;
	GtkWidget *save_as_fileselection_ok_button;
	GtkWidget *save_as_fileselection_cancel_button;
	Tcssed_fileselector_data *data;

	save_as_fileselection =
		gtk_file_selection_new (_("Seleccione Archivo"));
	gtk_container_set_border_width (GTK_CONTAINER (save_as_fileselection),
					10);
	gtk_window_set_position (GTK_WINDOW (save_as_fileselection),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (save_as_fileselection), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW
					    (save_as_fileselection), TRUE);

	save_as_fileselection_ok_button =
		GTK_FILE_SELECTION (save_as_fileselection)->ok_button;
	gtk_widget_show (save_as_fileselection_ok_button);
	GTK_WIDGET_SET_FLAGS (save_as_fileselection_ok_button,
			      GTK_CAN_DEFAULT);

	save_as_fileselection_cancel_button =
		GTK_FILE_SELECTION (save_as_fileselection)->cancel_button;
	gtk_widget_show (save_as_fileselection_cancel_button);
	GTK_WIDGET_SET_FLAGS (save_as_fileselection_cancel_button,
			      GTK_CAN_DEFAULT);

	data = g_malloc (sizeof (Tcssed_fileselector_data));
	data->notebook = window->notebook_docs;
	data->fileselector = save_as_fileselection;
	data->main_window = window;

	g_signal_connect ((gpointer) save_as_fileselection, "close",
				  G_CALLBACK (file_selector_close),
				  data);
	g_signal_connect ((gpointer)
				  save_as_fileselection_cancel_button,
				  "clicked", G_CALLBACK (file_selector_cancel_button_clicked),
				  data);
	g_signal_connect ((gpointer) save_as_fileselection_ok_button, "clicked", 
		G_CALLBACK (on_save_as_fileselection_ok_button_clicked), 
		(gpointer) data);	// must be freed by callers
		
	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (save_as_fileselection,
				    save_as_fileselection,
				    "save_as_fileselection");
	GLADE_HOOKUP_OBJECT_NO_REF (save_as_fileselection,
				    save_as_fileselection_ok_button,
				    "save_as_fileselection_ok_button");
	GLADE_HOOKUP_OBJECT_NO_REF (save_as_fileselection,
				    save_as_fileselection_cancel_button,
				    "save_as_fileselection_cancel_button");

	return save_as_fileselection;
}

GtkWidget *
create_open_fileselection (Tcssed_window * window)
{
	GtkWidget *open_fileselection;
	GtkWidget *open_fileselection_ok_button;
	GtkWidget *open_fileselection_cancel_button;
	Tcssed_fileselector_data *data;

	open_fileselection = gtk_file_selection_new (_("Select file"));
	gtk_container_set_border_width (GTK_CONTAINER (open_fileselection),
					10);
	gtk_window_set_position (GTK_WINDOW (open_fileselection),
				 GTK_WIN_POS_CENTER);

	gtk_window_set_modal (GTK_WINDOW (open_fileselection), TRUE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (open_fileselection),
					    TRUE);

	open_fileselection_ok_button =
		GTK_FILE_SELECTION (open_fileselection)->ok_button;
	gtk_widget_show (open_fileselection_ok_button);
	GTK_WIDGET_SET_FLAGS (open_fileselection_ok_button, GTK_CAN_DEFAULT);

	open_fileselection_cancel_button =
		GTK_FILE_SELECTION (open_fileselection)->cancel_button;
	gtk_widget_show (open_fileselection_cancel_button);
	GTK_WIDGET_SET_FLAGS (open_fileselection_cancel_button,
			      GTK_CAN_DEFAULT);

	data = g_malloc (sizeof (Tcssed_fileselector_data));
	data->notebook = window->notebook_docs;
	data->fileselector = open_fileselection;
	data->main_window = window;
	// FIXME memory leaks on close and cancelbutton cliked signals
	g_signal_connect ((gpointer) open_fileselection, "close",
				  G_CALLBACK (file_selector_close),
				  data);
	g_signal_connect((gpointer) open_fileselection_cancel_button,
				  "clicked", G_CALLBACK (file_selector_cancel_button_clicked),
				  data);
	g_signal_connect ((gpointer) open_fileselection_ok_button, "clicked", 
		G_CALLBACK (create_and_attach_new_named_doc_from_fileselector_cb), 
		(gpointer) data);	// must be freed by callers

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (open_fileselection, open_fileselection,
				    "open_fileselection");
	GLADE_HOOKUP_OBJECT_NO_REF (open_fileselection,
				    open_fileselection_ok_button,
				    "open_fileselection_ok_button");
	GLADE_HOOKUP_OBJECT_NO_REF (open_fileselection,
				    open_fileselection_cancel_button,
				    "open_fileselection_cancel_button");

	return open_fileselection;
}


GtkWidget *
create_error_dialog (void)
{
	GtkWidget *error_dialog;
	GtkWidget *dialog_vbox;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *image;
	GtkWidget *label_msg;
	GtkWidget *label_detail;
	GtkWidget *dialog_action_area;
	GtkWidget *errordialog_okbutton;

	error_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (error_dialog), "");
	gtk_window_set_position (GTK_WINDOW (error_dialog),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (error_dialog), TRUE);

	dialog_vbox = GTK_DIALOG (error_dialog)->vbox;
	gtk_widget_show (dialog_vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), hbox, TRUE, TRUE, 0);

	image = gtk_image_new_from_stock ("gtk-dialog-warning",
					  GTK_ICON_SIZE_DIALOG);
	gtk_widget_show (image);
	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 35);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 10);
	
	label_msg = gtk_label_new (NULL);
	gtk_widget_show (label_msg);
	gtk_box_pack_start (GTK_BOX (vbox), label_msg, TRUE, TRUE, 10);
	
	label_detail = gtk_label_new (NULL);
	gtk_widget_show (label_detail);
	gtk_box_pack_start (GTK_BOX (vbox), label_detail, TRUE, TRUE, 10);	
	
	dialog_action_area = GTK_DIALOG (error_dialog)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area),
				   GTK_BUTTONBOX_END);

	errordialog_okbutton = gtk_button_new_from_stock ("gtk-ok");
	gtk_widget_show (errordialog_okbutton);
	gtk_dialog_add_action_widget (GTK_DIALOG (error_dialog),
				      errordialog_okbutton, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS (errordialog_okbutton, GTK_CAN_DEFAULT);

	g_signal_connect_swapped ((gpointer) error_dialog, "destroy",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (error_dialog));
	g_signal_connect_swapped ((gpointer) error_dialog, "close",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (error_dialog));
	g_signal_connect_swapped ((gpointer) errordialog_okbutton, "clicked",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (error_dialog));

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (error_dialog, error_dialog,
				    "error_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (error_dialog, dialog_vbox, "dialog_vbox");
	GLADE_HOOKUP_OBJECT (error_dialog, hbox, "hbox");
	GLADE_HOOKUP_OBJECT (error_dialog, label_detail, "label_detail");
	GLADE_HOOKUP_OBJECT (error_dialog, image, "image");
	GLADE_HOOKUP_OBJECT (error_dialog, label_msg, "label_msg");
	GLADE_HOOKUP_OBJECT_NO_REF (error_dialog, dialog_action_area,
				    "dialog_action_area");
	GLADE_HOOKUP_OBJECT (error_dialog, errordialog_okbutton,
			     "errordialog_okbutton");

	return error_dialog;
}

GtkWidget *
create_about (void)
{
	const gchar *authors[] = {
		"Iago Rubio <iago.rubio@hispalinux.es>",
		NULL
	};
	const gchar *documenters[] =
		{ "Iago Rubio <iago.rubio@hispalinux.es>", NULL };
	/* TRANSLATORS: Replace this string with your names, one name per line. */
	gchar *translators[] = {  _("English translation by Iago Rubio <iago.rubio@hispalinux.es>") };
	const gchar* about_pixbuf_name = "cssed-about.png";
	GtkWidget *about;
	GdkPixbuf* about_pixbuf = create_pixbuf(about_pixbuf_name);

#ifdef DEBUG	
	if( iconpixbuf == NULL ){
		g_print("create_about(): Can't load icon pixbuf\n");
	}else{
		g_print("create_about(): Icon pixbuf loaded at address %p\n", iconpixbuf);
	}
#endif
	about = gnome_about_new ("cssed", VERSION,
				 _("Iago Rubio"),
				 _("Tiny css editor"),
				 &authors, 
				 &documenters, 
				 translators, 
				 about_pixbuf );
	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (about, about, "about");

	return about;
}

GtkWidget *
create_search_dialog (Tcssed_window * window)
{
	GtkWidget *search_dialog;
	GtkWidget *dialog_vbox;
	GtkWidget *vbox;
	GtkWidget *hbox4;
	GtkWidget *label;
	GtkWidget *entry_text_to_find;
	GtkWidget *table;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GSList *radiobutton_perl_regex_group = NULL;
	GtkWidget *radiobutton_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	GtkWidget *dialog_action_area;
	GtkWidget *dialog_search_help_button;
	GtkWidget *search_dialog_find_button;
	GtkWidget *search_dialog_cancel_button;

	search_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (search_dialog), _("Search"));
	gtk_window_set_position (GTK_WINDOW (search_dialog),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (search_dialog), TRUE);

	// not modal dialog
	gtk_window_set_modal (GTK_WINDOW (search_dialog), FALSE);

	dialog_vbox = GTK_DIALOG (search_dialog)->vbox;
	gtk_widget_show (dialog_vbox);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_box_pack_start (GTK_BOX (dialog_vbox), vbox, TRUE, TRUE, 0);

	hbox4 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox4);
	gtk_box_pack_start (GTK_BOX (vbox), hbox4, TRUE, TRUE, 0);

	label = gtk_label_new (_("Text to find"));
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox4), label, FALSE, FALSE, 5);
	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

	entry_text_to_find = gtk_entry_new ();
	gtk_widget_show (entry_text_to_find);
	gtk_box_pack_start (GTK_BOX (hbox4), entry_text_to_find, TRUE, TRUE,
			    3);

	table = gtk_table_new (3, 2, TRUE);
	gtk_widget_show (table);
	gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table), 5);

	checkbutton_regex =
		gtk_check_button_new_with_mnemonic (_("Regular expression"));
	gtk_widget_show (checkbutton_regex);
	gtk_table_attach (GTK_TABLE (table), checkbutton_regex, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	radiobutton_perl_regex =
		gtk_radio_button_new_with_mnemonic (NULL, _("Perl regexp"));
	gtk_widget_show (radiobutton_perl_regex);
	gtk_table_attach (GTK_TABLE (table), radiobutton_perl_regex, 1, 2, 1,
			  2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_perl_regex),
				    radiobutton_perl_regex_group);
	radiobutton_perl_regex_group =
		gtk_radio_button_get_group (GTK_RADIO_BUTTON
					    (radiobutton_perl_regex));

	radiobutton_regex =
		gtk_radio_button_new_with_mnemonic (NULL, _("Posix regexp"));
	gtk_widget_show (radiobutton_regex);
	gtk_table_attach (GTK_TABLE (table), radiobutton_regex, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_regex),
				    radiobutton_perl_regex_group);
	radiobutton_perl_regex_group =
		gtk_radio_button_get_group (GTK_RADIO_BUTTON
					    (radiobutton_regex));

	checkbutton_match_case =
		gtk_check_button_new_with_mnemonic (_("Match case"));
	gtk_widget_show (checkbutton_match_case);
	gtk_table_attach (GTK_TABLE (table), checkbutton_match_case, 0, 1, 0,
			  1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	checkbutton_whole_word =
		gtk_check_button_new_with_mnemonic (_("Whole world only"));
	gtk_widget_show (checkbutton_whole_word);
	gtk_table_attach (GTK_TABLE (table), checkbutton_whole_word, 0, 1, 1,
			  2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	checkbutton_word_start =
		gtk_check_button_new_with_mnemonic (_("Word start"));
	gtk_widget_show (checkbutton_word_start);
	gtk_table_attach (GTK_TABLE (table), checkbutton_word_start, 0, 1, 2,
			  3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	dialog_action_area = GTK_DIALOG (search_dialog)->action_area;
	gtk_widget_show (dialog_action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area),
				   GTK_BUTTONBOX_END);

	dialog_search_help_button = gtk_button_new_from_stock ("gtk-help");
	gtk_widget_show (dialog_search_help_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (search_dialog),
				      dialog_search_help_button,
				      GTK_RESPONSE_HELP);
	GTK_WIDGET_SET_FLAGS (dialog_search_help_button, GTK_CAN_DEFAULT);

	search_dialog_find_button = gtk_button_new_from_stock ("gtk-find");
	gtk_widget_show (search_dialog_find_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (search_dialog),
				      search_dialog_find_button,
				      GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (search_dialog_find_button, GTK_CAN_DEFAULT);

	search_dialog_cancel_button = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (search_dialog_cancel_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (search_dialog),
				      search_dialog_cancel_button,
				      GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (search_dialog_cancel_button, GTK_CAN_DEFAULT);

	g_signal_connect_swapped ((gpointer) search_dialog, "close",
				  G_CALLBACK (gtk_widget_destroy),
				  G_OBJECT (search_dialog));
	g_signal_connect_swapped ((gpointer) search_dialog, "destroy",
				  G_CALLBACK (gtk_widget_destroy),
				  G_OBJECT (search_dialog));
	g_signal_connect ((gpointer) dialog_search_help_button, "clicked",
			  G_CALLBACK (on_dialog_search_help_button_clicked),
			  NULL);
	g_signal_connect_swapped ((gpointer) search_dialog_cancel_button,
				  "clicked", G_CALLBACK (gtk_widget_destroy),
				  G_OBJECT (search_dialog));
	g_signal_connect ((gpointer) search_dialog_find_button, "clicked",
			  G_CALLBACK (on_search_dialog_find_button_clicked),
			  window);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (search_dialog, search_dialog,
				    "search_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (search_dialog, dialog_vbox,
				    "dialog_vbox");
	GLADE_HOOKUP_OBJECT (search_dialog, vbox, "vbox");
	GLADE_HOOKUP_OBJECT (search_dialog, hbox4, "hbox4");
	GLADE_HOOKUP_OBJECT (search_dialog, label, "label");
	GLADE_HOOKUP_OBJECT (search_dialog, entry_text_to_find,
			     "entry_text_to_find");
	GLADE_HOOKUP_OBJECT (search_dialog, table, "table");
	GLADE_HOOKUP_OBJECT (search_dialog, checkbutton_regex,
			     "checkbutton_regex");
	GLADE_HOOKUP_OBJECT (search_dialog, radiobutton_perl_regex,
			     "radiobutton_perl_regex");
	GLADE_HOOKUP_OBJECT (search_dialog, radiobutton_regex,
			     "radiobutton_regex");
	GLADE_HOOKUP_OBJECT (search_dialog, checkbutton_match_case,
			     "checkbutton_match_case");
	GLADE_HOOKUP_OBJECT (search_dialog, checkbutton_whole_word,
			     "checkbutton_whole_word");
	GLADE_HOOKUP_OBJECT (search_dialog, checkbutton_word_start,
			     "checkbutton_word_start");
	GLADE_HOOKUP_OBJECT_NO_REF (search_dialog, dialog_action_area,
				    "dialog_action_area");
	GLADE_HOOKUP_OBJECT (search_dialog, dialog_search_help_button,
			     "dialog_search_help_button");
	GLADE_HOOKUP_OBJECT (search_dialog, search_dialog_find_button,
			     "search_dialog_find_button");
	GLADE_HOOKUP_OBJECT (search_dialog, search_dialog_cancel_button,
			     "search_dialog_cancel_button");

	return search_dialog;
}

GtkWidget *
create_replace_dialog (Tcssed_window * window)
{
	GtkWidget *replace_dialog;
	GtkWidget *vbox_main;
	GtkWidget *vbox;
	GtkWidget *hbox5;
	GtkWidget *label4;
	GtkWidget *entry_find;
	GtkWidget *hbox6;
	GtkWidget *label5;
	GtkWidget *entry_replace;
	GtkWidget *table2;
	GtkWidget *checkbutton_regex;
	GtkWidget *radiobutton_perl_regex;
	GSList *radiobutton_perl_regex_group = NULL;
	GtkWidget *radiobutton_posix_regex;
	GtkWidget *checkbutton_match_case;
	GtkWidget *checkbutton_whole_word;
	GtkWidget *checkbutton_word_start;
	GtkWidget *hbuttonbox;
	GtkWidget *dialog_replace_help_button;
	GtkWidget *dialog_replace_button;
	GtkWidget *dialog_replace_apply_button;
	GtkWidget *dialog_replace_close_button;

	replace_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (replace_dialog), _("Replace"));
	gtk_window_set_position (GTK_WINDOW (replace_dialog),
				 GTK_WIN_POS_CENTER);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (replace_dialog),
					    TRUE);

	vbox_main = GTK_DIALOG (replace_dialog)->vbox;
	gtk_widget_show (vbox_main);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox);
	gtk_box_pack_start (GTK_BOX (vbox_main), vbox, TRUE, TRUE, 0);

	hbox5 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox5);
	gtk_box_pack_start (GTK_BOX (vbox), hbox5, TRUE, TRUE, 0);

	label4 = gtk_label_new (_("Search for   "));
	gtk_widget_show (label4);
	gtk_box_pack_start (GTK_BOX (hbox5), label4, FALSE, FALSE, 5);
	gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

	entry_find = gtk_entry_new ();
	gtk_widget_show (entry_find);
	gtk_box_pack_start (GTK_BOX (hbox5), entry_find, TRUE, TRUE, 3);

	hbox6 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox6);
	gtk_box_pack_start (GTK_BOX (vbox), hbox6, TRUE, TRUE, 0);

	label5 = gtk_label_new (_("Replace with"));
	gtk_widget_show (label5);
	gtk_box_pack_start (GTK_BOX (hbox6), label5, FALSE, FALSE, 5);
	gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_LEFT);

	entry_replace = gtk_entry_new ();
	gtk_widget_show (entry_replace);
	gtk_box_pack_start (GTK_BOX (hbox6), entry_replace, TRUE, TRUE, 3);

	table2 = gtk_table_new (3, 2, TRUE);
	gtk_widget_show (table2);
	gtk_box_pack_start (GTK_BOX (vbox), table2, FALSE, FALSE, 1);
	gtk_table_set_row_spacings (GTK_TABLE (table2), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table2), 5);

	checkbutton_regex =
		gtk_check_button_new_with_mnemonic (_("Regular expression"));
	gtk_widget_show (checkbutton_regex);
	gtk_table_attach (GTK_TABLE (table2), checkbutton_regex, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	radiobutton_perl_regex =
		gtk_radio_button_new_with_mnemonic (NULL, _("Perl regex"));
	gtk_widget_show (radiobutton_perl_regex);
	gtk_table_attach (GTK_TABLE (table2), radiobutton_perl_regex, 1, 2, 1,
			  2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_perl_regex),
				    radiobutton_perl_regex_group);
	radiobutton_perl_regex_group =
		gtk_radio_button_get_group (GTK_RADIO_BUTTON
					    (radiobutton_perl_regex));

	radiobutton_posix_regex =
		gtk_radio_button_new_with_mnemonic (NULL, _("Posix regex"));
	gtk_widget_show (radiobutton_posix_regex);
	gtk_table_attach (GTK_TABLE (table2), radiobutton_posix_regex, 1, 2,
			  2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON
				    (radiobutton_posix_regex),
				    radiobutton_perl_regex_group);
	radiobutton_perl_regex_group =
		gtk_radio_button_get_group (GTK_RADIO_BUTTON
					    (radiobutton_posix_regex));

	checkbutton_match_case =
		gtk_check_button_new_with_mnemonic (_("Match case"));
	gtk_widget_show (checkbutton_match_case);
	gtk_table_attach (GTK_TABLE (table2), checkbutton_match_case, 0, 1, 0,
			  1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	checkbutton_whole_word =
		gtk_check_button_new_with_mnemonic (_("Whole world only"));
	gtk_widget_show (checkbutton_whole_word);
	gtk_table_attach (GTK_TABLE (table2), checkbutton_whole_word, 0, 1, 1,
			  2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	checkbutton_word_start =
		gtk_check_button_new_with_mnemonic (_("Word start"));
	gtk_widget_show (checkbutton_word_start);
	gtk_table_attach (GTK_TABLE (table2), checkbutton_word_start, 0, 1, 2,
			  3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	hbuttonbox = GTK_DIALOG (replace_dialog)->action_area;
	gtk_widget_show (hbuttonbox);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox),
				   GTK_BUTTONBOX_END);

	dialog_replace_help_button = gtk_button_new_from_stock ("gtk-help");
	gtk_widget_show (dialog_replace_help_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (replace_dialog),
				      dialog_replace_help_button,
				      GTK_RESPONSE_HELP);
	GTK_WIDGET_SET_FLAGS (dialog_replace_help_button, GTK_CAN_DEFAULT);

	dialog_replace_button = gtk_button_new_from_stock ("gtk-find");
	gtk_widget_show (dialog_replace_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (replace_dialog),
				      dialog_replace_button, 0);
	GTK_WIDGET_SET_FLAGS (dialog_replace_button, GTK_CAN_DEFAULT);

	dialog_replace_apply_button = gtk_button_new_from_stock ("gtk-apply");
	gtk_widget_show (dialog_replace_apply_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (replace_dialog),
				      dialog_replace_apply_button,
				      GTK_RESPONSE_APPLY);
	GTK_WIDGET_SET_FLAGS (dialog_replace_apply_button, GTK_CAN_DEFAULT);

	dialog_replace_close_button = gtk_button_new_from_stock ("gtk-close");
	gtk_widget_show (dialog_replace_close_button);
	gtk_dialog_add_action_widget (GTK_DIALOG (replace_dialog),
				      dialog_replace_close_button,
				      GTK_RESPONSE_CLOSE);
	GTK_WIDGET_SET_FLAGS (dialog_replace_close_button, GTK_CAN_DEFAULT);

	g_signal_connect_swapped ((gpointer) replace_dialog, "close",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (replace_dialog));
	g_signal_connect_swapped ((gpointer) replace_dialog, "destroy",
				  G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (replace_dialog));
	g_signal_connect ((gpointer) dialog_replace_help_button, "clicked",
			  G_CALLBACK (on_dialog_replace_help_button_clicked),
			  NULL);

	g_signal_connect ((gpointer) dialog_replace_apply_button, "clicked",
			  G_CALLBACK (on_dialog_replace_apply_button_clicked),
			  window);
	g_signal_connect ((gpointer) dialog_replace_button, "clicked",
			  G_CALLBACK
			  (on_dialog_replace_search_button_clicked), window);


	g_signal_connect_swapped ((gpointer) dialog_replace_close_button,
				  "clicked", G_CALLBACK (gtk_widget_destroy),
				  GTK_OBJECT (replace_dialog));

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (replace_dialog, replace_dialog,
				    "replace_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (replace_dialog, vbox_main, "vbox_main");
	GLADE_HOOKUP_OBJECT (replace_dialog, vbox, "vbox");
	GLADE_HOOKUP_OBJECT (replace_dialog, hbox5, "hbox5");
	GLADE_HOOKUP_OBJECT (replace_dialog, label4, "label4");
	GLADE_HOOKUP_OBJECT (replace_dialog, entry_find, "entry_find");
	GLADE_HOOKUP_OBJECT (replace_dialog, hbox6, "hbox6");
	GLADE_HOOKUP_OBJECT (replace_dialog, label5, "label5");
	GLADE_HOOKUP_OBJECT (replace_dialog, entry_replace, "entry_replace");
	GLADE_HOOKUP_OBJECT (replace_dialog, table2, "table2");
	GLADE_HOOKUP_OBJECT (replace_dialog, checkbutton_regex,
			     "checkbutton_regex");
	GLADE_HOOKUP_OBJECT (replace_dialog, radiobutton_perl_regex,
			     "radiobutton_perl_regex");
	GLADE_HOOKUP_OBJECT (replace_dialog, radiobutton_posix_regex,
			     "radiobutton_posix_regex");
	GLADE_HOOKUP_OBJECT (replace_dialog, checkbutton_match_case,
			     "checkbutton_match_case");
	GLADE_HOOKUP_OBJECT (replace_dialog, checkbutton_whole_word,
			     "checkbutton_whole_word");
	GLADE_HOOKUP_OBJECT (replace_dialog, checkbutton_word_start,
			     "checkbutton_word_start");
	GLADE_HOOKUP_OBJECT_NO_REF (replace_dialog, hbuttonbox, "hbuttonbox");
	GLADE_HOOKUP_OBJECT (replace_dialog, dialog_replace_help_button,
			     "dialog_replace_help_button");
	GLADE_HOOKUP_OBJECT (replace_dialog, dialog_replace_button,
			     "dialog_replace_button");
	GLADE_HOOKUP_OBJECT (replace_dialog, dialog_replace_apply_button,
			     "dialog_replace_apply_button");
	GLADE_HOOKUP_OBJECT (replace_dialog, dialog_replace_close_button,
			     "dialog_replace_close_button");

	return replace_dialog;
}

void
set_css_properties_treeview (Tcssed_window * window)
{

	GtkTreeViewColumn *col;
	GtkCellRenderer *renderer;
	GtkTreeStore *model;
	GtkWidget *treeview;
	GtkTreeIter toplevel, child, third;
	Tcssed_tp_data data;

	treeview = window->tree_view_css_properties;

	/* --- Column #1 --- */
	col = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (col, "Version");
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (col, renderer, TRUE);
	gtk_tree_view_column_add_attribute (col, renderer, "text",
					    COL_CSS_VER);

	/* --- Column #2 --- */
	col = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (col, "Property");
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (col, renderer, TRUE);
	gtk_tree_view_column_add_attribute (col, renderer, "text", COL_ATTR);

	/* --- Column #3 --- */
	col = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title (col, "Value");
	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
	renderer = gtk_cell_renderer_text_new ();
	gtk_tree_view_column_pack_start (col, renderer, TRUE);
	gtk_tree_view_column_add_attribute (col, renderer, "text", COL_VALUE);

	model = gtk_tree_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING,
				    G_TYPE_STRING);

	// fill data struct values to pass to func
	data.first_col = &toplevel;
	data.second_col = &child;
	data.thrid_col = &third;
	data.store = model;
	data.window = window;
	// parse the xml and fill the values on the treeview
	parse_css_definition_and_fill_tree_view (&data);

	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
				 GTK_TREE_MODEL (model));
	g_object_unref (GTK_TREE_MODEL (model));	/* destroy model automatically with view */
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection
				     (GTK_TREE_VIEW (treeview)),
				     GTK_SELECTION_SINGLE);

}
